
#include "stdafx.h"

using namespace System;
using namespace System::Drawing;
using namespace System::Windows::Forms;
using namespace GeoFilterBank;

void GraphNode::CopyModuleToExecSettings()
{
	if (module)
	{
		if (exec_settings != nullptr) delete exec_settings;
		Byte* f = module->XML_Tree_Marshal();
		exec_settings = gcnew xml_node( f );
		delete [] f;
	}
	else if (exec_settings)
	{
		delete exec_settings;
		exec_settings = nullptr;
	}
}

void GraphNode::RestoreSavedNode(WorkflowManager^ p, String^ title, String^ filter_url, GraphNodeType gt)
{
	if (initialized) return;
	initialized = true;

	parentForm = p;
	mw = parentForm->GetParent();
	gnt = gt;

	if (gt != GraphNodeType::SPLITTER_MODULE && gt != GraphNodeType::NULLEXPORT_MODULE)
	{
		bool bs = false;
		for (int i = 0; i < filter_url->Length && !bs; i++) bs = (filter_url[i] == '\\');
		if (!bs) filter_url = Application::StartupPath + "\\" + filter_url;
	}

	// FILTER MODULE
	if (gt == GraphNodeType::FILTER_MODULE)
	{
		FilterName->Text = title;
		FilterName->Name = filter_url;

		array<Object^>^ args = gcnew array<Object^>(0);
		IntPtr^ ip;
		try{
			ip = (IntPtr^)mw->LoadDllMethod("FilterSpace.ImageFilter", "GetSupportedFormats", filter_url, args);
		} catch (Exception^) { return; }

		Byte* f = (Byte*)ip->ToPointer();
		module = gcnew xml_node( f );
		module->XmlSetChildValueByName("Run_Local", 1, false);
		delete [] f;

		int tmp = 0;
		module->XmlGetChildValueByName("InputImageCount", tmp);
		in_edges = tmp;

		tmp = 1;
		module->XmlGetChildValueByName("OutputImageCount", tmp);
		out_edges = tmp;

		CopyModuleToExecSettings();
	}
	// IMPORT MODULE
	else if (gt == GraphNodeType::IMPORT_MODULE)
	{
		FilterName->Text = title;
		FilterName->Name = filter_url;

		xml_node^ tmp;
		module = gcnew xml_node(nullptr, "", "", "");
		module->children.Add(tmp = gcnew xml_node(module, "OutputImage0", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Output", false);

		in_edges = 0;
		out_edges = 1;
	}
	// EXPORT MODULE
	else if (gt == GraphNodeType::EXPORT_MODULE)
	{
		FilterName->Text = title;
		FilterName->Name = filter_url;

		array<Object^>^ args = gcnew array<Object^>(0);
		String^ sFormat;
		try{
			sFormat = (String^)mw->LoadDllMethod("ImageSaver.ImageSaver", "GetExportFormatType", filter_url, args);
		} catch (Exception^) { return; }

		xml_node^ tmp;
		module = gcnew xml_node(nullptr, "", "", "");
		module->XmlSetChildValueByName("InputImageCount", 1);
		module->children.Add(tmp = gcnew xml_node(module, "InputImage0", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Input", false);

		if (sFormat == "RASTER")
			tmp->XmlSetChildValueByName("Width", 0, false);
		else if (sFormat == "VECTOR")
			tmp->XmlSetChildValueByName("BoundingBoxLeft", 0, false);

		CopyModuleToExecSettings();

		in_edges = 1;
		out_edges = 0;
	}
	// SPLITTER
	else if (gt == GraphNodeType::SPLITTER_MODULE)
	{
		FilterName->Text = "Splitter";
		FilterName->Name = "";

		xml_node^ tmp;
		module = gcnew xml_node(nullptr, "", "", "");
		module->XmlSetChildValueByName("InputImageCount", 1);
		module->XmlSetChildValueByName("OutputImageCount", 2);
		module->children.Add(tmp = gcnew xml_node(module, "InputImage0", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Input", false);
		tmp->XmlSetChildValueByName("ChannelsBitAllocation", "", false);
		module->children.Add(tmp = gcnew xml_node(module, "OutputImage0", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Input&&", false);
		module->children.Add(tmp = gcnew xml_node(module, "OutputImage1", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Input copy", false);

		CopyModuleToExecSettings();

		in_edges = 1;
		out_edges = 2;
	}
	// NULLEXPORT
	else if (gt == GraphNodeType::NULLEXPORT_MODULE)
	{
		FilterName->Text = "\"/dev/null\"";
		FilterName->Name = "";

		xml_node^ tmp;
		module = gcnew xml_node(nullptr, "", "", "");
		module->XmlSetChildValueByName("InputImageCount", 1);
		module->children.Add(tmp = gcnew xml_node(module, "InputImage0", "", ""));
		tmp->XmlSetChildValueByName("PinName", "Input", false);

		CopyModuleToExecSettings();

		in_edges = 1;
		out_edges = 0;
	}

	// --------------------


	//
	// EDGE CONTACTORS
	//
	edge_contactors = gcnew cli::array<System::Windows::Forms::PictureBox^>(in_edges + out_edges);
	edge_labels     = gcnew cli::array<System::Windows::Forms::Label^>(in_edges + out_edges);

	this->SuspendLayout();

	for (int i = 0; i < in_edges + out_edges; i++) 
	{
		this->edge_contactors[i] = (gcnew System::Windows::Forms::PictureBox());
		this->edge_labels[i] = (gcnew System::Windows::Forms::Label());
		(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->edge_contactors[i]))->BeginInit();
		this->Controls->Add(this->edge_contactors[i]);
		this->Controls->Add(this->edge_labels[i]);

		this->edge_contactors[i]->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::Edge_MouseDown);
		this->edge_contactors[i]->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::Edge_MouseMove);
		this->edge_contactors[i]->Name = "Edge" + i.ToString();
		this->edge_contactors[i]->Size = System::Drawing::Size(8, 8);
		this->edge_contactors[i]->BackColor = FreePinColor;
		this->edge_contactors[i]->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;

		this->edge_labels[i]->Enabled = false;
		this->edge_labels[i]->Font = (gcnew System::Drawing::Font(L"Arial", 8.00F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, static_cast<System::Byte>(238)));
		this->edge_labels[i]->Size = System::Drawing::Size((this->Width / 2) - edge_contactors[i]->Width - 4, 13);
		this->edge_labels[i]->ForeColor = System::Drawing::Color::SaddleBrown;
		this->edge_labels[i]->Name = L"";
	}

	for (int i = 0; i < in_edges; i++)
	{
		this->edge_contactors[i]->Location = System::Drawing::Point(0, 30 + i * 20);
		this->edge_contactors[i]->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::InEdge_MouseUp);
		this->ToolTips->SetToolTip(this->edge_contactors[i], L"Connect filters using these pins.\nPress the left mouse button over an input pin (at left side)\nand release the button over an output pin (at right side) of an other filter.\n\nDisconnect filters by right click on any pin used to connect them.");

		this->edge_labels[i]->Location = edge_contactors[i]->Location + System::Drawing::Size(edge_contactors[i]->Width + 4, - 3);
		this->edge_labels[i]->TextAlign = System::Drawing::ContentAlignment::TopLeft;
		if (module != nullptr)
		{
			xml_node^ xn;
			if (module->XmlGetChildByName(xn, "InputImage" + i.ToString()))
			{
				String^ s = "Input";
				xn->XmlGetChildValueByName("PinName", s);
				this->edge_labels[i]->Text = s;
			}
		}
	}
	for (int i = in_edges; i < in_edges + out_edges; i++)	
	{
		this->edge_contactors[i]->Location = System::Drawing::Point(this->Width - 9, 30 + (i - in_edges) * 20);
		this->edge_contactors[i]->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::OutEdge_MouseUp);
		this->ToolTips->SetToolTip(this->edge_contactors[i], L"Connect filters using these pins.\nPress the left mouse button over an output pin (at right side)\nand release the button over an input pin (at left side) of an other filter.\n\nDisconnect filters by right click on any pin used to connect them.");

		this->edge_labels[i]->Location = System::Drawing::Point(this->Width / 2, this->edge_contactors[i]->Top - 3);
		this->edge_labels[i]->TextAlign = System::Drawing::ContentAlignment::TopRight;
		if (module != nullptr)
		{
			xml_node^ xn;
			if (module->XmlGetChildByName(xn, "OutputImage" + (i - in_edges).ToString()))
			{
				String^ s = "Output";
				xn->XmlGetChildValueByName("PinName", s);
				this->edge_labels[i]->Text = s;
			}
		}
	}

	for (int i = 0; i < in_edges + out_edges; i++) (cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->edge_contactors[i]))->EndInit();
	this->ResumeLayout(false);
	this->PerformLayout();
}


bool GraphNode::InitializeNode(WorkflowManager^ p, String^ title, String^ filter_url, GraphNodeType gt)
{
	RestoreSavedNode(p, title, filter_url, gt);
	Settings_Click(nullptr, nullptr);

	if (module != nullptr)
	{
		if (parentForm->menuWorkflowShowXMLMetainfos->Checked)
		{
			XML_Editor^ xe = gcnew XML_Editor();
			xe->Text = "Initialize";
			xe->CreateXMLTree( module );
			xe->ShowDialog();
			delete xe;
		}
	}

	return !DestructOnCreate;
}

void GraphNode::RestoreSavedSettings(xml_node^ xn)
{
	if (gnt == GraphNodeType::FILTER_MODULE || gnt == GraphNodeType::IMPORT_MODULE) 
	{
		unsigned char* b = xn->XML_Tree_Marshal();
		module = gcnew xml_node(b);
		exec_settings = gcnew xml_node(b);
		delete [] b;
	}
}


bool GraphNode::IsMul(int num, int base)
{
	return ((num / base) * base == num);
}

bool GraphNode::IsPower(int num, int power)
{
	int i = 1;
	while (i < num) i = i * power;
	return (i == num);
}


bool GraphNode::IsComplete()
{
	for (int i = 0; i < in_edges + out_edges; i++) if (edge_contactors[i]->BackColor != LockedPinColor) return false;
	return true;
}

bool GraphNode::IsSettingValid()
{
	// GETTING INFORMATIONS
	int i;
	xml_node ^m, ^e;
	int **mod_params  = new int*[in_edges];
	int **exec_params = new int*[in_edges];

	for (i = 0; i < in_edges; i++) 
	{
		mod_params[i]  = new int[7];
		exec_params[i] = new int[7];

		module->XmlGetChildByName(m, "InputImage" + i.ToString());
		exec_settings->XmlGetChildByName(e, "InputImage" + i.ToString());

		GetOneImageSetting(m, mod_params[i][0], mod_params[i][1], mod_params[i][2], mod_params[i][3], mod_params[i][4], mod_params[i][5], mod_params[i][6]);
		GetOneImageSetting(e, exec_params[i][0], exec_params[i][1], exec_params[i][2], exec_params[i][3], exec_params[i][4], exec_params[i][5], exec_params[i][6]);
	}

	bool quit = false;
	// TEST COMPATIBILITY
	for (i = 0; i < in_edges && !quit; i++)
	{
		// WIDTH, HEIGHT, STRIDE
		for (int j = 0; j < 3 && !quit; j++)
		{
			// FIXED PARAMETRIC CHECKING
			if (mod_params[i][j] == 2 && !IsPower(exec_params[i][j], 2)) quit = true;
			if (mod_params[i][j] == 8 &&   !IsMul(exec_params[i][j], 8)) quit = true;

			// A NEGATIVE MOD PARAM MEANS, THAT ITS "ABSOLUTE VALUE - 1" INDEXING THE INPUT
			// THATS PARAM SHOULD EQUAL TO THE CURRENT PARAM (SELF-RELATIVE CHECK)
			if (mod_params[i][j] <  0 && exec_params[i][j] != exec_params[-mod_params[i][j] - 1][j]) quit = true;
		}

		// PACKED, INTERLEAVED, CHANNELCOUNT, PIXELSIZE
		for (int j = 3; j < 7 && !quit; j++)
		{
			if (mod_params[i][j] > 0 && exec_params[i][j] != mod_params[i][j]) quit = true;
			if (mod_params[i][j] < 0 && exec_params[i][j] != exec_params[-mod_params[i][j] - 1][j]) quit = true;
		}

		// NO VALIDATION OF BITSPERPIXEL COMPATIBILITY HERE YET (EVEN IF XML CONTAINS THE REQUIRED INFORMATION...)
	}

	// FREE UP
	for (i = 0; i < in_edges; i++)
	{
		delete [] mod_params[i];
		delete [] exec_params[i];
	}
	delete [] mod_params;
	delete [] exec_params;

	return true;
}


void GraphNode::GetOneImageSetting(xml_node^ fs, int& Width, int& Height, int& Stride, int& Packed, int& Interleaved, int& ChannelCount, int& PixelSize)
{
	fs->XmlGetChildValueByName("Width", Width);
	fs->XmlGetChildValueByName("Height", Height);
	fs->XmlGetChildValueByName("Stride", Stride);
	fs->XmlGetChildValueByName("Packed", Packed);
	fs->XmlGetChildValueByName("Interleaved", Interleaved);
	fs->XmlGetChildValueByName("ChannelCount", ChannelCount);
	fs->XmlGetChildValueByName("PixelSize", PixelSize);
}

int  GraphNode::IndexOfPointedInEdge(int x, int y)
{
	for (int i = 0; i < in_edges; i++) if (edge_contactors[i]->Bounds.Contains(x, y)) return i;
	return -1;
}

int  GraphNode::IndexOfPointedOutEdge(int x, int y)
{
	for (int i = in_edges; i < in_edges + out_edges; i++) if (edge_contactors[i]->Bounds.Contains(x, y)) return i;
	return -1;
}

void GraphNode::GetAbsolutePosOfEdge(int pin, int& x, int& y)
{
	if (pin >= 0 && pin < in_edges + out_edges)
	{
		x = this->Left + edge_contactors[pin]->Left + edge_contactors[pin]->Width / 2;
		y = this->Top + edge_contactors[pin]->Top + edge_contactors[pin]->Height / 2;
		if (y > this->Bottom) y = this->Bottom - 4;
	}
}


void GraphNode::ReleaseEdge(int pin)
{
	if (0 <= pin && pin < in_edges + out_edges) 
	{
		edge_contactors[pin]->BackColor = FreePinColor;
		this->BackColor = NormalBackColor;
	}

	if (gnt == GraphNodeType::SPLITTER_MODULE || gnt == GraphNodeType::NULLEXPORT_MODULE)
	{
		int cnt = 0;
		for (int i = 0; i < in_edges + out_edges; i++) if (edge_contactors[i]->BackColor == FreePinColor) cnt++;
		if (cnt == in_edges + out_edges) splittertype = "UNIVERSAL";
	}
}

bool GraphNode::LockEdge(int pin, String^ type)
{
	if (0 <= pin && pin < in_edges + out_edges) 
	{
		if (gnt == GraphNodeType::SPLITTER_MODULE || gnt == GraphNodeType::NULLEXPORT_MODULE)
		{
			if (splittertype != "UNIVERSAL" && type != splittertype) return false;
			splittertype = type;
		}

		if (edge_contactors[pin]->BackColor == FreePinColor)
		{
			if (GetPinType(pin) != type && type != "UNIVERSAL") return false;
			edge_contactors[pin]->BackColor = LockedPinColor;

			int cnt = 0;
			for (int i = 0; i < in_edges + out_edges; i++) if (edge_contactors[i]->BackColor == LockedPinColor) cnt++;
			if (cnt == in_edges + out_edges) this->BackColor = AllPinUsedBackColor;
			return true;
		}
	}
	return false;
}

String^ GraphNode::GetPinType(int pin)
{
	xml_node^ xt;

	if (gnt == GraphNodeType::SPLITTER_MODULE || gnt == GraphNodeType::NULLEXPORT_MODULE) return splittertype;

	if (gnt == GraphNodeType::UNDEFINED || pin < 0 || pin >= in_edges + out_edges) return "UNKNOWN (ERROR 0x1)";

	if (pin < in_edges)
	{
		if (!module->XmlGetChildByName(xt, "InputImage" + pin.ToString())) return "UNKNOWN (ERROR 0x2)";
	}
	else
	{
		if (!module->XmlGetChildByName(xt, "OutputImage" + (pin - in_edges).ToString())) return "UNKNOWN (ERROR 0x3)";
	}

	if (xt->XmlIsChildExist("Width"))           return "RASTER";
	if (xt->XmlIsChildExist("BoundingBoxLeft")) return "VECTOR";

	return "UNKNOWN (ERROR 0x4)";
}


void GraphNode::Edge_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	int j = -1;
	for (int i = 0; i < in_edges + out_edges && j == -1; i++) if (edge_contactors[i] == ((PictureBox^)(sender))) j = i;

	if (parentForm != nullptr)
	{
		parentForm->m_x = this->Left + ((PictureBox^)sender)->Left + ((PictureBox^)sender)->Width / 2;
		parentForm->m_y = this->Top + ((PictureBox^)sender)->Top + ((PictureBox^)sender)->Height / 2;

		if (e->Button == System::Windows::Forms::MouseButtons::Left)
		{
			parentForm->src = this;
			parentForm->pin = j;
			parentForm->edge_construct = true;
		}
		else if (e->Button == System::Windows::Forms::MouseButtons::Right)
		{
			parentForm->DeleteEdge(this, j);
		}
		parentForm->Refresh();
	}
}

void GraphNode::InEdge_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	if (parentForm != nullptr)
	{
		if (e->Button == System::Windows::Forms::MouseButtons::Left)
		{
			if (parentForm->edge_construct) 
			{
				int j = -1;
				for (int i = 0; i < in_edges && j == -1; i++) if (edge_contactors[i] == ((PictureBox^)(sender))) j = i;
				parentForm->pin = j;
				parentForm->CreateEdge(false);
			}
			parentForm->edge_construct = false;
		}
		else if (e->Button == System::Windows::Forms::MouseButtons::Right)
		{
			parentForm->edge_construct = false;
		}
		parentForm->Refresh();
	}
}

void GraphNode::OutEdge_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	if (parentForm != nullptr)
	{
		if (e->Button == System::Windows::Forms::MouseButtons::Left)
		{
			if (parentForm->edge_construct) parentForm->CreateEdge(true);
			parentForm->edge_construct = false;
		}
		else if (e->Button == System::Windows::Forms::MouseButtons::Right)
		{
			parentForm->edge_construct = false;
		}
		parentForm->Refresh();
	}
}

void GraphNode::Edge_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	if (parentForm != nullptr)
	{
		parentForm->x = this->Left + ((PictureBox^)sender)->Left + e->X;
		parentForm->y = this->Top + ((PictureBox^)sender)->Top + e->Y;
		parentForm->Refresh();
	}
}

// GraphNode Mouse Events

void GraphNode::GraphNode_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	moving = false;
}

void GraphNode::GraphNode_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	if (moving)
	{
		this->Location = System::Drawing::Point(this->Location.X + e->X - x, this->Location.Y + e->Y - y);
		if (parentForm != nullptr) parentForm->Refresh();
	}
}

void GraphNode::GraphNode_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	x = e->X;
	y = e->Y;
	moving = true;
}

void GraphNode::GraphNode_Resize(System::Object^  sender, System::EventArgs^  e) 
{
	if (this->Width < 146) this->Width = 146;
	if (this->Height < 58) this->Height = 58;
	Sizer->Location = System::Drawing::Point(this->Width - 18, this->Height - 18);

	FilterProgressBar->Location = System::Drawing::Point(FilterProgressBar->Left, this->Height - FilterProgressBar->Height - Sizer->Height);
	FilterProgressBar->Width = this->Width - 2 * FilterProgressBar->Left;
	SelfRemove->Location = System::Drawing::Point(Sizer->Left, FilterName->Bottom - 16);
	Settings->Location = System::Drawing::Point(Sizer->Left - 20, FilterName->Bottom - 16);
	for (int i = in_edges; i < in_edges + out_edges; i++)	this->edge_contactors[i]->Left = this->Width - 9;

	for (int i = 0; i < in_edges; i++)
	{
		this->edge_labels[i]->Size     = System::Drawing::Size((this->Width / 2) - 12, 13);
		this->edge_labels[i]->Location = System::Drawing::Point(12, 30 + i * 20 - 3);
	}
	for (int i = in_edges; i < in_edges + out_edges; i++)	
	{
		this->edge_labels[i]->Size     = System::Drawing::Size((this->Width / 2) - 12, 13);
		this->edge_labels[i]->Location = System::Drawing::Point(this->Width / 2, 30 + (i - in_edges) * 20 - 3);
	}
}

void GraphNode::Sizer_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	resizing = false;
}

void GraphNode::Sizer_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	if (resizing) 
	{
		int sx = Sizer->Left + e->X - x;
		int sy = Sizer->Top + e->Y - y;
		if (sx <= 128) sx = 128;
		if (sy <= 40) sy = 40;
		Sizer->Location = System::Drawing::Point(sx, sy);
		this->Size = System::Drawing::Size(sx + 18, sy + 18);
		if (parentForm != nullptr) parentForm->Refresh();
	}
}

void GraphNode::Sizer_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	x = e->X;
	y = e->Y;
	resizing = true;
}

void GraphNode::SelfRemove_Click(System::Object^  sender, System::EventArgs^  e)
{
	if (parentForm != nullptr) parentForm->DeleteNode(this);
}

void GraphNode::GraphNode_MouseDoubleClick(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
{
	Settings_Click(nullptr, nullptr);
}

void GraphNode::Settings_Click(System::Object^  sender, System::EventArgs^  e)
{
	if      (gnt == GraphNodeType::IMPORT_MODULE) DestructOnCreate = !SettingsImport();
	else if (gnt == GraphNodeType::EXPORT_MODULE) DestructOnCreate = !SettingsExport();
	else if (gnt == GraphNodeType::FILTER_MODULE) DestructOnCreate = !SettingsFilter();
}

bool GraphNode::SettingsImport()
{
	// Creating image filetypes filter
	int i, j;
	String^ filter = "";
	String^ tmp = "";
	array<__wchar_t>^ c = gcnew array<__wchar_t>(1);
	c[0] = ',';
	for (i = 0; i < mw->image_plugins.Count; ++i)
	{
		filter += mw->image_plugins[i].Key->ToUpper() + " " + mw->language_container->XmlGetChildStringValueByName("Files") + "|";
		array<String^>^ extensions = mw->image_plugins[i].Key->Split(c);

		for (j = 0; j < extensions->Length; ++j) 
		{
			filter += "*." + extensions[j];
			tmp += "*." + extensions[j] + ";";
			if (j != extensions->Length - 1)
			{
				filter += ";";
			}
		}
		filter += "|";

		delete extensions;
	}
	filter = mw->language_container->XmlGetChildStringValueByName("ImageFiles") + "|" + tmp + "|" + filter;
	filter = filter->Substring(0, filter->Length - 1); // - "|"

	openFileDialog->Filter = filter;

	if (mw->image_plugins.Count <= 0)
	{
		MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("NoImportModules"), mw->language_container->XmlGetChildStringValueByName("Open"), System::Windows::Forms::MessageBoxButtons::OK, System::Windows::Forms::MessageBoxIcon::Warning);
		delete c;
		return false;
	}

	if (openFileDialog->ShowDialog() == ::DialogResult::Cancel) 
	{
		delete c;
		return false;
	}

	// Determine image filetype
	int  dll_index = -1;
	bool megvan = false;
	for (i = 0; i < mw->image_plugins.Count && !megvan; ++i)
	{
		array<String^>^ extensions = mw->image_plugins[i].Key->Split(c);

		for (j = 0; j < extensions->Length && !megvan; ++j) 
			megvan = openFileDialog->FileName->EndsWith(extensions[j], System::StringComparison::InvariantCultureIgnoreCase);

		delete extensions;

		dll_index = i;
	}
	delete c;

	if (dll_index < 0)
	{
		MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("NoImportModules"), mw->language_container->XmlGetChildStringValueByName("Open"), System::Windows::Forms::MessageBoxButtons::OK, System::Windows::Forms::MessageBoxIcon::Warning);
		return false;
	}

    array<Object^>^ args = gcnew array<Object^>(1);
	args[0] = openFileDialog->FileName;

	IntPtr ip = IntPtr(0);
	try{
		ip = (IntPtr)mw->LoadDllMethod("image_loader.image_loader", "GetImageProperties", mw->image_plugins[dll_index].Value, args);
		delete args;
	} catch (Exception^ ) 
	{ 
		delete args; 
		return false; 
	}

	if (ip == IntPtr(0)) // Filter aborted or filter could not executed
	{
		MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("WrongImportModule"), mw->language_container->XmlGetChildStringValueByName("ImportError"));
		return false;
	}

	if (module != nullptr) delete module;
	if (exec_settings != nullptr) delete exec_settings;

	FilterName->Name = mw->image_plugins[dll_index].Value;
	FilterName->Text = openFileDialog->FileName;

	Byte* f = (Byte*)ip.ToPointer();
	module = gcnew xml_node(nullptr, "", "", "");
	module->children.Add(gcnew xml_node( f ));
	module->children[0]->name = "OutputImage0"; // RENAMING

	CopyModuleToExecSettings();

	XML_Editor^ xe = gcnew XML_Editor();
	xe->CreateXMLTree( module );
	if (parentForm->menuWorkflowShowXMLMetainfos->Checked) xe->ShowDialog();
	delete xe;

	return true;
}


bool GraphNode::SettingsExport()
{
	if (saveFileDialog->ShowDialog() == ::DialogResult::Cancel) return false;
	FilterName->Text = saveFileDialog->FileName;
	return true;
}


bool GraphNode::SettingsFilter()
{
	IntPtr^ ip;
    array<Object^>^ args = gcnew array<Object^>(2);
	args[0] = (IntPtr)(void*)(module->XML_Tree_Marshal());
    args[1] = nullptr; // loud mode :D (not silent)  (nullptr == Boxed(false))

	try{
		ip = (IntPtr^)mw->LoadDllMethod("FilterSpace.ImageFilter", "CreateFilterSettings", FilterName->Name, args);
	} catch (Exception^ ex) 
	{ 
		if (parentForm->menuWorkflowShowXMLMetainfos->Checked)
			MessageBox::Show("settingsfilter exception: " + ex->ToString()); 
		return false; 
	}

	Byte* f = (Byte*)ip->ToPointer();
	if (!f) return true;

	delete module;
	delete exec_settings;
	module = gcnew xml_node( f );
	delete [] f;

	CopyModuleToExecSettings();

	if (parentForm->menuWorkflowShowXMLMetainfos->Checked)
	{
		XML_Editor^ xe = gcnew XML_Editor();
		xe->Text = "CreateFilterSettings";
		xe->CreateXMLTree( module );
		xe->ShowDialog();
		delete xe;
	}

	return true;
}


bool GraphNode::ReplaceXMLPointedMemoryToClone(xml_node^ xt, String^ tag, int memorysize)
{
	if (memorysize <= 0) return false;
	int data = 0;
	xt->XmlGetChildValueByName(tag, data);
	if (!data) return false;
	Byte* ndata = new Byte[memorysize];
	memcpy(ndata, (void*)(data), memorysize);
	xt->XmlSetChildValueByName(tag, (int)(void*)(ndata), false);
	return true;
}

bool GraphNode::Run()
{
	// MAKE PRE-SET ON FILTER MODULES, REPEAT SILENT PREPARE TO THE FILTER

	if (gnt == GraphNodeType::FILTER_MODULE)
	{
		if (!IsSettingValid()) return false;

		SettingsClearOutputImages();

		// SHOW XML
		XML_Editor^ xe = gcnew XML_Editor();
		xe->Text = "Preparing filter run";
		xe->CreateXMLTree( exec_settings );
		if (parentForm->menuWorkflowShowXMLMetainfos->Checked) xe->ShowDialog();
		delete xe;

		// RERUN ONCE AND LAST AGAIN THE "CreateFilterSettings"
		IntPtr^ ip;
		array<Object^>^ args = gcnew array<Object^>(2);
		args[0] = (IntPtr)(void*)(exec_settings->XML_Tree_Marshal());
		args[1] = true; // silent! mode

		try
		{
			ip = (IntPtr^)mw->LoadDllMethod("FilterSpace.ImageFilter", "CreateFilterSettings", FilterName->Name, args);
		} 
		catch (Exception^) 
		{
			MessageBox::Show("ip = (IntPtr^)mw->LoadDllMethod(\"FilterSpace.ImageFilter\", \"CreateFilterSettings\", FilterName->Name, args);"); 
			return false; 
		}

		// UPDATE exec_settings
		delete exec_settings;
		Byte* f = (Byte*)ip->ToPointer();
		exec_settings = gcnew xml_node( f );
		delete [] f;

		// SHOW XML
		xe = gcnew XML_Editor();
		xe->Text = "Filter CreateFilterSettings result";
		xe->CreateXMLTree( exec_settings );
		if (parentForm->menuWorkflowShowXMLMetainfos->Checked) xe->ShowDialog();
		delete xe;

		// DO WE HAVE OVERWRITE SUPPORT?
		args->Resize(args, 0);
		bool canlocalize = (bool)mw->LoadDllMethod("FilterSpace.ImageFilter", "CanLocalize", FilterName->Name, args);


		// EXECUTING FILTER BY REMOTE METHOD INVOCATION
		// *******************
		bool result   = false;
		bool finished = false;
		int      iptr = 0, isize = 0;
		img_ptr       = &iptr;
		img_size      = &isize;

		int  nCoreKillSignal = 0;

		exec_settings->XmlSetChildValueByName("Processing_Ptr",   ((int)(void*)(&iptr)), false);
		exec_settings->XmlSetChildValueByName("Processing_Size",  ((int)(void*)(&isize)), false);
		exec_settings->XmlSetChildValueByName("nCore!KillSignal", ((int)(void*)(&nCoreKillSignal)), false);
		exec_settings->XmlSetChildValueByName("nCore!Enabled",    1, false);
		exec_settings->XmlSetChildValueByName("Run_Local",        1, false);

		args->Resize(args, 1);
		args[0] = (IntPtr)(void*)exec_settings->XML_Tree_Marshal();
		ThreadWrapper^ filterThread = gcnew ThreadWrapper(&finished, FilterName->Name, "FilterSpace.ImageFilter", "ExecuteFilter", args, mw->language_container);
		Thread^ thread = gcnew Thread(gcnew ThreadStart( filterThread, &ThreadWrapper::Start ) );
		thread->Start();

		ProgressTimer->Start();

		while (!finished) Application::DoEvents();

		result = (filterThread->res != nullptr) ? (bool)filterThread->res : false;

		if (!result)
		{
			nCoreKillSignal = 1;
			for (int countdown = 0; countdown < 40 && nCoreKillSignal > 0; countdown++) Thread::Sleep(100);
			if (nCoreKillSignal > 0) MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("nCoreKillError"), "", MessageBoxButtons::OK, MessageBoxIcon::Warning);
		}
		thread->Abort();
		ProgressTimer->Stop();
		delete thread;
		FilterProgressBar->Value = 100;

		// *******************
	}
	else if (gnt == GraphNodeType::IMPORT_MODULE || gnt == GraphNodeType::EXPORT_MODULE)
	{
		bool      finished = false, saved = false;
		Byte*     img_data = 0, *loaded = 0;
		int       ip = 0, is = 0;
		img_ptr = &ip;
		img_size = &is;

		array<Object^>^ args = gcnew array<Object^>(4);
		args[0] = FilterName->Text;
		args[1] = (IntPtr)(void*)(&ip);
		args[2] = (IntPtr)(void*)(&is);
		args[3] = (IntPtr)(void*)(&img_data); 

		if (gnt == GraphNodeType::EXPORT_MODULE) 
		{
			xml_node^ xn;
			exec_settings->XmlGetChildByName(xn, "InputImage0");
			args[3] = (IntPtr)(void*)xn->XML_Tree_Marshal();
		}

		ThreadWrapper^ filterThread;
		if (gnt == GraphNodeType::IMPORT_MODULE) filterThread = gcnew ThreadWrapper(&finished, FilterName->Name, "image_loader.image_loader", "Load_Image", args, mw->language_container);
		else                                     filterThread = gcnew ThreadWrapper(&finished, FilterName->Name, "ImageSaver.ImageSaver",     "Save_Image", args, mw->language_container);

		Thread^ thread = gcnew Thread(gcnew ThreadStart( filterThread, &ThreadWrapper::Start ) );
		thread->Start();
		ProgressTimer->Start();

		while (!finished) Application::DoEvents();

		if (gnt == GraphNodeType::IMPORT_MODULE) loaded = (filterThread->res != nullptr) ? (Byte*)(void*)(IntPtr)(filterThread->res) : 0;
		else                                     saved  = (filterThread->res != nullptr) ? (bool)(filterThread->res) : 0;

		thread->Abort();
		ProgressTimer->Stop();
		delete thread;
		FilterProgressBar->Value = 100;

		if (gnt == GraphNodeType::IMPORT_MODULE)
		{
			if (!finished || !loaded) // Filter aborted or filter could not executed
			{
				if (finished) MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("WrongImportModule"), mw->language_container->XmlGetChildStringValueByName("ImportError"));
				if (img_data) delete [] img_data;
				delete args;
				return false;
			}

			if (exec_settings != nullptr) delete exec_settings;
			exec_settings = gcnew xml_node(nullptr, "", "", "");
			exec_settings->children.Add(gcnew xml_node( loaded ));
			exec_settings->children[0]->name = "OutputImage0"; // RENAMING

			XML_Editor^	xe = gcnew XML_Editor();
			xe->CreateXMLTree(exec_settings);
			xe->Text = "AFTER PROCESSING";
			if (parentForm->menuWorkflowShowXMLMetainfos->Checked) xe->ShowDialog();
			delete xe;
		}
		else
		{
			if (finished && !saved) // Filter aborted or filter could not executed
				MessageBox::Show(mw->language_container->XmlGetChildStringValueByName("WrongExportModule"), mw->language_container->XmlGetChildStringValueByName("ExportError"));
		}
	}
	else if (gnt == GraphNodeType::SPLITTER_MODULE)
	{
		// CLONE "InputImage0" TO "OutputImage0"
		xml_node^ xt = nullptr;
		exec_settings->XmlRemoveChildByName("OutputImage0");
		exec_settings->XmlRemoveChildByName("OutputImage1");
		exec_settings->XmlGetChildByName(xt, "InputImage0");
		Byte* f = xt->XML_Tree_Marshal();
		xt = gcnew xml_node(f);
		xt->name = "OutputImage0";
		exec_settings->children.Add( xt );
		xt = gcnew xml_node(f);
		xt->name = "OutputImage1";
		exec_settings->children.Add( xt );
		delete [] f;
		// IT WORKS AS REFERENCE BASED PARAMETER PASS

		if (splittertype == "RASTER")
		{
			int stride = 0, height = 0;
			xt->XmlGetChildValueByName("Stride", stride);
			xt->XmlGetChildValueByName("Height", height);
			if (!ReplaceXMLPointedMemoryToClone(xt, "Data", stride * height)) return false;
		}
		else // "VECTOR"
		{
			int ptc = 0, plc = 0, plp = 0, pgc = 0, pgp = 0;

			xt->XmlGetChildValueByName("PointsCount",         ptc);
			xt->XmlGetChildValueByName("PolylinesPointCount", plc);
			xt->XmlGetChildValueByName("PolylinesPartCount",  plp);
			xt->XmlGetChildValueByName("PolygonsPointCount",  pgc);
			xt->XmlGetChildValueByName("PolygonsPartCount",   pgp);

			ReplaceXMLPointedMemoryToClone(xt, "PointsXArray",             ptc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PointsYArray",             ptc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PolylinesXArray",          plc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PolylinesYArray",          plc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PolylinesStartArray",      plp * sizeof(int));
			ReplaceXMLPointedMemoryToClone(xt, "PolylinesPartLengthArray", plp * sizeof(int));
			ReplaceXMLPointedMemoryToClone(xt, "PolygonsXArray",           pgc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PolygonsYArray",           pgc * sizeof(double));
			ReplaceXMLPointedMemoryToClone(xt, "PolygonsStartArray",       pgp * sizeof(int));
			ReplaceXMLPointedMemoryToClone(xt, "PolygonsPartLengthArray",  pgp * sizeof(int));
		}

		// SHOW XML
		if (parentForm->menuWorkflowShowXMLMetainfos->Checked)
		{
			XML_Editor^ xe = gcnew XML_Editor();
			xe->Text = "Splitter result";
			xe->CreateXMLTree( exec_settings );
			xe->ShowDialog();
			delete xe;
		}
		FilterProgressBar->Value = 100;
	}
	else if (gnt == GraphNodeType::NULLEXPORT_MODULE)
	{
		// DO NOTHING
		FilterProgressBar->Value = 100;
	}

	return true;
}

void GraphNode::GCMap_IncreaseReferenceCounter(std::map<int, int>& gcmap, String^ s, xml_node^ xt)
{
	int data = 0;
	if (!xt->XmlGetChildValueByName(s, data)) return;
	if (data) gcmap[data] = (gcmap.find(data) != gcmap.end()) ? gcmap[data] + 1 : 1;
}

void GraphNode::GCMap_DecreaseReferenceCounter(std::map<int, int>& gcmap, String^ s, xml_node^ xt)
{
	int data = 0;
	if (!xt->XmlGetChildValueByName(s, data)) return;
	if (data) if (gcmap.find(data) != gcmap.end()) if (gcmap[data] > 0) gcmap[data]--;
}


void GraphNode::GCMap_Update(std::map<int, int>& gcmap)
{
	xml_node^ xt;
	if (exec_settings == nullptr) return;

	for (int i = 0; i < out_edges; i++)
	{
		if (!exec_settings->XmlGetChildByName(xt, "OutputImage" + i.ToString())) continue;

		// RASTER
		GCMap_IncreaseReferenceCounter(gcmap, "Data", xt);

		// VECTOR
		GCMap_IncreaseReferenceCounter(gcmap, "PointsXArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PointsYArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolylinesStartArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolylinesPartLengthArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolylinesXArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolylinesYArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolygonsStartArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolygonsPartLengthArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolygonsXArray", xt);
		GCMap_IncreaseReferenceCounter(gcmap, "PolygonsYArray", xt);
	}

	for (int i = 0; i < in_edges; i++)
	{
		if (!exec_settings->XmlGetChildByName(xt, "InputImage" + i.ToString())) continue;

		// RASTER
		GCMap_DecreaseReferenceCounter(gcmap, "Data", xt);

		// VECTOR
		GCMap_DecreaseReferenceCounter(gcmap, "PointsXArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PointsYArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolylinesStartArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolylinesPartLengthArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolylinesXArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolylinesYArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolygonsStartArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolygonsPartLengthArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolygonsXArray", xt);
		GCMap_DecreaseReferenceCounter(gcmap, "PolygonsYArray", xt);
	}
}



void GraphNode::SettingsClearOutputImages()
{
	xml_node^ xn = nullptr;
	for (int i = 0; i < out_edges; i++)
	{
		if (exec_settings->XmlGetChildByName(xn, "OutputImage" + i.ToString()))
		{
			xn->XmlSetChildValueByName("Width",  0, false);
			xn->XmlSetChildValueByName("Height", 0, false);
			xn->XmlSetChildValueByName("Stride", 0, false);
			xn->XmlSetChildValueByName("Data",   0, false);
		}
	}
}



void GraphNode::ProgressTimer_Tick(System::Object^  sender, System::EventArgs^  e)
{
	if (*img_size) 
	{
		int val = (int)((((double)(*img_ptr)) * 100.0) / ((double)(*img_size)));
		if (val < FilterProgressBar->Minimum) val = FilterProgressBar->Minimum;
		if (val > FilterProgressBar->Maximum) val = FilterProgressBar->Maximum;
		FilterProgressBar->Value = val;
	}
}
