#pragma once

#include "stdafx.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;


namespace GeoFilterBank {

	/// <summary>
	/// Summary for GraphComponent
	/// </summary>
	ref class WorkflowManager;
	ref class MainWindow;
	enum class GraphNodeType {IMPORT_MODULE, EXPORT_MODULE, FILTER_MODULE, SPLITTER_MODULE, NULLEXPORT_MODULE, UNDEFINED };

	public ref class GraphNode : public System::Windows::Forms::UserControl
	{
	public:
		GraphNode()
		{
			InitializeComponent();

			module = nullptr;
			exec_settings = nullptr;
			gnt = GraphNodeType::UNDEFINED;
			initialized = false;
			parentForm = nullptr;
			moving = resizing = false;
			splittertype = "UNIVERSAL";
			DestructOnCreate = false;

			FreePinColor = Color::OrangeRed;
			LockedPinColor = Color::LimeGreen;
			AllPinUsedBackColor = Color::FromArgb(192, 255, 192);
			NormalBackColor = Color::Bisque;
		}

	protected:
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		~GraphNode()
		{
			if (components)
			{
				delete components;
			}

			if (module        != nullptr) delete module;
			if (exec_settings != nullptr) delete exec_settings;
		}

	public:
		System::Windows::Forms::Label^			FilterName;
		System::Windows::Forms::ProgressBar^	FilterProgressBar;
	private:
		System::Windows::Forms::PictureBox^		Sizer;
		System::Windows::Forms::PictureBox^		Settings;
		System::Windows::Forms::PictureBox^		SelfRemove;
		System::Windows::Forms::OpenFileDialog^	openFileDialog;
		System::Windows::Forms::SaveFileDialog^	saveFileDialog;
		System::Windows::Forms::ToolTip^		ToolTips;
		System::Windows::Forms::Timer^			ProgressTimer;
		System::ComponentModel::IContainer^		components;

		cli::array<System::Windows::Forms::PictureBox^>^	edge_contactors;
		cli::array<System::Windows::Forms::Label^>^			edge_labels;

	private:


#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->components = (gcnew System::ComponentModel::Container());
			System::ComponentModel::ComponentResourceManager^  resources = (gcnew System::ComponentModel::ComponentResourceManager(GraphNode::typeid));
			this->Sizer = (gcnew System::Windows::Forms::PictureBox());
			this->FilterName = (gcnew System::Windows::Forms::Label());
			this->FilterProgressBar = (gcnew System::Windows::Forms::ProgressBar());
			this->Settings = (gcnew System::Windows::Forms::PictureBox());
			this->SelfRemove = (gcnew System::Windows::Forms::PictureBox());
			this->ToolTips = (gcnew System::Windows::Forms::ToolTip(this->components));
			this->openFileDialog = (gcnew System::Windows::Forms::OpenFileDialog());
			this->saveFileDialog = (gcnew System::Windows::Forms::SaveFileDialog());
			this->ProgressTimer = (gcnew System::Windows::Forms::Timer(this->components));
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Sizer))->BeginInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Settings))->BeginInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->SelfRemove))->BeginInit();
			this->SuspendLayout();
			// 
			// Sizer
			// 
			this->Sizer->Image = (cli::safe_cast<System::Drawing::Image^  >(resources->GetObject(L"Sizer.Image")));
			this->Sizer->Location = System::Drawing::Point(185, 64);
			this->Sizer->Name = L"Sizer";
			this->Sizer->Size = System::Drawing::Size(16, 16);
			this->Sizer->TabIndex = 0;
			this->Sizer->TabStop = false;
			this->ToolTips->SetToolTip(this->Sizer, L"Resize filter by drag\'n\'drop style");
			this->Sizer->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::Sizer_MouseDown);
			this->Sizer->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::Sizer_MouseMove);
			this->Sizer->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::Sizer_MouseUp);
			// 
			// FilterName
			// 
			this->FilterName->AutoSize = true;
			this->FilterName->Enabled = false;
			this->FilterName->Font = (gcnew System::Drawing::Font(L"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				static_cast<System::Byte>(238)));
			this->FilterName->ForeColor = System::Drawing::Color::SaddleBrown;
			this->FilterName->Location = System::Drawing::Point(12, 6);
			this->FilterName->Name = L"FilterName";
			this->FilterName->Size = System::Drawing::Size(71, 13);
			this->FilterName->TabIndex = 1;
			this->FilterName->Text = L"Filter Name";
			this->ToolTips->SetToolTip(this->FilterName, L"Name of the filter");
			// 
			// FilterProgressBar
			// 
			this->FilterProgressBar->Location = System::Drawing::Point(27, 30);
			this->FilterProgressBar->Name = L"FilterProgressBar";
			this->FilterProgressBar->Size = System::Drawing::Size(102, 12);
			this->FilterProgressBar->TabIndex = 2;
			// 
			// Settings
			// 
			this->Settings->Image = (cli::safe_cast<System::Drawing::Image^  >(resources->GetObject(L"Settings.Image")));
			this->Settings->Location = System::Drawing::Point(174, 3);
			this->Settings->Name = L"Settings";
			this->Settings->Size = System::Drawing::Size(16, 16);
			this->Settings->TabIndex = 3;
			this->Settings->TabStop = false;
			this->ToolTips->SetToolTip(this->Settings, L"Click here to adjust filter properties");
			this->Settings->Click += gcnew System::EventHandler(this, &GraphNode::Settings_Click);
			// 
			// SelfRemove
			// 
			this->SelfRemove->Image = (cli::safe_cast<System::Drawing::Image^  >(resources->GetObject(L"SelfRemove.Image")));
			this->SelfRemove->Location = System::Drawing::Point(196, 3);
			this->SelfRemove->Name = L"SelfRemove";
			this->SelfRemove->Size = System::Drawing::Size(16, 16);
			this->SelfRemove->TabIndex = 4;
			this->SelfRemove->TabStop = false;
			this->ToolTips->SetToolTip(this->SelfRemove, L"Click to remove this filter");
			this->SelfRemove->Click += gcnew System::EventHandler(this, &GraphNode::SelfRemove_Click);
			// 
			// ToolTips
			// 
			this->ToolTips->AutomaticDelay = 0;
			this->ToolTips->AutoPopDelay = 15000;
			this->ToolTips->BackColor = System::Drawing::Color::Bisque;
			this->ToolTips->ForeColor = System::Drawing::Color::SaddleBrown;
			this->ToolTips->InitialDelay = 0;
			this->ToolTips->IsBalloon = true;
			this->ToolTips->ReshowDelay = 100;
			this->ToolTips->ShowAlways = true;
			this->ToolTips->ToolTipIcon = System::Windows::Forms::ToolTipIcon::Info;
			this->ToolTips->ToolTipTitle = L"Quick Info:";
			// 
			// saveFileDialog
			// 
			this->saveFileDialog->Title = L"Export";
			// 
			// ProgressTimer
			// 
			this->ProgressTimer->Interval = 200;
			this->ProgressTimer->Tick += gcnew System::EventHandler(this, &GraphNode::ProgressTimer_Tick);
			// 
			// GraphNode
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->BackColor = System::Drawing::Color::Bisque;
			this->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->Controls->Add(this->SelfRemove);
			this->Controls->Add(this->Settings);
			this->Controls->Add(this->FilterProgressBar);
			this->Controls->Add(this->FilterName);
			this->Controls->Add(this->Sizer);
			this->Name = L"GraphNode";
			this->Size = System::Drawing::Size(215, 114);
			this->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::GraphNode_MouseDown);
			this->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::GraphNode_MouseMove);
			this->MouseDoubleClick += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::GraphNode_MouseDoubleClick);
			this->Resize += gcnew System::EventHandler(this, &GraphNode::GraphNode_Resize);
			this->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &GraphNode::GraphNode_MouseUp);
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Sizer))->EndInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Settings))->EndInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->SelfRemove))->EndInit();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion
	public: 
		bool InitializeNode(WorkflowManager^ p, String^ title, String^ filter_url, GraphNodeType gt);
		void RestoreSavedNode(WorkflowManager^ p, String^ title, String^ filter_url, GraphNodeType gt);
		void RestoreSavedSettings(xml_node^ xn);
		bool IsComplete();
		bool IsSettingValid();
		int  IndexOfPointedInEdge(int x, int y);
		int  IndexOfPointedOutEdge(int x, int y);
		void GetAbsolutePosOfEdge(int pin, int& x, int& y);
		void ReleaseEdge(int pin);
		bool LockEdge(int pin, String^ type);
		bool Run();
		void GCMap_Update(std::map<int, int>& gcmap);
		int  InputImageIndex( int pin){ return pin; }
		int  OutputImageIndex(int pin){ return pin - in_edges; }
		int  GetInEdges(){  return in_edges; }
		int  GetOutEdges(){ return out_edges; }
		String^ GetPinType(int pin);


		GraphNodeType	GetNodeType(){		 return gnt; }
		xml_node^		GetXmlDescription(){ return module; }
		xml_node^		GetXmlExecutionSettings(){ return exec_settings; }
		bool			DestructOnCreate;

	private:
		void Edge_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void Edge_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void InEdge_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void OutEdge_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);

		void GraphNode_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void GraphNode_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void GraphNode_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void GraphNode_MouseDoubleClick(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void GraphNode_Resize(System::Object^  sender, System::EventArgs^  e);

		void Sizer_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void Sizer_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);
		void Sizer_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e);

		void SelfRemove_Click(System::Object^  sender, System::EventArgs^  e);
		void Settings_Click(System::Object^  sender, System::EventArgs^  e);
		void ProgressTimer_Tick(System::Object^  sender, System::EventArgs^  e);

		void GetOneImageSetting(xml_node^ fs, int& Width, int& Height, int& Stride, int& Packed, int& Interleaved, int& ChannelCount, int& PixelSize);
		bool IsMul(int num, int base);
		bool IsPower(int num, int power);
		bool SettingsImport();
		bool SettingsExport();
		bool SettingsFilter();
		void CopyModuleToExecSettings();
		void SettingsClearOutputImages();
		bool ReplaceXMLPointedMemoryToClone(xml_node^ xt, String^ tag, int memorysize);

		void GCMap_IncreaseReferenceCounter(std::map<int, int>& gcmap, String^ s, xml_node^ xt);
		void GCMap_DecreaseReferenceCounter(std::map<int, int>& gcmap, String^ s, xml_node^ xt);

		int				x, y;
		int				*img_ptr, *img_size;
		int				in_edges, out_edges;
		bool			moving, resizing, initialized;
		GraphNodeType	gnt;
		String^			splittertype;

		WorkflowManager^	parentForm;
		MainWindow^			mw;
		xml_node^			module;
		xml_node^			exec_settings;

		Color			FreePinColor, LockedPinColor, AllPinUsedBackColor, NormalBackColor;
};

	public ref class GraphEdge
	{
	public:
		GraphEdge(GraphNode^ from, GraphNode^ to, int src, int dest) : source(from), target(to), source_pin(src), target_pin(dest){}
		GraphNode^ source;
		GraphNode^ target;
		int source_pin, target_pin;
	};

}
