
#include "stdafx.h"



namespace GeoFilterBank {


void InputsFinder::CreateAdditionalControls(xml_node^ fs)
{
	int count;
	if (!fs->XmlGetChildValueByName("InputImageCount", count)) return;

	this->Height = this->Height + 30 * count;
	Button_OK->Top = this->Height - Button_OK->Height - 40;
	Button_Cancel->Top = this->Height - Button_Cancel->Height - 40;

	System::Windows::Forms::Label^ lbl;
	System::Windows::Forms::ComboBox^ cbx;

	for (int i = 0; i < count; ++i)
	{
		lbl = gcnew Label();
		lbl->Text = "Bemenet " + (i + 1).ToString() + " :";
		lbl->Left = label2->Left;
		lbl->Top = label2->Top + 30 + i * 30;
		lbl->AutoSize = true;
		lbl->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
		this->Controls->Add( lbl );

		cbx = gcnew ComboBox();
		cbx->Left = label3->Left;
		cbx->Top = lbl->Top;
		cbx->Width = 270;
		cbx->Height = 20;
		cbx->Name = i.ToString();
		comboBoxes.Add( cbx );
		comboBoxesReferences.Add( gcnew List<LayerContainer^> );
		this->Controls->Add( cbx );

		xml_node^ fs2;
		if ( fs->XmlGetChildByName( fs2, "InputImage" + i.ToString() ) )
		{
			String^ strPinName;
			if ( fs2->XmlGetChildValueByName( "PinName", strPinName ) ) lbl->Text = strPinName;
			for (int j = 0; j < mainWindow->MdiChildren->Length; ++j)
			{
				LayerContainer^ lc = (LayerContainer^)(mainWindow->MdiChildren[j]);
				if ( IsLayerCompatible( lc, fs2, mainWindow->debug_mode ) )
				{
					cbx->Items->Add(lc->Text);
					comboBoxesReferences[i]->Add( lc );
				}
			}
		}

		if (cbx->Items->Count == 0)
		{
			cbx->BackColor = Color::Red;
			cbx->Enabled = false;
			usable = false;
		}
		else
		{
			cbx->SelectedIndex = 0;
		}

		PictureBox^ pic = gcnew PictureBox();
		pic->Top = cbx->Top + 2;
		pic->Left = cbx->Left + cbx->Width + 5;
		pic->Width = 16;
		pic->Height = 16;
		pic->Image = iconSet->Images[((cbx->Items->Count) ? 0 : 1)];
		pic->Tag = (Object^)fs2;
		pic->Click += gcnew System::EventHandler(this, &InputsFinder::pictureBox_Click);
		this->Controls->Add( pic );
	}
}
//

#define COND_TEST(condition, msg) { if (condition) { if (debug_mode) MessageBox::Show(msg); return false; } }

bool InputsFinder::IsLayerCompatible(LayerContainer^ lc, xml_node^ fs, bool debug_mode)
{
	if (fs->XmlIsChildExist("Width"))
	{
		if (debug_mode) MessageBox::Show("Checking raster layer compatibility");

		Byte* data;
		xml_node^ channel_alloc;
		xml_node^ lc_channel_alloc;
		int ch_fs, ch_lc;
		int i, width, height, stride, packed, interleaved, channelcount, pixelsize;
		int lc_width, lc_height, lc_stride, lc_packed, lc_interleaved, lc_channelcount, lc_pixelsize;

		COND_TEST(ImageFilter::GetOneImageSetting(fs, width, height, stride, packed, interleaved, channelcount, pixelsize, data), "GetOneImageSetting (ImageNode) N/A");
		COND_TEST(! fs->XmlGetChildByName(channel_alloc, "ChannelsBitAllocation" ), "ChannelsBitAllocation (Filter) N/A");

		// Getting infos on source
		if (lc->geoViewer->IsImageDataLoaded())
		{
			COND_TEST(ImageFilter::GetOneImageSetting(lc->geoViewer->ImageNode, lc_width, lc_height, lc_stride, lc_packed, lc_interleaved, lc_channelcount, lc_pixelsize, data), "GetOneImageSetting (ImageNode) N/A");
			COND_TEST(!lc->geoViewer->ImageNode->XmlGetChildByName(lc_channel_alloc, "ChannelsBitAllocation" ), "ChannelsBitAllocation (ImageNode) N/A");
		}
		else
		{
			COND_TEST(true, "ImageDataLoaded == False");
		}

		// OPTIONAL COMPARE STRUCTURES --> MATCH-MAKING

		if (width == 2) 
		{
			for (i = 1; i < lc_width; i <<= 1);
			COND_TEST(i != lc_width, "x^2");
		}
		if (height == 2) 
		{
			for (i = 1; i < lc_height; i <<= 1);
			COND_TEST(i != lc_height, "y^2");
		}
		if (width  == 8) COND_TEST(((lc_width  >> 3) << 3) !=  lc_width,  "width*8");
		if (height == 8) COND_TEST(((lc_height >> 3) << 3) !=  lc_height, "height*8");

		// TEST PACKING COMPATIBILITY -->
		COND_TEST(packed != lc_packed, "packing");

		// TEST INTERLEAVING COMPATIBILITY -->
		COND_TEST(interleaved != lc_interleaved, "interleaving");

		// TEST STRIDE COMPATIBILITY -->
		if (stride) COND_TEST(stride != lc_stride, "stride: " + lc_stride.ToString());

		// TEST CHANNEL BIT ALLOCATION COMPATIBILITY -->
		if (channelcount)
		{
			COND_TEST(channelcount != lc_channelcount, "channelcount");

			for (int i = 0; i < channelcount; ++i)
			{
				COND_TEST(!channel_alloc->XmlGetChildValueByName("Channel" + i.ToString(), ch_fs), "No channel" + i.ToString() + " on filter");
				COND_TEST(!lc_channel_alloc->XmlGetChildValueByName("Channel" + i.ToString(), ch_lc), "No channel" + i.ToString() + " on layer");
				COND_TEST(ch_fs != ch_lc && ch_fs != 0, "Channel condition on Filter [" + ch_fs.ToString() + "] != Layer [" + ch_lc.ToString() + "] on channel " + i.ToString());
			}
		}
		else
		{
			COND_TEST(!channel_alloc->XmlGetChildValueByName("Channel0", ch_fs), "No channel0 on filter");

			for (int i = 0; i < channelcount; ++i)
			{
				COND_TEST(!lc_channel_alloc->XmlGetChildValueByName("Channel" + i.ToString(), ch_lc), "No channel" + i.ToString() + " on layer");
				COND_TEST(ch_fs != ch_lc && ch_fs != 0, "Channel condition on Filter [" + ch_fs.ToString() + "] != Layer [" + ch_lc.ToString() + "] on channel " + i.ToString());
			}
		}

		// TEST PIXELSIZE COMPATIBILITY -->
		if (pixelsize) COND_TEST(pixelsize != lc_pixelsize, "Layer pixelsize " + lc_pixelsize.ToString() + " != Filter pixelsize " + pixelsize.ToString());

	}
	else
	{
		if (debug_mode) MessageBox::Show("Checking vector layer compatibility");

		if (! lc->geoViewer->IsVectorDataLoaded()) return false;
	}

	if (debug_mode) MessageBox::Show("Layer compatibility ok.");

	return true;
}



//
void InputsFinder::Button_OK_Click(System::Object^  sender, System::EventArgs^  e)
{
	for (int i = 0; i < comboBoxes.Count; ++i)
	{
		if (comboBoxes[i]->Enabled) 
		{
			int j = comboBoxes[i]->SelectedIndex;
			mainWindow->layers_for_filter.Add( comboBoxesReferences[i][j] );
		}
	}
	this->Close();
}


void InputsFinder::Button_Cancel_Click(System::Object^  sender, System::EventArgs^  e)
{
	this->Close();
}


void InputsFinder::pictureBox_Click(System::Object^  sender, System::EventArgs^  e)
{
	PictureBox^ pic = (PictureBox^)sender;
	xml_node^ xn = (xml_node^)pic->Tag;
	XML_Editor^ xe = gcnew XML_Editor();
	xe->CreateXMLTree( xn );
	xe->Text = "Szr forrskvetelmnye";
	xe->ShowDialog();
}

}