
#include "stdafx.h"

using namespace System;
using namespace System::IO;
using namespace System::Collections;
using namespace System::Collections::Generic;
using namespace System::ComponentModel;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;
using namespace System::Text;
using namespace System::Windows::Forms;
using namespace System::Drawing;
using namespace System::Reflection;
using namespace System::Threading;
using namespace System::Runtime::InteropServices;


#define IMAGEFILTER_API_VERSION VER_GEN(1,2,0,0)

#define SPLASH_LABEL_INFO_DELAYING 25

using namespace GeoFilterBank;

ref class FilterThreadWrapper 
{
public:
	FilterThreadWrapper(bool* finish, String^ dll, String^ classname, String^ methodname, array<Object^>^ paramlist, xml_node^ lang_cont)
	{
		res = nullptr;
		finished = finish;
		*finished = false;
		language_container = lang_cont;
		dll_name = dll;
		class_name = classname;
		method_name = methodname;
		parameter_list = paramlist;
	}
	FilterThreadWrapper(FilterThreadWrapper%){}

	Object^ LoadDllMethod()
	{
		Assembly^ u = Assembly::LoadFile(dll_name);
		Type^ t = u->GetType(class_name);
		if (t != nullptr)
		{
			MethodInfo^ m = t->GetMethod(method_name);
			if (m != nullptr)
			{
				if (parameter_list->Length >= 1)
					return m->Invoke(nullptr, parameter_list);
				else
					return m->Invoke(nullptr, nullptr);
			}
			else
			{
				Exception^ ex = gcnew Exception(language_container->XmlGetChildStringValueByName("MethodNotFound")  + ": " + method_name);
				throw ex;
			}
		}
		else
		{
			Exception^ ex = gcnew Exception(language_container->XmlGetChildStringValueByName("ClassNotFound") + ": " + class_name);
			throw ex;
		}
	}

	void FilterStart()
	{
		try { res = LoadDllMethod(); }
		catch (Exception^ ) {}
		*finished = true;
	}

	Object^ res;
	bool* finished;

private:
	xml_node^ language_container;
	String^ dll_name;
	String^ class_name;
	String^ method_name;
	array<Object^>^ parameter_list;
};



MainWindow::MainWindow()
{
	InitializeComponent();
	
	splashsreen = gcnew SplashScreen();
	splashsreen->LabelInfo = "";
	splashsreen->ShowSplash();
	Thread::Sleep(SPLASH_LABEL_INFO_DELAYING);
	splashscreen_shown = true;

	RefreshLanguageList();

	RefreshFilterList();
	RefreshImageLoaderList();
	RefreshImageSaverList();

	Thread::Sleep(200);
	splashscreen_shown = false;
	splashsreen->CloseSplash();
	delete splashsreen;

	menuFileOpenImage->Image = menuFileImageList->Images[0];
	menuFileSave->Image = menuFileImageList->Images[1];
	menuFileCloseWorkspace->Image = menuFileImageList->Images[2];
	menuFileExit->Image = menuFileImageList->Images[3];

	menuEditSelectNone->Image = menuEditImageList->Images[0];
	menuEditSelectionToImage->Image = menuEditImageList->Images[1];

	menuFilterOverWriteToolStripMenuItem->Image= menuFilterImageList->Images[2];
	menuFilterRefresh->Image = menuFilterImageList->Images[0];

	menuViewWorkflow->Image = menuViewImageList->Images[0];
	menuViewToolbox->Image = menuViewImageList->Images[1];
	menuViewSelections->Image = menuViewImageList->Images[2];
	menuViewScrollBars->Image = menuViewImageList->Images[3];
	menuViewDeveloperMode->Image = menuViewImageList->Images[4];
	menuViewImageInformation->Image = menuViewImageList->Images[5];
	toolbox_exist = false;
	workflowmanager_exist = false;
	debug_mode = false;
	language_container = gcnew xml_node(nullptr, "", "", "");
	menuLanguage_Click(nullptr, nullptr);
}

Object^ MainWindow::LoadDllMethod(String^ title_class, String^ title_void, String^ path, array<Object^>^ parameters)
{
	Assembly^ u = Assembly::LoadFile(path);
    Type^ t = u->GetType(title_class);
    if (t != nullptr)
    {
        MethodInfo^ m = t->GetMethod(title_void);
        if (m != nullptr)
        {
            if (parameters->Length >= 1)
                return m->Invoke(nullptr, parameters);
            else
                return m->Invoke(nullptr, nullptr);
        }
		else
		{
			Exception^ ex = gcnew Exception(language_container->XmlGetChildStringValueByName("MethodNotFound")  + ": " + title_void);
			throw ex;
		}
    }
	else
	{
		Exception^ ex = gcnew Exception(language_container->XmlGetChildStringValueByName("ClassNotFound") + ": " + title_class);
		throw ex;
	}
}

void MainWindow::RefreshFilterList()
{
    int count = menuFilter->DropDownItems->Count;
    int marad = 3;
    for (int i = marad; i < count; ++i) menuFilter->DropDownItems->RemoveAt(marad);

    menuFilter->DropDownItems->Add("-");

	int framework_api_version = IMAGEFILTER_API_VERSION;
	array<String^>^ dlls = Directory::GetFiles(Application::StartupPath, "Filter*.dll", SearchOption::TopDirectoryOnly);
    array<Object^>^ args = gcnew array<Object^>(0);
	for (int i = 0; i < dlls->Length; ++i)
    {
		String^ f = dlls[i];
		String^ dll_title;
		String^ filter_group = "";
		int  version, from, to;
		bool add_filter = true;
        try
        {
            dll_title = (String^)LoadDllMethod("FilterSpace.ImageFilter", "GetFilterName", f, args);
			version = (int)LoadDllMethod("FilterSpace.ImageFilter", "GetAPIVersion", f, args);
			if (framework_api_version != version)
			{
				add_filter = false;
				if (debug_mode) MessageBox::Show("Dll API: " + version.ToString() + "\nGEO API: " + framework_api_version.ToString(), dll_title);
			}
        }
		catch (Exception^ ex)
		{
			MessageBox::Show(ex->ToString(), f);
			add_filter = false;
		}

		try{ filter_group = (String^)LoadDllMethod("FilterSpace.ImageFilter", "GetFilterGroupName", f, args); }
		catch (Exception^ ){ filter_group = "";  }

		// FILTER GROUPING EXTENSION - 2010 apr 25

		if (add_filter)
		{
			ToolStripMenuItem^ place_of_filter = menuFilter;
			if (filter_group != "")
			{
				count = menuFilter->DropDownItems->Count;
				for (int i = marad + 1; i < count; i++)
				{
					if (menuFilter->DropDownItems[i]->Text == filter_group)
					{
						place_of_filter = (ToolStripMenuItem^)(menuFilter->DropDownItems[i]);
						break;
					}
				}
				if (place_of_filter == menuFilter)
				{
					to = count;
					for (int i = marad + 1; i < to; i++) 
						if (((ToolStripMenuItem^)menuFilter->DropDownItems[i])->DropDownItems->Count == 0) 
						{
							to = i;
							break;
						}
					int group_pos = FindFilterNamePlace(filter_group, menuFilter, marad + 1, to);
					menuFilter->DropDownItems->Insert(group_pos, place_of_filter = (gcnew ToolStripMenuItem(filter_group)));
				}
			}

			int last = place_of_filter->DropDownItems->Count;
			for (from = ((place_of_filter == menuFilter) ? (marad + 1) : 0); from < last; from++) 
				if (((ToolStripMenuItem^)place_of_filter->DropDownItems[from])->DropDownItems->Count == 0) 
					break;

			int filter_pos = FindFilterNamePlace(dll_title, place_of_filter, from, last);

			ToolStripMenuItem^ newItem = gcnew ToolStripMenuItem(dll_title);
			newItem->Click += gcnew System::EventHandler(this, &MainWindow::menuFilterFilterExecute_Click);
			newItem->Name = f;
			newItem->Image = menuFilterImageList->Images[3];
			place_of_filter->DropDownItems->Insert(filter_pos, newItem);

			if (splashscreen_shown)
			{
				splashsreen->LabelInfo = "Filter: " + dll_title;
				Thread::Sleep(SPLASH_LABEL_INFO_DELAYING);
			}
		}
    }
}

int MainWindow::FindFilterNamePlace(String^ name, ToolStripMenuItem^ item, int from, int to)
{
	for (int i = from; i < to; i++) 
		if (0 >= String::Compare(name, item->DropDownItems[i]->Text)) 
			return i;
	return to;
}


void MainWindow::RefreshImageLoaderList()
{
	image_plugins.Clear();

	array<String^>^ dlls = Directory::GetFiles(Application::StartupPath, "in_*.dll", SearchOption::TopDirectoryOnly);
    array<Object^>^ args = gcnew array<Object^>(0);
	for (int i = 0; i < dlls->Length; ++i)
    {
		String^ f = dlls[i];
        try
        {
            String^ file_extension = (String^)LoadDllMethod("image_loader.image_loader", "SupportedFileExtension", f, args);
			KeyValuePair< String^, String^ > valami(file_extension->ToLower(), f);
			image_plugins.Add( valami );

			if (splashscreen_shown)
			{
				splashsreen->LabelInfo = "Import: " + file_extension->ToLower();
				Thread::Sleep(SPLASH_LABEL_INFO_DELAYING);
			}
        }
		catch (Exception^){ }
    }
}


void MainWindow::RefreshImageSaverList()
{
    menuFileSave->DropDownItems->Clear();

	int framework_api_version = IMAGEFILTER_API_VERSION;
	array<String^>^ dlls = Directory::GetFiles(Application::StartupPath, "out_*.dll", SearchOption::TopDirectoryOnly);
	if (debug_mode) MessageBox::Show("Found " + dlls->Length.ToString() + " export modules");
    array<Object^>^ args = gcnew array<Object^>(0);
	for (int i = 0; i < dlls->Length; ++i)
    {
		String^ f = dlls[i];
        try
        {
            String^ dll_title = (String^)LoadDllMethod("ImageSaver.ImageSaver", "GetExportModuleName", f, args);
			if (debug_mode) MessageBox::Show(f + ": GetExportModuleName -> OK");
			int version = (int)LoadDllMethod("ImageSaver.ImageSaver", "GetAPIVersion", f, args);
			if (debug_mode) MessageBox::Show(dll_title + ": GetAPIVersion -> OK");
			if (framework_api_version == version)
			{
				menuFileSave->DropDownItems->Add(dll_title);
				menuFileSave->DropDownItems[i]->Click += gcnew System::EventHandler(this, &MainWindow::menuFileSaveImage_Click);
				menuFileSave->DropDownItems[i]->Name = f;
				menuFileSave->DropDownItems[i]->Image = menuFileImageList->Images[1];
				if (debug_mode) MessageBox::Show("Adding export module: ");

				if (splashscreen_shown)
				{
					splashsreen->LabelInfo = "Export: " + dll_title;
					Thread::Sleep(SPLASH_LABEL_INFO_DELAYING);
				}
			}
			else
			{
				if (debug_mode) MessageBox::Show("Dll API: " + version.ToString() + "\nExport modul API: " + framework_api_version.ToString(), dll_title);
			}
        }
		catch (Exception^ ex)
		{
			if (debug_mode) MessageBox::Show(ex->ToString(), f);
		}
    }

	if (menuFileSave->DropDownItems->Count == 0)
	{
		String^ str = language_container->XmlGetChildStringValueByName("NoExportModules");
		menuFileSave->DropDownItems->Add(str);
		menuFileSave->DropDownItems[0]->Enabled = false;
	}
}

void MainWindow::RefreshLanguageList()
{
	menuLanguage->DropDownItems->Clear();

	array<String^>^ langs = Directory::GetFiles(Application::StartupPath, "*.lang", SearchOption::TopDirectoryOnly);
	for (int i = 0; i < langs->Length; ++i)
    {
		String^ f = langs[i];
		String^ lang = f->Substring(0, f->Length - 5);
		for (int j = lang->Length - 1; j > 0; --j)
		{
			if (lang[j] == '\\')
			{
				lang = lang->Substring(j+1);
				break;
			}
		}
		menuLanguage->DropDownItems->Add(lang);
		menuLanguage->DropDownItems[i]->Name = f;
		menuLanguage->DropDownItems[i]->Click += gcnew System::EventHandler(this, &MainWindow::menuLanguage_Click);
    }

	MainWindow::menuLanguage_Click(nullptr, nullptr);
}


void MainWindow::ToolBoxClose()
{
	toolbox_exist = false;
    menuViewToolbox->Checked = false;
}

void MainWindow::WorkflowManagerClose()
{
	workflowmanager_exist = false;
	menuViewWorkflow->Checked = false;
}

// Sajt beptett kpkezel rendszerrel 
void MainWindow::menuFileOpenImage_Click(System::Object^  sender, System::EventArgs^  e) 
{
	// Creating image filetypes filter
	int i, j;
	String^ filter = "";
	String^ tmp = "";
	array<__wchar_t>^ c = gcnew array<__wchar_t>(1);
	c[0] = ',';
	for (i = 0; i < image_plugins.Count; ++i)
	{
		filter += image_plugins[i].Key->ToUpper() + " " + language_container->XmlGetChildStringValueByName("Files") + "|";
		array<String^>^ extensions = image_plugins[i].Key->Split(c);

		for (j = 0; j < extensions->Length; ++j) 
		{
			filter += "*." + extensions[j];
			tmp += "*." + extensions[j] + ";";
			if (j != extensions->Length - 1)
			{
				filter += ";";
			}
		}
		filter += "|";

		delete extensions;
	}
	filter = language_container->XmlGetChildStringValueByName("ImageFiles") + "|" + tmp + "|" + filter;
	filter = filter->Substring(0, filter->Length - 1); // - "|"

	openFileDialog->Filter = filter;

	if (image_plugins.Count <= 0)
	{
		MessageBox::Show(language_container->XmlGetChildStringValueByName("NoImportModules"), language_container->XmlGetChildStringValueByName("Open"), System::Windows::Forms::MessageBoxButtons::OK, System::Windows::Forms::MessageBoxIcon::Warning);
		delete c;
		return;
	}

	if (openFileDialog->ShowDialog() == ::DialogResult::Cancel) 
	{
		delete c;
		return;
	}

	// Determine image filetype
	int  dll_index = -1;
	bool megvan = false;
	Byte* loaded = 0;
	for (i = 0; i < image_plugins.Count && !megvan; ++i)
	{
		array<String^>^ extensions = image_plugins[i].Key->Split(c);

		for (j = 0; j < extensions->Length && !megvan; ++j) 
			megvan = openFileDialog->FileName->EndsWith(extensions[j], System::StringComparison::InvariantCultureIgnoreCase);

		delete extensions;

		dll_index = i;
	}
	delete c;

	if (dll_index < 0)
	{
		MessageBox::Show(language_container->XmlGetChildStringValueByName("NoImportModules"), language_container->XmlGetChildStringValueByName("Open"), System::Windows::Forms::MessageBoxButtons::OK, System::Windows::Forms::MessageBoxIcon::Warning);
		return;
	}

	bool finished = false;
	int img_ptr = 0, img_size = 0;
	Byte* img_data = 0;
    array<Object^>^ args = gcnew array<Object^>(4);
	args[0] = openFileDialog->FileName;
	args[1] = (IntPtr)(void*)(&img_ptr);
	args[2] = (IntPtr)(void*)(&img_size);
	args[3] = (IntPtr)(void*)(&img_data);

	ProcessStatus^ ps = gcnew ProcessStatus(&img_ptr, &img_size, &finished, language_container);
	ps->Text = openFileDialog->FileName;
	FilterThreadWrapper^ filterThread = gcnew FilterThreadWrapper(&finished, image_plugins[dll_index].Value, "image_loader.image_loader", "Load_Image", args, language_container);
	Thread^ thread = gcnew Thread(gcnew ThreadStart( filterThread, &FilterThreadWrapper::FilterStart ) );
	thread->Start();
	ps->ShowDialog();
	loaded = (filterThread->res != nullptr) ? (Byte*)(void*)(IntPtr)(filterThread->res) : 0;
	thread->Abort();
	delete thread;


	if (!finished || !loaded) // Filter aborted or filter could not executed
	{
		if (finished)
		{
			MessageBox::Show(language_container->XmlGetChildStringValueByName("WrongImportModule"), language_container->XmlGetChildStringValueByName("ImportError"));
		}
		if (img_data)
		{
			delete [] img_data;
		}
		delete args;
		return;
	}

	xml_node^ node_tmp = gcnew xml_node( loaded );

	// ---
	if (debug_mode) 
	{
		XML_Editor^ xe2 = gcnew XML_Editor();
		xe2->Text = "Opening image";
		xe2->CreateXMLTree( node_tmp );
		xe2->ShowDialog();
	}
	// ---

	LayerContainer^ f = gcnew LayerContainer();
	String^ sTmp;

	// CONTAIN BITMAP IMAGE?
	if (node_tmp->XmlGetChildValueByName("Width", sTmp))           f->geoViewer->SetBitmapImage( node_tmp );
	// CONTAIN VECTOR IMAGE?
	if (node_tmp->XmlGetChildValueByName("BoundingBoxLeft", sTmp)) f->geoViewer->SetVectorImage( node_tmp );

	delete node_tmp;
	AddOwnedForm(f);
    f->MdiParent = this;
	String^ fname = openFileDialog->FileName;
	for (int i = fname->Length; i; --i) if (fname[i-1] == '\\') { fname = fname->Substring(i); break; }
	f->Text = fname;
    f->Show();
}



void MainWindow::menuFileSaveImage_Click(System::Object^  sender, System::EventArgs^  e)
{
    if (this->MdiChildren->Length == 0) return;
	ToolStripMenuItem^ t = (ToolStripMenuItem^)sender;
	xml_node^ image = nullptr;

	array<Object^>^ args0 = gcnew array<Object^>(0);
	String^ sFormat;
	try{
		sFormat = (String^)LoadDllMethod("ImageSaver.ImageSaver", "GetExportFormatType", t->Name, args0);
	} catch (Exception^) { return; }

	if      (sFormat == "RASTER") image = ((LayerContainer^)(this->ActiveMdiChild))->geoViewer->ImageNode;
	else if (sFormat == "VECTOR") image = ((LayerContainer^)(this->ActiveMdiChild))->geoViewer->VectorNode();

	if (image != nullptr)
	{
		::DialogResult res = saveFileDialog->ShowDialog();
		if (res == ::DialogResult::Cancel) return;

		bool finished = false;
		int img_ptr = 0, img_size = 0;
		array<Object^>^ args = gcnew array<Object^>(4);
		args[0] = saveFileDialog->FileName;
		args[1] = (IntPtr)(void*)(&img_ptr);
		args[2] = (IntPtr)(void*)(&img_size);
		args[3] = (IntPtr)(void*)image->XML_Tree_Marshal();

		ProcessStatus^ ps = gcnew ProcessStatus(&img_ptr, &img_size, &finished, language_container);
		ps->Text = "Export: " + saveFileDialog->FileName;
		FilterThreadWrapper^ filterThread = gcnew FilterThreadWrapper(&finished, t->Name, "ImageSaver.ImageSaver", "Save_Image", args, language_container);
		Thread^ thread = gcnew Thread(gcnew ThreadStart( filterThread, &FilterThreadWrapper::FilterStart ) );
		thread->Start();
		ps->ShowDialog();
		bool saved = (filterThread->res != nullptr) ? (bool)(filterThread->res) : 0;
		thread->Abort();
		delete thread;
		delete args;

		if (finished && !saved) // Filter aborted or filter could not executed
		{
			MessageBox::Show(language_container->XmlGetChildStringValueByName("WrongExportModule"), language_container->XmlGetChildStringValueByName("ExportError"));
		}
	}
	else
	{
		if      (sFormat == "RASTER") MessageBox::Show(language_container->XmlGetChildStringValueByName("ExportLayerMissRaster"), language_container->XmlGetChildStringValueByName("ExportError"));
		else if (sFormat == "VECTOR") MessageBox::Show(language_container->XmlGetChildStringValueByName("ExportLayerMissVector"), language_container->XmlGetChildStringValueByName("ExportError"));
	}
}



void MainWindow::menuFileCloseWorkspace_Click(System::Object^  sender, System::EventArgs^  e) 
{
	while (MdiChildren->Length != 0) MdiChildren[0]->Close();
}

void MainWindow::menuFileExit_Click(System::Object^  sender, System::EventArgs^  e) 
{
	Application::Exit();
}



//
void MainWindow::menuEditSelectNone_Click(System::Object^  sender, System::EventArgs^  e) 
{
    if (MdiChildren->Length != 0)
    {
		((LayerContainer^)ActiveMdiChild)->geoViewer->ClearSelections();
        ActiveMdiChild->Refresh();
    }
}


void MainWindow::menuEditSelectionToImage_Click(System::Object^  sender, System::EventArgs^  e)
{
    if (this->MdiChildren->Length == 0) return;
	LayerContainer^ lc = (LayerContainer^)(this->ActiveMdiChild);

	if (!lc->geoViewer->ImageDisplayed) return;

	if (lc->geoViewer->SelectionsView->Items->Count != 1)
	{
		MessageBox::Show(language_container->XmlGetChildStringValueByName("SelectionExactlyOneRectangle"), language_container->XmlGetChildStringValueByName("Error"), MessageBoxButtons::OK, MessageBoxIcon::Information);
		return;
	}

	GeoViewer::GeoViewer::SelectionStruct^ ss = (GeoViewer::GeoViewer::SelectionStruct^)(lc->geoViewer->SelectionsView->Items[0]->Tag);

	if (ss->selectiontype != RECTANGLE)
	{
		MessageBox::Show(language_container->XmlGetChildStringValueByName("SelectionExactlyOneRectangle"), language_container->XmlGetChildStringValueByName("Error"), MessageBoxButtons::OK, MessageBoxIcon::Information);
		return;
	}

	System::Drawing::Rectangle r = ss->rectangle;

	xml_node^ img_src = lc->geoViewer->ImageNode;

	Byte* temp = img_src->XML_Tree_Marshal();
	xml_node^ img_dest = gcnew xml_node( temp );
	delete [] temp;

	Byte* data_src;
	int pixelsize, stride_src;
	img_dest->XmlSetChildValueByName("Width", r.Width, false);
	img_dest->XmlSetChildValueByName("Height", r.Height, false);
	if (!img_src->XmlGetChildValueByName("PixelSize", pixelsize))
	{
		delete img_dest;
		return;
	}
	if (!img_src->XmlGetChildValueByName("Data", stride_src))
	{
		delete img_dest;
		return;
	}
	data_src = (Byte*)(void*)stride_src;
	if (!img_src->XmlGetChildValueByName("Stride", stride_src))
	{
		delete img_dest;
		return;
	}


	int stride = (r.Width * pixelsize + 7) / 8; // in bytes
	stride = ((stride + 3) / 4) * 4; // in dword
	img_dest->XmlSetChildValueByName("Stride", stride, false);
	Byte* data = new Byte[stride * r.Height];
	if (!data)
	{
		MessageBox::Show(language_container->XmlGetChildStringValueByName("MemoryAllocationFailed"), language_container->XmlGetChildStringValueByName("Error"), MessageBoxButtons::OK, MessageBoxIcon::Information);
		delete img_dest;
		return;
	}
	img_dest->XmlSetChildValueByName("Data", (int)(void*)data, false);

	int widthbytes = (r.Width * pixelsize + 7) / 8;
	for (int j = 0; j < r.Height; ++j)
	{
		Byte* ptr_in = data_src + (r.Y + j) * stride_src + (r.X * pixelsize / 8);
		Byte* ptr_out = data + stride * j; 
		for (int i = 0; i < widthbytes; ++i) ptr_out[i] = ptr_in[i];
	}

	LayerContainer^ lc2 = gcnew LayerContainer();
	lc2->geoViewer->SetBitmapImage(img_dest);
    AddOwnedForm(lc2);
    lc2->MdiParent = this;
    lc2->Show();
}
//
void MainWindow::menuFilterOverwriteAlwaysAsk_Click(System::Object^  sender, System::EventArgs^  e) 
{
	menuFilterOverwriteAlwaysAsk->Checked = !menuFilterOverwriteAlwaysAsk->Checked;
}

void MainWindow::menuFilterOverwriteIfPossible_Click(System::Object^  sender, System::EventArgs^  e) 
{
	menuFilterOverwriteIfPossible->Checked = !menuFilterOverwriteIfPossible->Checked;
}

void MainWindow::menuFilterRefresh_Click(System::Object^  sender, System::EventArgs^  e)
{
    RefreshFilterList();
}

void MainWindow::menuFilternCore_Click(System::Object^  sender, System::EventArgs^  e)
{
	menuFilternCore->Checked = !menuFilternCore->Checked;
}

void MainWindow::menuFilterFilterExecute_Click(System::Object^  sender, System::EventArgs^  e)
{
	xml_node^ root = gcnew xml_node(nullptr, "", "", "");
    if (this->MdiChildren->Length == 0) return;
    ToolStripMenuItem^ t = (ToolStripMenuItem^)sender;
	Filter_Execute(t->Name, t->Text);
}
//
void MainWindow::Filter_Execute(String^ dll_name, String^ filter_name)
{
	int ic = 0, oc = 0, cnt = 0;

	if (MdiChildren->Length == 0) return;
	LayerContainer^ lc = (LayerContainer^)ActiveMdiChild;
	layers_for_filter.Clear();

	// DO WE HAVE OVERWRITE SUPPORT?
	System::Windows::Forms::DialogResult res = ::DialogResult::No;
	array<Object^>^ args = gcnew array<Object^>(0);
	bool canlocalize = (bool)LoadDllMethod("FilterSpace.ImageFilter", "CanLocalize", dll_name, args);

    if (canlocalize)
    {
        if (menuFilterOverwriteAlwaysAsk->Checked)
			res = MessageBox::Show(language_container->XmlGetChildStringValueByName("OverwriteSupport"), filter_name + " " + language_container->XmlGetChildStringValueByName("ExecutionMode"), MessageBoxButtons::YesNoCancel, MessageBoxIcon::Question, MessageBoxDefaultButton::Button1 );
		else if (menuFilterOverwriteIfPossible->Checked)
			res = ::DialogResult::Yes;
    }
	if (res == ::DialogResult::Cancel) return;


	// GET SUPPORT SETTINGS ---->
	IntPtr^ ip;
	try{
		ip = (IntPtr^)LoadDllMethod("FilterSpace.ImageFilter", "GetSupportedFormats", dll_name, args);
	} catch (Exception^) { return; }

	if (ip->ToPointer() == 0) return;

	Byte* f = (Byte*)ip->ToPointer();
	xml_node^ iroot = gcnew xml_node( f );
	delete [] f;

	iroot->XmlGetChildValueByName("InputImageCount",  ic);
	iroot->XmlGetChildValueByName("OutputImageCount", oc);

	// RUNNING MATCH-MAKER ---->
	bool matchmake = Filter_SearchForInputs( iroot );
	if (!matchmake) 
	{
		delete iroot;
		return;
	}

	// ALLOC INPUTS RETRIEVED FROM MATCH-MAKING ---->
	xml_node^ root = gcnew xml_node(nullptr, "", "", "");
	root->XmlSetChildValueByName("InputImageCount",  ic, false);
	root->XmlSetChildValueByName("OutputImageCount", oc, false);
	root->XmlSetChildValueByName("Run_Local",        ((res == ::DialogResult::Yes) ? 1 : 0), false);

	try
	{
		Filter_AllocPointers( root, iroot );
		delete iroot; // INPUT CRITERIAS
	}
	catch (Exception^)
	{
		delete root;
		delete iroot;
		return;
	}

	// CREATING OUTPUT SECTION
	xml_node^ xntmp;
	if (!root->XmlGetChildByName( xntmp, "InputImage0" ))
	{
		delete root;
		return;
	}

	// CREATING MASK, IF NEEDED + call: CreateImageSettings
	if (!Filter_CreateMask( root, dll_name, lc->geoViewer->ImageWidth, lc->geoViewer->ImageHeight, lc ))
	{
		delete root; // added 2010.04.17.,21:48
		return;
	}

	if (!root->XmlGetChildByName(xntmp, "OutputImage0"))
	{
		if (debug_mode) MessageBox::Show("OutputImage section not found in XML Tree after CreateImageSettings(...)", "Error");
		delete root;
		return;
	}

	// ---
	if (debug_mode) 
	{
		XML_Editor^ xe2 = gcnew XML_Editor();
		xe2->Text = "Filter_CreateMask after";
		xe2->CreateXMLTree( root );
		xe2->ShowDialog();
	}
	// ---

	// EXECUTING FILTER BY REMOTE METHOD INVOCATION
	// *******************
	bool result = false;
	bool finished = false;
	int data_ptr = 0, data_size = 0, nCoreKillSignal = 0;
	int nCore = (menuFilternCore->Checked) ? 1 : 0;

	root->XmlSetChildValueByName("Processing_Ptr",   ((int)(void*)(&data_ptr)),              false);
	root->XmlSetChildValueByName("Processing_Size",  ((int)(void*)(&data_size)),             false);
	root->XmlSetChildValueByName("nCore!Enabled",    nCore, false);
	root->XmlSetChildValueByName("nCore!KillSignal", ((int)(void*)(&nCoreKillSignal)),       false);

	args->Resize(args, 1);
	args[0] = (IntPtr)(void*)root->XML_Tree_Marshal();
	ProcessStatus^ ps = gcnew ProcessStatus(&data_ptr, &data_size, &finished, language_container);
	ps->Text = filter_name;
	ps->LabelVersionLogo->Visible = menuFilternCore->Checked;
	FilterThreadWrapper^ filterThread = gcnew FilterThreadWrapper(&finished, dll_name, "FilterSpace.ImageFilter", "ExecuteFilter", args, language_container);
	Thread^ thread = gcnew Thread(gcnew ThreadStart( filterThread, &FilterThreadWrapper::FilterStart ) );
	thread->Start();
	ps->ShowDialog();
	result = (filterThread->res != nullptr) ? (bool)filterThread->res : false;

	//MessageBox::Show("result = " + result.ToString() + "\nfinished = " + finished.ToString());

	if (!finished)
	{
		nCoreKillSignal = 1;
		for (int countdown = 0; countdown < 40 && nCoreKillSignal > 0; countdown++) Thread::Sleep(100);
		if (nCoreKillSignal > 0) MessageBox::Show(language_container->XmlGetChildStringValueByName("nCoreKillError"), "", MessageBoxButtons::OK, MessageBoxIcon::Warning);
	}
	thread->Abort();
	delete thread;

	// *******************

	Filter_DestroyImageMask( root );

	delete [] (Byte*)((void*)(IntPtr)args[0]); // Marhalled root xml_tree

	// ****

	// COLLECT PREVIOUSLY ALREADY EXISTING BUFFERS
	xml_node^ xt;
	int *ibufs = new int[ic];
	int *obufs = new int[oc];

	for (int i = 0; i < ic; i++)
	{
		xt = nullptr;
		root->XmlGetChildByName(xt, "InputImage" + i.ToString());
		if (xt != nullptr)
		{
			if (!xt->XmlGetChildValueByName("Data", ibufs[cnt++])) cnt--;
		}
	}
	ic = cnt;

	// CHECK OUTPUT BUFFERS, WHETHER THEY ARE NEW OR NOT
	for (int i = 0; i < oc; i++)
	{
		bool found = false;
		int  data  = 0;
		xt         = nullptr;
		obufs[i]   = 0;
		// GET "OutputImageX" SUBTREE
		root->XmlGetChildByName(xt, "OutputImage" + i.ToString());

		if (xt != nullptr)
		{
			if (!xt->XmlGetChildValueByName("Data", data)) continue; // DATA FIELD IS NOT EXISTING
			if (data == 0) continue; // INCORRECT DATA
		}
		else
		{
			continue; // SUBTREE NOT FOUND
		}

		// EXISTING BUFFER?
		for (int j = 0; j < ic && !found; j++) found = (data == ibufs[j]);

		// IF BUFFER IS NEW -> DELETE!
		if (!found) obufs[i] = data;
	}

	delete [] ibufs;

	// ****

	if (!finished || (finished && !result)) // Filter aborted or filter could not executed
	{
		// FREE UP NEWLY ALLOCATED BUFFERS (BY FILTER)
		for (int i = 0; i < oc; i++) if (obufs[i]) delete [] ((Byte*)(void*)(obufs[i]));
		delete [] obufs;
		delete root;
		return;
	}


	// ---
	if (debug_mode) 
	{
		XML_Editor^ xe = gcnew XML_Editor();
		xe->CreateXMLTree( root );
		xe->ShowDialog();
	}
	// ---

	// CREATE OUTPUT FOR NEW BUFFERS
	for (int i = 0; i < oc; i++) 
	{
		xt = nullptr;
		root->XmlGetChildByName(xt, "OutputImage" + i.ToString());
		if (xt == nullptr) continue;

		if ((obufs[i] != 0) || (!xt->XmlIsChildExist("Width")))
		{
			LayerContainer^ lc2 = gcnew LayerContainer();
			if (obufs[i]) 
				lc2->geoViewer->SetBitmapImage(xt);
			else
				lc2->geoViewer->SetVectorImage(xt);
			AddOwnedForm(lc2);

			String^ pinname = "";
			xt->XmlGetChildValueByName("PinName", pinname);
			if (pinname != "") lc2->Text = pinname;

			lc2->MdiParent = this;
			lc2->Show();		
		}
	}

	lc->geoViewer->RebindBitmap();

	delete [] obufs;
	delete root;
	layers_for_filter.Clear();
}

bool MainWindow::Filter_SearchForInputs(xml_node^ root)
{
	LayerContainer^ lc = (LayerContainer^)ActiveMdiChild;

	bool matchmaking_success = false;
	int imagecount;
	xml_node^ xntmp;
	if (!root->XmlGetChildValueByName("InputImageCount", imagecount)) return false;
	if ( imagecount < 1 ) return false;

	if ( imagecount != 1 )
	{
		InputsFinder^ ifinder = gcnew InputsFinder(this, root);
		ifinder->Text = language_container->XmlGetChildStringValueByName("SourceSettings");
		ifinder->ShowDialog();
		matchmaking_success = (layers_for_filter.Count == imagecount);
	}
	else
	{
		if (!root->XmlGetChildByName(xntmp, "InputImage0")) return false;
		matchmaking_success = InputsFinder::IsLayerCompatible( lc, xntmp, debug_mode );

		if (!matchmaking_success)
		{
			MessageBox::Show(language_container->XmlGetChildStringValueByName("CurrentImageNotCompatible"), "");
			InputsFinder^ ifinder = gcnew InputsFinder(this, root);
			ifinder->Text = language_container->XmlGetChildStringValueByName("SourceSettings");
			ifinder->ShowDialog();
			matchmaking_success = (layers_for_filter.Count == imagecount);
		}
		else
		{
			layers_for_filter.Add( lc ); // GUARANTEE SAME BEHAVIOUR
		}
	}
	return matchmaking_success;
}



void MainWindow::Filter_AllocPointers(xml_node^ root, xml_node^ iroot)
{
	xml_node^ xntmp;
	// Allocate Inputs
	for (int i = 0; i < layers_for_filter.Count; ++i)
	{
		Byte* mrs;
		iroot->XmlGetChildByName(xntmp, "InputImage" + i.ToString());

		if (xntmp->XmlIsChildExist("Width"))
		{
			if (debug_mode) MessageBox::Show("Allocating " + i.ToString() + ". raster layer...");
			mrs = layers_for_filter[i]->geoViewer->ImageNode->XML_Tree_Marshal();
		}
		else
		{
			if (debug_mode) MessageBox::Show("Allocating " + i.ToString() + ". vector layer...");
			mrs = layers_for_filter[i]->geoViewer->VectorNode()->XML_Tree_Marshal();
		}

		xntmp = gcnew xml_node( mrs );
		delete mrs;
		xntmp->parent = root;
		xntmp->name = "InputImage" + i.ToString();
		xntmp->value = "";
		xntmp->XmlSetChildValueByName("PinName", layers_for_filter[i]->Text, false);
		root->children.Add( xntmp );
	}
}
bool MainWindow::Filter_CreateMask(xml_node^& filtersetting, String^ dll, int Width, int Height, LayerContainer^& lc)
{
    IntPtr ip = IntPtr(0);

	int count = lc->geoViewer->SelectionsView->Items->Count;

    if (count != 0)
    {
		ip = ImageFilter::CreateImageMask(Width, Height);
        
        for (int i = 0; i < count; ++i)
        {
			GeoViewer::GeoViewer::SelectionStruct^ s = (GeoViewer::GeoViewer::SelectionStruct^)lc->geoViewer->SelectionsView->Items[i]->Tag;
			if (s->selectiontype == RECTANGLE)
            {
				ImageFilter::AddMaskRectangle(ip, Width, s->rectangle.X, s->rectangle.Y, s->rectangle.Width, s->rectangle.Height);
            }
			else if (s->selectiontype == POLIGON)
            {
                int cnt = s->polygonpoints.Count * 2;
				int* pts = new int[cnt];
                for (int j = 0; j < cnt / 2; ++j)
                {
                    pts[j * 2] = s->polygonpoints[j].X;
                    pts[j * 2 + 1] = s->polygonpoints[j].Y;
                }
                IntPtr ip2 = (IntPtr)(void*)(pts);
				ImageFilter::AddMaskPolygon(ip, Width, Height, s->polygonpoints.Count, ip2);
				delete [] pts;
            }
        }
    }

	xml_node^ mask = gcnew xml_node(filtersetting, "Mask", "", "");
	mask->children.Add( gcnew xml_node( mask, "Data", "", "" ) );
	mask->XmlSetChildValueByName("Data", (int)(void*)ip);
	filtersetting->children.Add( mask );

    array<Object^>^ args = gcnew array<Object^>(2);
	args[0] = (IntPtr)(void*)(filtersetting->XML_Tree_Marshal());
    args[1] = nullptr; // silent mode  (nullptr == Boxed(false))
	IntPtr^ ip3 = (IntPtr)LoadDllMethod("FilterSpace.ImageFilter", "CreateFilterSettings", dll, args);
	Byte* valami = (Byte*)ip3->ToPointer();
	if (valami == 0)
	{
		delete [] (Byte*)((void*)(IntPtr)args[0]);
		if (debug_mode) MessageBox::Show("GEO : CreateFilterSettings() failed");
		return false;
	}
	delete [] (Byte*)((void*)(IntPtr)args[0]);
	filtersetting = gcnew xml_node( valami );

	// ---
	if (debug_mode) 
	{
		XML_Editor^ xe2 = gcnew XML_Editor();
		xe2->Text = "Filter_CreateMask";
		xe2->CreateXMLTree( filtersetting );
		xe2->ShowDialog();
	}
	// ---

	delete [] valami;

	return (valami != 0);
}
void MainWindow::Filter_DestroyImageMask(xml_node^ root)
{
	int data;
	xml_node^ xntmp;
	if (!root->XmlGetChildByName(xntmp, "Mask")) return;
	if (!xntmp->XmlGetChildValueByName("Data", data)) return;
	if (data) delete [] ((Byte*)(void*)data);
}
//

void MainWindow::menuViewWorkflow_Click(System::Object^  sender, System::EventArgs^  e)
{
	if (workflowmanager_exist)
    {
		workflowmanager->Visible = !menuViewWorkflow->Checked;
		menuViewWorkflow->Checked = workflowmanager->Visible;
    }
    else
    {
		workflowmanager = gcnew WorkflowManager(this);
		workflowmanager_exist = true;
		menuViewWorkflow->Checked = true;
        workflowmanager->Visible = true;
    }
}

void MainWindow::menuViewToolbox_Click(System::Object^  sender, System::EventArgs^  e) 
{
	if (toolbox_exist)
    {
        toolbox->Visible = !menuViewToolbox->Checked;
        menuViewToolbox->Checked = toolbox->Visible;
    }
    else
    {
		toolbox = gcnew Toolbox(this);
		toolbox_exist = true;
		menuViewToolbox->Checked = true;
        toolbox->Visible = true;
    }
}

void MainWindow::menuViewSelections_Click(System::Object^  sender, System::EventArgs^  e) 
{
    if (this->MdiChildren->Length == 0) return;
    menuViewSelections->Checked = !menuViewSelections->Checked;
    LayerContainer^ lc = (LayerContainer^)this->ActiveMdiChild;
    lc->geoViewer->SetSelectionsVisibility(menuViewSelections->Checked);
}

void MainWindow::menuViewScrollBars_Click(System::Object^  sender, System::EventArgs^  e) 
{
    if (this->MdiChildren->Length == 0) return;
    menuViewScrollBars->Checked = !menuViewScrollBars->Checked;
    LayerContainer^ lc = (LayerContainer^)this->ActiveMdiChild;
    lc->geoViewer->SetScrollVisibility(menuViewScrollBars->Checked);
}



void MainWindow::menuViewDeveloperMode_Click(System::Object^  sender, System::EventArgs^  e)
{
	menuViewDeveloperMode->Checked = !menuViewDeveloperMode->Checked;
	debug_mode = menuViewDeveloperMode->Checked;
}



void MainWindow::menuViewImageInformation_Click(System::Object^  sender, System::EventArgs^  e)
{
    if (this->MdiChildren->Length == 0) return;
    LayerContainer^ lc = (LayerContainer^)this->ActiveMdiChild;
	XML_Editor^ xe = gcnew XML_Editor();
	xe->CreateXMLTree( lc->geoViewer->ImageNode );
	xe->Show();
}
//
void MainWindow::menuLanguage_Click(System::Object^  sender, System::EventArgs^  e)
{
	delete language_container;
	language_container = gcnew xml_node(nullptr, "", "", "");

	if (sender != nullptr)
	{
		ToolStripMenuItem^ t = (ToolStripMenuItem^)sender;
		Encoding^ enc = Encoding::Default;
		StreamReader^ sr = gcnew StreamReader(t->Name, enc);
		xml_parser::read_xml_into_xmlsubtree(sr, language_container);
		delete sr;
	}

	String^ s;

	// SKIN SECTION
	language_container->XmlSetChildValueByName("MenuFile", "Fjl", true);
	language_container->XmlSetChildValueByName("MenuFileOpenGEO", "Megnyits (GEO API)", true);
	language_container->XmlSetChildValueByName("MenuFileSave", "Ments", true);
	language_container->XmlSetChildValueByName("MenuFileCloseWorksheet", "Munka bezrsa", true);
	language_container->XmlSetChildValueByName("MenuFileExit", "Kilps", true);

	language_container->XmlSetChildValueByName("MenuEdit", "Szerkeszts", true);
	language_container->XmlSetChildValueByName("MenuEditClearSelection", "Kijellsek trlse", true);
	language_container->XmlSetChildValueByName("MenuEditSelectionToImage", "Kijells --> Kp", true);

	language_container->XmlSetChildValueByName("MenuFilters", "Szrk", true);
	language_container->XmlSetChildValueByName("MenuFiltersOverwriteMode", "Fellrsi md", true);
	language_container->XmlSetChildValueByName("MenuFiltersOverwriteModeAlwaysAsk", "Mindig rkrdez", true);
	language_container->XmlSetChildValueByName("MenuFiltersOverwriteModeOverWriteIfPossible", "Fellr, ha lehetsges", true);
	language_container->XmlSetChildValueByName("MenuFiltersRefreshList", "Szrlista frisstse", true);

	language_container->XmlSetChildValueByName("MenuView", "Nzet", true);
	language_container->XmlSetChildValueByName("MenuViewWorkflowManager", "Workflow Menedzser", true);
	language_container->XmlSetChildValueByName("MenuViewToolbox", "Eszkztr", true);
	language_container->XmlSetChildValueByName("MenuViewSelections", "Kijellsek", true);
	language_container->XmlSetChildValueByName("MenuViewScrollBars", "Grdtsvok", true);
	language_container->XmlSetChildValueByName("MenuViewDeveloperMode", "Fejleszti zemmd", true);
	language_container->XmlSetChildValueByName("MenuViewImageInformation", "Kp informci", true);

	language_container->XmlSetChildValueByName("MenuLanguage", "Nyelv", true);

	// RE-SKIN SYSTEM GUI
	language_container->XmlGetChildValueByName("MenuFile", s);
	menuFile->Text = s;
	language_container->XmlGetChildValueByName("MenuFileOpenGEO", s);
	menuFileOpenImage->Text = s;
	language_container->XmlGetChildValueByName("MenuFileSave", s);
	menuFileSave->Text = s;
	language_container->XmlGetChildValueByName("MenuFileCloseWorksheet", s);
	menuFileCloseWorkspace->Text = s;
	language_container->XmlGetChildValueByName("MenuFileExit", s);
	menuFileExit->Text = s;

	language_container->XmlGetChildValueByName("MenuEdit", s);
	menuEdit->Text = s;
	language_container->XmlGetChildValueByName("MenuEditClearSelection", s);
	menuEditSelectNone->Text = s;
	language_container->XmlGetChildValueByName("MenuEditSelectionToImage", s);
	menuEditSelectionToImage->Text = s;

	language_container->XmlGetChildValueByName("MenuFilters", s);
	menuFilter->Text = s;
	language_container->XmlGetChildValueByName("MenuFiltersOverwriteMode", s);
	menuFilterOverWriteToolStripMenuItem->Text = s;
	language_container->XmlGetChildValueByName("MenuFiltersOverwriteModeAlwaysAsk", s);
	menuFilterOverwriteAlwaysAsk->Text = s;
	language_container->XmlGetChildValueByName("MenuFiltersOverwriteModeOverWriteIfPossible", s);
	menuFilterOverwriteIfPossible->Text = s;
	language_container->XmlGetChildValueByName("MenuFiltersRefreshList", s);
	menuFilterRefresh->Text = s;

	language_container->XmlGetChildValueByName("MenuView", s);
	menuView->Text = s;
	language_container->XmlGetChildValueByName("MenuViewWorkflowManager", s);
	menuViewWorkflow->Text = s;
	language_container->XmlGetChildValueByName("MenuViewToolbox", s);
	menuViewToolbox->Text = s;
	language_container->XmlGetChildValueByName("MenuViewSelections", s);
	menuViewSelections->Text = s;
	language_container->XmlGetChildValueByName("MenuViewScrollBars", s);
	menuViewScrollBars->Text = s;
	language_container->XmlGetChildValueByName("MenuViewDeveloperMode", s);
	menuViewDeveloperMode->Text = s;
	language_container->XmlGetChildValueByName("MenuViewImageInformation", s);
	menuViewImageInformation->Text = s;

	language_container->XmlGetChildValueByName("MenuLanguage", s);
	menuLanguage->Text = s;

	// DIALOG TEXTS --------------------------------->
	
	// LoadDllMethod:
	language_container->XmlSetChildValueByName("MethodNotFound", "A metdus nem tallhat", true);
	language_container->XmlSetChildValueByName("ClassNotFound", "Az ImageFilter osztly nem tallhat a szrben", true);
	language_container->XmlSetChildValueByName("nCoreKillError", "A szr nem vlaszol a megszaktsra vagy hibsan mkdik.\nElkpzelhet, hogy a program emiatt memria szemetet fog hagyni, vagy hibsan fog mkdni.", true);

	// menuFileOpenImage_Click: 
	language_container->XmlSetChildValueByName("ImageFiles", "Kp fjlok", true);
	language_container->XmlSetChildValueByName("Files", "fjlok", true);
	language_container->XmlSetChildValueByName("Open", "Megnyits", true);
	language_container->XmlSetChildValueByName("NoExportModules", "Nincsenek elrhet export modulok", true);
	language_container->XmlSetChildValueByName("NoImportModules", "Nincs elrhet import modul az adott fjltpushoz!\nEllenrzze a teleptst.", true);
	language_container->XmlSetChildValueByName("WrongImportModule", "A fjltpushoz tartoz import modul hibs.\n\n", true);
	language_container->XmlSetChildValueByName("ImportError", "Import hiba", true);

	// menuFileSaveImage_Click: 
	language_container->XmlSetChildValueByName("WrongExportModule", "A fjltpushoz tartoz export modult nem sikerlt hasznlni.\n\n", true);
	language_container->XmlSetChildValueByName("ExportError", "Export hiba", true);
	language_container->XmlSetChildValueByName("ExportLayerMissVector", "Az aktulis ablak nem tartalmaz vektor rteget.", true);
	language_container->XmlSetChildValueByName("ExportLayerMissRaster", "Az aktulis ablak nem tartalmaz raszter rteget.", true);

	// menuEditSelectionToImage_Click: 
	language_container->XmlSetChildValueByName("Error", "Hiba", true);
	language_container->XmlSetChildValueByName("SelectionExactlyOneRectangle", "Pontosan egy tglalap kijellst ksztsen.", true);
	language_container->XmlSetChildValueByName("MemoryAllocationFailed", "A memriafoglals sikertelen volt.", true);

	// Filter_Execute: 
	language_container->XmlSetChildValueByName("OverwriteSupport", "Ez a szr tmogatja a fellrst.\n\nEz gyorsthatja a vgrehajtst s memrit takart meg. \nAz egyszer vgrehajts az eredmnyt j ablakban jelenti meg s tbblet memrira van szksge. \n\nEngedlyezi a fellrst?", true);
	language_container->XmlSetChildValueByName("ExecutionMode", "vgrehajtsi mdja", true);

	// Filter_SearchForInputs: 
	language_container->XmlSetChildValueByName("SourceSettings", "Forrsok belltsa", true);
	language_container->XmlSetChildValueByName("CurrentImageNotCompatible", "Az aktulis kp nem felelt meg a szr alkalmazshoz.\nA tovbbiakban megprblhat msik kpet megjellni forrsknt.", true);

	// ProcessStatus Dialog:
	language_container->XmlSetChildValueByName("AbortButtonText", "Megszakts", true);
	language_container->XmlSetChildValueByName("RemainTimeLabel", "Htralv id: {0}:{1}", true);

	// ----------------------------------------------->
	// Workflow Manager language section

	language_container->XmlSetChildValueByName("WorkflowManager", "Workflow Menedzser", true);
	language_container->XmlSetChildValueByName("WorkflowManagerQuit", "Ha bezrja a workflow menedzsert, akkor elveszti a workflowt.\nMegtarthatja a munkjt, ha elrejti az ablakot az albbi mdon: Geo FilterBank -> View -> Workflow Menedzser.\n\nBiztosan bezrja ezt az ablakot?", true);
	language_container->XmlSetChildValueByName("WorkflowMenuFile", "Fjl", true);
	language_container->XmlSetChildValueByName("WorkflowMenuFileOpen", "Megnyits", true);
	language_container->XmlSetChildValueByName("WorkflowMenuFileSave", "Ments", true);
	language_container->XmlSetChildValueByName("WorkflowMenuFileSaveAs", "Ments msknt", true);
	language_container->XmlSetChildValueByName("WorkflowMenuFileClose", "Bezrs", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowNewComponent", "j komponens", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowNewComponentImport", "Import", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowNewComponentExport", "Export", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowNewComponentFilter", "Szr", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowInteractiveMode", "Interaktv md", true);
	language_container->XmlSetChildValueByName("WorkflowMenuWorkflowShowXMLMetainfos", "Mutassa az XML meta adatokat", true);

	// ----------------------------------------------->
	// GeoViewer language section

	language_container->XmlSetChildValueByName("GeoViewerRemoveSelection", "Kijells trlse", true);
	language_container->XmlSetChildValueByName("GeoViewerSelection", "Kijells: {0},{1} x {2},{3}", true);
	language_container->XmlSetChildValueByName("GeoViewerPosition", "Pozci: {0},{1}", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingTitle", "Csatorna szne", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingOriginal", "Eredeti", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingGrayscale", "Szrke rnyalatos", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingRed", "Vrs", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingGreen", "Zld", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingBlue", "Kk", true);
	language_container->XmlSetChildValueByName("GeoViewerColormappingCustom", "Egyni", true);

	RefreshImageSaverList();

	
	for (int i = 0; i < MdiChildren->Length; i++) ((LayerContainer^)MdiChildren[i])->geoViewer->SetInterfaceLanguage(language_container);
}