
#include "stdafx.h"


using namespace System;
using namespace System::Drawing;
using namespace System::Windows::Forms;
using namespace GeoFilterBank;
using namespace std;


WorkflowManager::WorkflowManager(MainWindow^ mainwnd)
{
	mouse_button_down = false;
	mw = mainwnd;
	edge_construct = false;
	WorkflowFile = "";
	prev_scroll_x = 0;
	prev_scroll_y = 0;

	InitializeComponent();
	WorkflowManager_Resize(nullptr, nullptr);

	String^ s;

	mw->language_container->XmlGetChildValueByName("WorkflowManager", s);
	this->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuFile", s);
	menuFile->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuFileOpen", s);
	menuFileOpen->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuFileSave", s);
	menuFileSave->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuFileSaveAs", s);
	menuFileSaveAs->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuFileClose", s);
	menuFileClose->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowNewComponent", s);
	menuAddComponent->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowNewComponentImport", s);
	menuAddImport->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowNewComponentExport", s);
	menuAddExport->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowNewComponentFilter", s);
	menuAddFilter->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowInteractiveMode", s);
	menuWorkflowInteractiveMode->Text = s;
	mw->language_container->XmlGetChildValueByName("WorkflowMenuWorkflowShowXMLMetainfos", s);
	menuWorkflowShowXMLMetainfos->Text = s;

	menuFileOpen->Image = menuImageList->Images[0];
	menuFileSave->Image = menuImageList->Images[1];
	menuFileSaveAs->Image = menuImageList->Images[1];
	menuFileClose->Image = menuImageList->Images[3];
	menuAddImport->Image = menuImageList->Images[0];
	menuAddExport->Image = menuImageList->Images[1];
	menuAddFilter->Image = menuImageList->Images[2];
	menuAddComponent->Image = menuImageList->Images[4];
	menuWorkflowStart->Image = menuImageList->Images[5];
	menuAddSplitter->Image = menuImageList->Images[6];

	int last, c = mw->menuFilter->DropDownItems->Count;
	for (int i = 4; i < c; i++)
	{
		menuAddFilter->DropDownItems->Add("");
		ToolStripMenuItem^ mainfromitem = (ToolStripMenuItem^)(mw->menuFilter->DropDownItems[i]);
		ToolStripMenuItem^ mainitem     = (ToolStripMenuItem^)(menuAddFilter->DropDownItems[menuAddFilter->DropDownItems->Count - 1]);
		mainitem->Text = mainfromitem->Text;
		mainitem->Name = mainfromitem->Name;

		// GROUPING SUPPORT EXTENSION - 2010 APR 25
		int cnt = ((ToolStripMenuItem^)(mw->menuFilter->DropDownItems[i]))->DropDownItems->Count;
		if (cnt > 0)
		{
			int j = 0;
			for each (ToolStripItem^ ti in ((ToolStripMenuItem^)mw->menuFilter->DropDownItems[i])->DropDownItems)
			{
				mainitem->DropDownItems->Add(ti->Text);
				mainitem->DropDownItems[j]->Click += gcnew System::EventHandler(this,  &WorkflowManager::menuAddFilter_Click);
				mainitem->DropDownItems[j]->Name   = ti->Name;
				mainitem->DropDownItems[j]->Image  = menuImageList->Images[2];
				j++;
			}
		}
		else
		{
			mainitem->Image = menuImageList->Images[2];
			mainitem->Click += gcnew System::EventHandler(this,  &WorkflowManager::menuAddFilter_Click);
		}
	}

	c = (mw->menuFileSave->DropDownItems[0]->Enabled) ? mw->menuFileSave->DropDownItems->Count : 0;
	for (int i = 0; i < c; i++)
	{
		menuAddExport->DropDownItems->Add(mw->menuFileSave->DropDownItems[i]->Text);
		last = menuAddExport->DropDownItems->Count - 1;
		menuAddExport->DropDownItems[last]->Image = menuImageList->Images[1];
		menuAddExport->DropDownItems[last]->Name = mw->menuFileSave->DropDownItems[i]->Name;
		menuAddExport->DropDownItems[last]->Click += gcnew System::EventHandler(this,  &WorkflowManager::menuAddExport_Click);
	}


	//
	//TODO: Add the constructor code here
	//
	nodeList = gcnew List<GeoFilterBank::GraphNode^>;
	edgeList = gcnew List<GeoFilterBank::GraphEdge^>;
}

bool WorkflowManager::IsComplete()
{
	int c = nodeList->Count;
	for (int i = 0; i < c; i++) if (!nodeList[i]->IsComplete()) return false;
	return true;
}

int* WorkflowManager::DetermineExecutionOrder()
{
	std::set<int>  S;
	int  i, j, k;
	int  Vcount     = nodeList->Count;
	int  Ecount     = edgeList->Count;
	int *order      = new int[Vcount];
	bool has_edges  = false;
	dag_node *nodes = new dag_node[Vcount];

	// create our node list
	for (i = 0; i < Vcount; i++)
	{
		GraphNode^ gn = nodeList[i];
		for (j = 0; j < Ecount; j++)
			if (edgeList[j]->source == gn)
				for (k = 0; k < Vcount; k++)
					if (nodeList[k] == edgeList[j]->target)
					{
						nodes[i].neighbours.insert(k);
						break;
					}
	}

	for (i = 0; i < Vcount; i++)
		for (std::set<int>::iterator it = nodes[i].neighbours.begin(); it != nodes[i].neighbours.end(); *it++)
			nodes[*it].incoming++;

	for (i = 0; i < Vcount; i++) 
		if (nodes[i].incoming == 0)
			S.insert(i);

	k = 0;
	while (S.size() > 0)
	{
		S.erase(i = *S.begin());
		order[k++] = i;
		while (nodes[i].neighbours.size() > 0)
		{
			j = *(nodes[i].neighbours.begin());
			nodes[i].neighbours.erase(j);
			nodes[j].incoming--;
			if (nodes[j].incoming == 0) S.insert(j);
		}
	}

	for (i = 0; i < Vcount && !has_edges; i++) has_edges = (nodes[i].incoming > 0);
	delete [] nodes;

	if (has_edges) 
	{
		delete [] order;
		order = 0;
	}

	return order;
}




void WorkflowManager::WorkflowManager_Paint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) 
{
	Graphics^ g = e->Graphics;
	if (edge_construct)
	{
		Pen^ p = gcnew Pen(Color::Gold, 2);

		g->DrawLine(p, m_x, m_y, (m_x + x) / 2, m_y);
		g->DrawLine(p, (m_x + x) / 2, m_y, (m_x + x) / 2, y);
		g->DrawLine(p, (m_x + x) / 2, y, x, y);

		delete p;
	}

	Pen^ p = gcnew Pen(Color::LightSteelBlue, 2);
	int c = edgeList->Count;
	for (int i = 0; i < c; i++) 
	{
		int sx = 0, sy = 0, dx = 0, dy = 0;
		edgeList[i]->source->GetAbsolutePosOfEdge(edgeList[i]->source_pin, sx, sy);
		edgeList[i]->target->GetAbsolutePosOfEdge(edgeList[i]->target_pin, dx, dy);

		g->DrawLine(p, sx, sy, (sx + dx) / 2, sy);
		g->DrawLine(p, (sx + dx) / 2, sy, (sx + dx) / 2, dy);
		g->DrawLine(p, (sx + dx) / 2, dy, dx, dy);
	}
	delete p;
}

void WorkflowManager::WorkflowManager_Resize(System::Object^  sender, System::EventArgs^  e)
{
	WorkflowScrollBarH->Left = 0;
	WorkflowScrollBarH->Top = this->ClientSize.Height - WorkflowScrollBarH->Height;
	WorkflowScrollBarH->Width = this->ClientSize.Width - WorkflowScrollBarV->Width;
	WorkflowScrollBarV->Left = this->ClientSize.Width - WorkflowScrollBarV->Width;
	WorkflowScrollBarV->Top = 0;
	WorkflowScrollBarV->Height = this->ClientSize.Height - WorkflowScrollBarH->Height;
}


void WorkflowManager::WorkflowScrollEvent(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e)
{
	for (int i = 0; i < nodeList->Count; i++)
	{
		nodeList[i]->Left -= WorkflowScrollBarH->Value - prev_scroll_x;
		nodeList[i]->Top  -= WorkflowScrollBarV->Value - prev_scroll_y;
	}

	prev_scroll_x = WorkflowScrollBarH->Value;
	prev_scroll_y = WorkflowScrollBarV->Value;

	this->Refresh();
}


void WorkflowManager::WorkflowManager_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e)
{
	prev_mouse_x = e->X;
	prev_mouse_y = e->Y;
	mouse_button_down = true;
}

void WorkflowManager::WorkflowManager_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e)
{
	if (!mouse_button_down) return;

	int x = e->X;
	int y = e->Y;
	int h_val = WorkflowScrollBarH->Value - x + prev_mouse_x;
	int v_val = WorkflowScrollBarV->Value - y + prev_mouse_y;
	WorkflowScrollBarH->Value = (h_val < 0) ? 0 : ((h_val > WorkflowScrollBarH->Maximum) ? WorkflowScrollBarH->Maximum : h_val);
	WorkflowScrollBarV->Value = (v_val < 0) ? 0 : ((v_val > WorkflowScrollBarV->Maximum) ? WorkflowScrollBarV->Maximum : v_val);
	prev_mouse_x = x;
	prev_mouse_y = y;
	WorkflowScrollEvent(nullptr, nullptr);
}

void WorkflowManager::WorkflowManager_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e)
{
	mouse_button_down = false;
}




void WorkflowManager::menuFileNew_Click(System::Object^  sender, System::EventArgs^  e)
{
	if (nodeList->Count || WorkflowFile != "")
	{
		System::Windows::Forms::DialogResult res = MessageBox::Show("Menti a vltozsokat?", "j workflow", System::Windows::Forms::MessageBoxButtons::YesNoCancel);

		if (res == System::Windows::Forms::DialogResult::Cancel) return;
		if (res == System::Windows::Forms::DialogResult::Yes)    menuFileSave_Click(nullptr, nullptr);

		for (int i = 0; i < edgeList->Count; i++) delete edgeList[i];
		for (int i = 0; i < nodeList->Count; i++) delete nodeList[i];

		edgeList->Clear();
		nodeList->Clear();
	}

	WorkflowFile = "";
	WorkflowScrollBarH->Value = 0;
	WorkflowScrollBarV->Value = 0;
	prev_scroll_x = prev_scroll_y = 0;
}

void WorkflowManager::menuFileOpen_Click(System::Object^  sender, System::EventArgs^  e)
{
	menuFileNew_Click(sender, e);

	OpenFileDialog^ sd = gcnew OpenFileDialog();
	sd->Title = "Workflow megnyitsa";
	sd->Filter = "Workflow (*.workflow)|*.workflow";

	if (sd->ShowDialog() == Windows::Forms::DialogResult::Cancel)
	{
		delete sd;
		return;
	}
	
	Encoding^     enc = Encoding::Default;
	StreamReader^ sr  = gcnew StreamReader(sd->FileName, enc);
	xml_node^     xn  = gcnew xml_node(nullptr, "", "", "");
	xml_parser::read_xml_into_xmlsubtree(sr, xn);

	if (false)
	{
		XML_Editor^ xe = gcnew XML_Editor();
		xe->CreateXMLTree(xn);
		xe->ShowDialog();
		delete xe;
	}

	bool skip = (xn->children.Count == 0);

	xml_node^ xc = xn->children[0];
	xc->XmlGetChildByName(xc, "COMPONENTS");

	for (int i = 0; i < xc->children.Count; i++)
	{
		GraphNodeType g = GraphNodeType::UNDEFINED;
		String ^file_url, ^filter_url, ^ctype;
		int x, y, w, h;
		xc->children[i]->XmlGetChildValueByName("FILE_URL", file_url);
		xc->children[i]->XmlGetChildValueByName("FILTER_URL", filter_url);
		xc->children[i]->XmlGetChildValueByName("COMPONENT_XPOS", x);
		xc->children[i]->XmlGetChildValueByName("COMPONENT_YPOS", y);
		xc->children[i]->XmlGetChildValueByName("COMPONENT_WIDTH", w);
		xc->children[i]->XmlGetChildValueByName("COMPONENT_HEIGHT", h);
		xc->children[i]->XmlGetChildValueByName("COMPONENT_TYPE", ctype);

		if (ctype == "SPLITTER")		g = GraphNodeType::SPLITTER_MODULE;
		if (ctype == "NULL_EXPORT" )	g = GraphNodeType::NULLEXPORT_MODULE;
		if (ctype == "IMAGE_IMPORT" )	g = GraphNodeType::IMPORT_MODULE;
		if (ctype == "IMAGE_EXPORT")	g = GraphNodeType::EXPORT_MODULE;
		if (ctype == "IMAGE_FILTER")	g = GraphNodeType::FILTER_MODULE;

		AddGraphNode(file_url, filter_url, g, Drawing::Point(x, y), Drawing::Size(w, h), true);
		nodeList[nodeList->Count - 1]->RestoreSavedSettings(xc->children[i]);
	}

	xc = xn->children[0];
	xc->XmlGetChildByName(xc, "EDGES");

	for (int i = 0; i < xc->children.Count; i++)
	{
		int a = -1 , b = -1, c = -1, d = -1;
		xc->children[i]->XmlGetChildValueByName("FROM", a);
		xc->children[i]->XmlGetChildValueByName("FROM_PIN", b);
		xc->children[i]->XmlGetChildValueByName("TO", c);
		xc->children[i]->XmlGetChildValueByName("TO_PIN", d);
		if (a == -1 || b == -1 || c == -1 || d == -1) continue;
		edgeList->Add(gcnew GraphEdge(nodeList[a], nodeList[c], b, d));
		nodeList[a]->LockEdge(b, nodeList[c]->GetPinType(d));
		nodeList[c]->LockEdge(d, nodeList[a]->GetPinType(b));
	}	

	WorkflowFile = sd->FileName;

	delete xn;
	delete sr;
	delete sd;

	WorkflowScrollEvent(nullptr, nullptr);
}

void WorkflowManager::SaveWorkflow(String^ filename)
{
	WorkflowFile = filename;

	Encoding^ enc = Encoding::Default;
	StreamWriter^ sw = gcnew StreamWriter(filename, false, enc);
	sw->WriteLine("<WORKFLOW VERSION=\"GEOFILTERBANK 1.2\">");
	sw->WriteLine("\t<COMPONENTS_COUNT>" + nodeList->Count.ToString() + "</COMPONENTS_COUNT>");
	sw->WriteLine("\t<COMPONENTS>");

	for (int i = 0; i < nodeList->Count; i++)
	{
		xml_node^ xn = nodeList[i]->GetXmlDescription();
		if (xn != nullptr) 
		{
			xn->name = "COMPONENT";
			xn->XmlSetChildValueByName("FILE_URL", nodeList[i]->FilterName->Text, false);
			String^ s = nodeList[i]->FilterName->Name;
			int l = 0;
			for (int j = s->Length - 1; j >= 0 && !l; j--) if (s[j] == '\\') l = j + 1;
			xn->XmlSetChildValueByName("FILTER_URL", s->Substring(l), false);
			xn->XmlSetChildValueByName("COMPONENT_XPOS",   nodeList[i]->Left + WorkflowScrollBarH->Value,   false);
			xn->XmlSetChildValueByName("COMPONENT_YPOS",   nodeList[i]->Top + WorkflowScrollBarV->Value,    false);
			xn->XmlSetChildValueByName("COMPONENT_WIDTH",  nodeList[i]->Width,  false);
			xn->XmlSetChildValueByName("COMPONENT_HEIGHT", nodeList[i]->Height, false);

			switch (nodeList[i]->GetNodeType())
			{
			case GraphNodeType::EXPORT_MODULE:   s = "IMAGE_EXPORT"; break;
			case GraphNodeType::FILTER_MODULE:   s = "IMAGE_FILTER"; break;
			case GraphNodeType::IMPORT_MODULE:   s = "IMAGE_IMPORT"; break;
			case GraphNodeType::SPLITTER_MODULE: s = "SPLITTER";     break;
			case GraphNodeType::NULLEXPORT_MODULE: s = "NULL_EXPORT"; break;
			}

			xn->XmlSetChildValueByName("COMPONENT_TYPE", s, false);

			xml_parser::write_xmltree_into_file(sw, xn, "\t\t");
		}
	}

	sw->WriteLine("\t</COMPONENTS>");
	sw->WriteLine("\t<EDGES_COUNT>" + edgeList->Count.ToString() + "</EDGES_COUNT>");
	sw->WriteLine("\t<EDGES>");
	
	int    c      = nodeList->Count;
	int    cnt    = edgeList->Count;
	int    from, to;

	for (int j = 0; j < cnt; j++)
	{
		for (int k = 0; k < c; k++)
		{
			if (edgeList[j]->source == nodeList[k]) from = k;
			if (edgeList[j]->target == nodeList[k]) to   = k;
		}
		sw->WriteLine("\t\t<EDGE>");
		sw->WriteLine("\t\t\t<FROM>" + from.ToString() + "</FROM>");
		sw->WriteLine("\t\t\t<FROM_PIN>" + edgeList[j]->source_pin.ToString() + "</FROM_PIN>");
		sw->WriteLine("\t\t\t<TO>" + to.ToString() + "</TO>");
		sw->WriteLine("\t\t\t<TO_PIN>" + edgeList[j]->target_pin.ToString() + "</TO_PIN>");
		sw->WriteLine("\t\t</EDGE>");
	}

	sw->WriteLine("\t</EDGES>");
	sw->WriteLine("</WORKFLOW>");
	sw->Flush();
	sw->Close();
	delete sw;
}

void WorkflowManager::menuFileSave_Click(System::Object^  sender, System::EventArgs^  e)
{
	if (WorkflowFile == "") 
		menuFileSaveAs_Click(sender, e);
	else
		SaveWorkflow(WorkflowFile);
}

void WorkflowManager::menuFileSaveAs_Click(System::Object^  sender, System::EventArgs^  e)
{
	SaveFileDialog^ sd = gcnew SaveFileDialog();
	sd->Title = "Workflow mentse";
	sd->Filter = "Workflow (*.workflow)|*.workflow";

	if (sd->ShowDialog() == Windows::Forms::DialogResult::Cancel)
	{
		delete sd;
		return;
	}
	
	SaveWorkflow(sd->FileName);
	delete sd;
}




void WorkflowManager::WorkflowManager_FormClosing(System::Object^  sender, System::Windows::Forms::FormClosingEventArgs^  e)
{
	/*
	String ^s, ^s2;
	mw->language_container->XmlGetChildValueByName("WorkflowManagerQuit", s);
	mw->language_container->XmlGetChildValueByName("WorkflowManager", s2);
	
	System::Windows::Forms::DialogResult res = MessageBox::Show(s, s2, System::Windows::Forms::MessageBoxButtons::YesNoCancel, System::Windows::Forms::MessageBoxIcon::Warning);

	if (res == System::Windows::Forms::DialogResult::Yes) mw->WorkflowManagerClose();
	else e->Cancel = true;
	*/

	mw->WorkflowManagerClose();
}


void WorkflowManager::menuAddImport_Click(System::Object^  sender, System::EventArgs^  e)
{
	AddGraphNode("", "", GraphNodeType::IMPORT_MODULE);
}

void WorkflowManager::menuAddExport_Click(System::Object^  sender, System::EventArgs^  e)
{
	ToolStripMenuItem^ mi = ((ToolStripMenuItem^)sender);
	AddGraphNode(mi->Text, mi->Name, GraphNodeType::EXPORT_MODULE);
}

void WorkflowManager::menuAddFilter_Click(System::Object^  sender, System::EventArgs^  e)
{
	ToolStripMenuItem^ mi = ((ToolStripMenuItem^)sender);
	AddGraphNode(mi->Text, mi->Name, GraphNodeType::FILTER_MODULE);
}

void WorkflowManager::menuAddSplitter_Click(System::Object^  sender, System::EventArgs^  e)
{
	AddGraphNode("", "", GraphNodeType::SPLITTER_MODULE);
}

void WorkflowManager::menuAddNullExport_Click(System::Object^  sender, System::EventArgs^  e)
{
	AddGraphNode("", "", GraphNodeType::NULLEXPORT_MODULE);
}

void WorkflowManager::menuWorkflowStart_Click(System::Object^  sender, System::EventArgs^  e)
{
	int c = nodeList->Count;
	for (int i = 0; i < c; i++) nodeList[i]->FilterProgressBar->Value = 0;

	if (!IsComplete())
	{
		MessageBox::Show("The workflow is incomplete. Check the pins, that all of them are connected to other pin.");
		return;
	}

	int* order = DetermineExecutionOrder();
	if (order == 0)
	{
		MessageBox::Show("The workflow should not have any directed cycle!");
		return;
	}


	map<int, int> pointers_in_use;
	map<int, int>::iterator ptrs_iter;

	//String^ s = "Execution Order is\n\n";
	//for (int i = 0; i < nodeList->Count; i++) s += nodeList[order[i]]->FilterName->Text + "\n";
	//MessageBox::Show(s, "Determined execution order");

	for (int i = 0; i < c; i++)
	{
		int k = order[i];

		if (nodeList[k]->GetNodeType() == GraphNodeType::IMPORT_MODULE || nodeList[k]->GetNodeType() == GraphNodeType::UNDEFINED)
		{
			// DO NOTHING :)
		}
		else
		{
			xml_node^ xd;
			xml_node^ dest = nodeList[k]->GetXmlExecutionSettings();

			for (int j = 0; j < edgeList->Count; j++)
			{
				if (edgeList[j]->target == nodeList[k]) // FOUND A SOURCE
				{
					int src_pin = edgeList[j]->source_pin;
					int dest_pin = edgeList[j]->target_pin;
					xml_node^ src = edgeList[j]->source->GetXmlExecutionSettings();

					// COPY XML INFORMATION

					String^ ss = "OutputImage" + edgeList[j]->source->OutputImageIndex(src_pin).ToString();
					String^ sd = "InputImage"  + edgeList[j]->target->InputImageIndex(dest_pin).ToString();

					if (!src->XmlGetChildByName(src, ss))
					{
						MessageBox::Show("!src->XmlGetChildByName(src, ss)");
						delete [] order;
						return;
					}

					if (!dest->XmlGetChildByName(xd, sd))
					{
						MessageBox::Show("!dest->XmlGetChildByName(xd, sd)");
						delete [] order;
						return;
					}

					if (!CopyImageSetting(src, xd))
					{
						MessageBox::Show("!CopyImageSetting(src, xd): " + edgeList[j]->source->FilterName->Text + " -> " + nodeList[k]->FilterName->Text);
						delete [] order;
						return;
					}
				}
			}
		}

		if (!nodeList[k]->Run()) 
		{
			MessageBox::Show("!nodeList[k]->Run()");
			break;
		}

		// SHOW INTERMEDIATE IMAGES -->
		if (menuWorkflowInteractiveMode->Checked && nodeList[k]->GetNodeType() != GraphNodeType::SPLITTER_MODULE)
		{
			int ie = nodeList[k]->GetInEdges();
			int c  = nodeList[k]->GetOutEdges();
			for (int i = 0; i < c; i++)
			{
				xml_node^ xt = nodeList[k]->GetXmlExecutionSettings();
				if (xt == nullptr) continue;
				xt->XmlGetChildByName(xt, "OutputImage" + i.ToString());
				if (xt == nullptr) continue;

				LayerContainer^ lc = gcnew LayerContainer();
				if (nodeList[k]->GetPinType(i + ie) == "RASTER") 
					lc->geoViewer->SetBitmapImage(xt);
				else
					lc->geoViewer->SetVectorImage(xt);

				lc->geoViewer->SetVectorImageLiveOnFreeUp(true);
				lc->geoViewer->SetBitmapImageLiveOnFreeUp(true);

				lc->ShowDialog();
			}
		}
		
		// GARBAGE COLLECTION
		nodeList[k]->GCMap_Update(pointers_in_use);

		for (ptrs_iter = pointers_in_use.begin(); ptrs_iter != pointers_in_use.end();)
		{
			if (ptrs_iter->second == 0) 
			{
				byte* p = (byte*)(void*)(ptrs_iter->first);
				delete [] p;
				ptrs_iter = pointers_in_use.erase(ptrs_iter);
			}
			else						
				ptrs_iter++;
		}
	}

	// FREE UP POSSIBLE MEMORY LEAKS
	for (ptrs_iter = pointers_in_use.begin(); ptrs_iter != pointers_in_use.end(); ptrs_iter++)
	{
			byte* p = (byte*)(void*)(ptrs_iter->first);
			delete [] p;
	}
	delete [] order;

	for (int i = 0; i < c; i++) nodeList[i]->FilterProgressBar->Value = 0;
}

void WorkflowManager::CreateEdge(bool forward)
{
	int i = GetPointedNode();
	if (0 <= i)
	{
		if (src == nodeList[i]) return;
		int x0 = x - nodeList[i]->Left;
		int y0 = y - nodeList[i]->Top;
		int j = (forward) ? nodeList[i]->IndexOfPointedInEdge(x0, y0) : nodeList[i]->IndexOfPointedOutEdge(x0, y0);
		if (0 <= j)
		{
			bool b1 = nodeList[i]->LockEdge(j, src->GetPinType(pin));
			bool b2 = src->LockEdge(pin, nodeList[i]->GetPinType(j));
			if (b1 && b2)
			{
				if (forward) edgeList->Add(gcnew GraphEdge(src, nodeList[i], pin, j));
				else         edgeList->Add(gcnew GraphEdge(nodeList[i], src, j, pin));
			}
			else
			{
				if (b1) nodeList[i]->ReleaseEdge(j);
				if (b2) src->ReleaseEdge(pin);

				MessageBox::Show("Invalid edge contact: " + src->GetPinType(pin) + " <-> " + nodeList[i]->GetPinType(j));
			}
		}
	}
}


void WorkflowManager::DeleteEdge(GraphNode^ gn, int pin)
{
	for (int i = edgeList->Count - 1; i >= 0; i--) 
	{
		if ((edgeList[i]->source == gn && edgeList[i]->source_pin == pin) || (edgeList[i]->target == gn && edgeList[i]->target_pin == pin))
		{
			edgeList[i]->source->ReleaseEdge(edgeList[i]->source_pin);
			edgeList[i]->target->ReleaseEdge(edgeList[i]->target_pin);
			delete edgeList[i];
			edgeList->RemoveAt(i);
			break;
		}
	}
}

void WorkflowManager::DeleteNode(GraphNode^ gn)
{
	for (int i = edgeList->Count - 1; i >= 0; i--) 
	{
		if (edgeList[i]->source == gn || edgeList[i]->target == gn)
		{
			edgeList[i]->source->ReleaseEdge(edgeList[i]->source_pin);
			edgeList[i]->target->ReleaseEdge(edgeList[i]->target_pin);
			delete edgeList[i];
			edgeList->RemoveAt(i);
		}
	}
	for (int i = nodeList->Count - 1; i >= 0; i--) 
	{
		if (nodeList[i] == gn)
		{
			nodeList->RemoveAt(i);
			break;
		}
	}
	this->Controls->Remove(gn);
	delete gn;
	this->Refresh();
}


int  WorkflowManager::GetPointedNode()
{
	int c = nodeList->Count;
	for (int i = 0; i < c; i++) if (nodeList[i]->Bounds.Contains(x, y)) return i;
	return -1;
}

void WorkflowManager::AddGraphNode(String^ title, String^ filter_url, GraphNodeType gt, System::Drawing::Point location, System::Drawing::Size dimension, bool restore)
{
	GraphNode^ gc = gcnew GeoFilterBank::GraphNode();
	this->SuspendLayout();
	gc->Location = location;
	gc->Size = dimension;
	gc->Name = L"graphNode";

	bool cont = true;
	if (restore)	gc->RestoreSavedNode(this, title, filter_url, gt);
	else			cont = gc->InitializeNode(  this, title, filter_url, gt);

	if (cont)
	{
		nodeList->Add(gc);
		this->Controls->Add(gc);
	}
	this->ResumeLayout(false);
}

void WorkflowManager::AddGraphNode(String^ title, String^ filter_url, GraphNodeType gt)
{
	AddGraphNode(title, filter_url, gt, System::Drawing::Point(354, 96), System::Drawing::Size(198, 117), false);
}




#define COND_TEST(condition) { if (condition) { return false; } }
bool WorkflowManager::IsEdgeValid(GeoFilterBank::GraphEdge ^edge)
{
	return true;
}

bool WorkflowManager::CopyImageSetting(xml_node^ fs, xml_node^ fd)
{
	if (fs == nullptr || fd == nullptr) return false;

	int tmp, cc = 0;
	if (! fs->XmlGetChildValueByName("Width"       , tmp) ) return false;
	fd->XmlSetChildValueByName("Width", tmp);
	if (! fs->XmlGetChildValueByName("Height"      , tmp) ) return false;
	fd->XmlSetChildValueByName("Height", tmp);
	if (! fs->XmlGetChildValueByName("Stride"      , tmp) ) return false;
	fd->XmlSetChildValueByName("Stride", tmp);
	if (! fs->XmlGetChildValueByName("Packed"      , tmp) ) return false;
	fd->XmlSetChildValueByName("Packed", tmp);
	if (! fs->XmlGetChildValueByName("Interleaved" , tmp) ) return false;
	fd->XmlSetChildValueByName("Interleaved", tmp);
	if (! fs->XmlGetChildValueByName("ChannelCount", cc) )  return false;
	fd->XmlSetChildValueByName("ChannelCount", cc);
	if (! fs->XmlGetChildValueByName("PixelSize"   , tmp) ) return false;
	fd->XmlSetChildValueByName("PixelSize", tmp);
	if (! fs->XmlGetChildValueByName("Data"        , tmp) ) return false;
	fd->XmlSetChildValueByName("Data", tmp);
	if (!fs->XmlGetChildByName(fs, "ChannelsBitAllocation")) return false;
	fd->XmlSetChildValueByName("ChannelsBitAllocation", "", false);
	if (!fd->XmlGetChildByName(fd, "ChannelsBitAllocation")) return false;

	for (int i = 0; i < cc; i++)
	{
		if (! fs->XmlGetChildValueByName("Channel" + i.ToString(), tmp) ) return false;
		fd->XmlSetChildValueByName("Channel" + i.ToString(), tmp);
	}

	String ^sTmp;
	if ( fs->XmlGetChildValueByName("ColorMap", sTmp) ) fd->XmlSetChildValueByName("ColorMap", sTmp, false);

	return true;
}

