#ifndef IMAGEFILTER_H
#define IMAGEFILTER_H

#pragma once

#include "stdafx.h"

using namespace System;

#define IMAGEFILTER_API_VERSION VER_GEN(1,2,0,0)

namespace GeoFilterBank
{
	struct polygon_node { int x, y; };


	public ref class ImageFilter
	{
	public:
		static int     GetOneImageSetting(xml_node^ fs, int& Width, int& Height, int& Stride, int& Packed, int& Interleaved, int& ChannelCount, int& PixelSize, byte*& Data)
		{
			int tmp;
			if (fs == nullptr) return 1;

			if (! fs->XmlGetChildValueByName("Width", Width) ) return 10;
			if (! fs->XmlGetChildValueByName("Height", Height) ) return 11;
			if (! fs->XmlGetChildValueByName("Stride", Stride) ) return 12;
			if (! fs->XmlGetChildValueByName("Packed", Packed) ) return 13;
			if (! fs->XmlGetChildValueByName("Interleaved", Interleaved) ) return 14;
			if (! fs->XmlGetChildValueByName("ChannelCount", ChannelCount) ) return 15;
			if (! fs->XmlGetChildValueByName("PixelSize", PixelSize) ) return 16;
			if (! fs->XmlGetChildValueByName("Data", tmp) ) return 17;
			Data = (byte*)(void*)tmp;

			return 0;
		}
		static void    CreateImageSetting(xml_node^ fs, int Width, int Height, int Stride, int Packed, int Interleaved, int ChannelCount, int PixelSize, byte* Data)
		{
			if (fs == nullptr) return;
			fs->children.Clear();
			fs->children.Add( gcnew xml_node(fs, "Width", "", Width.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Height", "", Height.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Stride", "", Stride.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Packed", "", Packed.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Interleaved", "", Interleaved.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "ChannelCount", "", ChannelCount.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "PixelSize", "", PixelSize.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Data", "", ((int)(void*)Data).ToString()) );
			fs->children.Add( gcnew xml_node(fs, "ChannelsBitAllocation", "", "" ) );
		}

		static void    SetChannelBitAllocation(xml_node^ fs, int Channel, int BitAllocation)
		{
			if (fs == nullptr) return;
			xml_node^ v;
			if (! fs->XmlGetChildByName(v, "Channel" + Channel.ToString()) )
				fs->children.Add( gcnew xml_node( fs, "Channel" + Channel.ToString(), "", BitAllocation.ToString() ) );
			else
				v->XmlSetIntValue( BitAllocation );
		}


		static IntPtr  CreateImageMask(int width, int height)
		{
			if (width < 1 || height < 1) return IntPtr(0);
			int size = ((width + 7) >> 3) * height;
			byte* mask = new byte[size];
			memset(mask, 0, size);
			return (IntPtr)(void*)mask;
		}

		static void    AddMaskRectangle(IntPtr mask, int imagewidth, int x, int y, int width, int height)
		{
			if (!(void*)mask) return;
			int stride = ((imagewidth + 7) >> 3);
			byte* m = (byte*)(void*)mask + y * stride;

			int x1 = x;
			int x2 = x + width - 1;
			int xbegin = (x1 + 7) >> 3;         // First byte, where first bit is 1
			int xend = ((x2 + 1) >> 3) - 1;     // Last  byte, where last  bit is 1
			int xbeginremain = (xbegin << 3) - x1;
			int xendremain = (x2 & 7) ? x2 & 7 : 0;
			int fullbits = xend - xbegin + 1;
			byte leadbyte = ( 255 - ((1 << (8 - xbeginremain)) - 1) );
			byte tailbyte = (1 << xendremain) - 1;

			for (int j = y; j < y + height; ++j)
			{
				if (xbeginremain)   m[xbegin - 1] |= leadbyte;
				if (xendremain)     m[xend + 1]   |= tailbyte;
				if (xbegin <= xend) memset(m + xbegin, 255, fullbits);
				m += stride;
			}
		}

		static void    AddMaskPolygon(IntPtr mask, int imagewidth, int imageheight, int node_count, IntPtr nodes)
		{
			// First and last node should be the same (closed curve)

			float m;
			int node_swap, x, y, i, j;
			int stride = ((imagewidth + 7) >> 3);
			byte* mmask = (byte*)(void*)mask;
			polygon_node nodeA, nodeB;
			polygon_node* pnode = (polygon_node*)(void*)nodes;

			// Return if any of the pointers is null
			if (!mmask || !pnode) return;

			// Allocate temporary buffer
			int** boundaries_in_row = new int*[imageheight];
			for (i = 0; i < imageheight; ++i) boundaries_in_row[i] = new int[node_count];
			int* boundaries_in_row_count = new int[imageheight];
			memset(boundaries_in_row_count, 0, sizeof(int) * imageheight);
			
			// Process sections  (POLIMORPHED VERSION)
			for (j = 0; j < node_count - 1; ++j)
			{
				int xdiff = (pnode[j].x - pnode[j + 1].x);
				int ydiff = (pnode[j].y - pnode[j + 1].y);
				int pluszegy = (xdiff * xdiff > ydiff * ydiff) ? 1 : 0;

				node_swap = (pnode[j].y < pnode[j + 1].y) ? 0 : 1;
				nodeA = pnode[node_swap + j];
				nodeB = pnode[(1 - node_swap) + j];
				m = ((float)(nodeB.x - nodeA.x)) / ((float)(nodeB.y - nodeA.y));

				for (y = nodeA.y + 1; y <= nodeB.y; ++y)
				{
					float f = m * (float)(y - nodeA.y) + (float)nodeA.x;
					x = (int)f;
					if (f - (float)x >= 0.5) x++;

					byte* mp = mmask + y * stride;
					byte hehe = x % 8;
					hehe = (hehe) ? (1 << hehe) : 1;
					mp[x / 8] |= hehe;
					
					boundaries_in_row[y][boundaries_in_row_count[y]] = x;
					boundaries_in_row_count[y]++;
					if (boundaries_in_row_count[y] > node_count)
					{
						System::Windows::Forms::MessageBox::Show("Baromarc");
						return;
					}
				}
			}

			// Intersection points exist, but are unsorted now in "boundaries_in_row" --> Sort them
			for (i = 0; i < imageheight; ++i)
			{
				if (boundaries_in_row_count[i] > 0)
				{
					qsort((void*)boundaries_in_row[i], boundaries_in_row_count[i], sizeof(int), compare);
					
					// We believe that polygon lies inside the image with its boundaries
					for (j = 0; j < boundaries_in_row_count[i] / 2; j += 1)
					{
						int x1 = boundaries_in_row[i][j * 2] + 1;
						int x2 = boundaries_in_row[i][j * 2 + 1];
						
						// First block
						if (x1 & 7) 
						{
							int shift = x1 & 7;
							int bits = (x1 >> 3 == x2 >> 3) ? (x2 - x1 + 1) : (8 - (x1 & 7));
							byte masker = (1 << bits) - 1;
							if (shift) masker = masker << shift;

							mmask[x1 >> 3] |= masker;
						}
						
						// Last block is not a complete block and is not in the first block
						if ( ((x2 + 1) & 7) && (x1 >> 3 != x2 >> 3) )
						{
							int bits = (x2 & 7) + 1;
							byte masker = (1 << bits) - 1;
							mmask[x2 >> 3] |= masker;
						}
						
						// If there are any complete blocks 
						if ( ((x2 + 1) >> 3) - ((x1 + 7) >> 3) > 0 )
						{
							int full_blocks = ((x2 + 1) >> 3) - ((x1 + 7) >> 3);
							int begin = (x1 + 7) >> 3;
							memset(mmask + begin, 255, full_blocks);
						}
					}
				}
				mmask += stride;
			}

			// Free up memory
			for (i = 0; i < imageheight; ++i) delete [] boundaries_in_row[i];
			delete [] boundaries_in_row;
			delete [] boundaries_in_row_count;
		}

		static void    DestroyImageMask(IntPtr mask){ if ((void*)mask) delete [] ((byte*)(void*)mask); }


	};

}

#endif
