#pragma once
#pragma warning(disable:4244)


#include "stdafx.h"

using namespace System;
using namespace System::Collections::Generic;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Text;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;
using namespace System::Runtime::InteropServices;


#define CLICK_SELECT          0
// IMAGE MODEL
#define IMAGE_VIEW            1
#define MULTIRECTANGLE_SELECT 2
#define MULTIPOLIGON_SELECT   3
// VECTOR MODEL
#define SELECT_A_POINT        4
#define SELECT_A_POLYLINE     5
#define SELECT_A_POLYGON      6


#define NONE      0
#define RECTANGLE 1
#define POLIGON   2


namespace GeoViewer {


	public ref class GeoViewer : public System::Windows::Forms::UserControl
	{
	public:
		ref struct SelectionStruct
		{
			int selectiontype;
			List<Point> polygonpoints;
			System::Drawing::Rectangle rectangle;
			Color color;
		};

		ref struct VectorModel
		{
			VectorModel():PointsCount(0), PolylinesPointCount(0), PolygonsPointCount(0), leavedata(false) {}
			~VectorModel()
			{
				if (!leavedata)
				{
					if (PointsCount)
					{
						delete [] PointsXArray;
						delete [] PointsYArray;
					}
					if (PolylinesPointCount)
					{
						delete [] PolylinesStartArray;
						delete [] PolylinesPartLengthArray;
						delete [] PolylinesXArray;
						delete [] PolylinesYArray;
					}
					if (PolygonsPointCount)
					{
						delete [] PolygonsStartArray;
						delete [] PolygonsPartLengthArray;
						delete [] PolygonsXArray;
						delete [] PolygonsYArray;
					}
				}
			}

			xml_node^ GetXmlDescription()
			{
				xml_node^ xn = gcnew xml_node(nullptr, "", "", "");

				// BOUNDING BOX
				xn->XmlSetChildValueByName("BoundingBoxLeft",   box.Left.ToString(),   false);
				xn->XmlSetChildValueByName("BoundingBoxTop",    box.Top.ToString(),    false);
				xn->XmlSetChildValueByName("BoundingBoxRight",  box.Right.ToString(),  false);
				xn->XmlSetChildValueByName("BoundingBoxBottom", box.Bottom.ToString(), false);

				// POINTS
				xn->XmlSetChildValueByName("PointsCount",  PointsCount,                false);
				xn->XmlSetChildValueByName("PointsXArray", (int)(void*)(PointsXArray), false);
				xn->XmlSetChildValueByName("PointsYArray", (int)(void*)(PointsYArray), false);

				// POLYLINES
				xn->XmlSetChildValueByName("PolylinesPartCount",       PolylinesPartCount,                     false);
				xn->XmlSetChildValueByName("PolylinesPointCount",      PolylinesPointCount,                    false);
				xn->XmlSetChildValueByName("PolylinesStartArray",      (int)(void*)(PolylinesStartArray),      false);
				xn->XmlSetChildValueByName("PolylinesPartLengthArray", (int)(void*)(PolylinesPartLengthArray), false);
				xn->XmlSetChildValueByName("PolylinesXArray",          (int)(void*)(PolylinesXArray),          false);
				xn->XmlSetChildValueByName("PolylinesYArray",          (int)(void*)(PolylinesYArray),          false);

				// POLYGONS
				xn->XmlSetChildValueByName("PolygonsPartCount",        PolygonsPartCount,                     false);
				xn->XmlSetChildValueByName("PolygonsPointCount",       PolygonsPointCount,                    false);
				xn->XmlSetChildValueByName("PolygonsStartArray",       (int)(void*)(PolygonsStartArray),      false);
				xn->XmlSetChildValueByName("PolygonsPartLengthArray",  (int)(void*)(PolygonsPartLengthArray), false);
				xn->XmlSetChildValueByName("PolygonsXArray",           (int)(void*)(PolygonsXArray),          false);
				xn->XmlSetChildValueByName("PolygonsYArray",           (int)(void*)(PolygonsYArray),          false);

				return xn;
			}

			// Point
			int     PointsCount;
			double* PointsXArray;
			double* PointsYArray;

			// Polyline
			int     PolylinesPartCount;
			int     PolylinesPointCount;
			int*    PolylinesStartArray;
			int*    PolylinesPartLengthArray;
			double* PolylinesXArray;
			double* PolylinesYArray;

			// Polygon
			int     PolygonsPartCount;
			int     PolygonsPointCount;
			int*    PolygonsStartArray;
			int*    PolygonsPartLengthArray;
			double* PolygonsXArray;
			double* PolygonsYArray;

			// Bounding rectangle - georeferred
			System::Drawing::RectangleF box;
			bool                        leavedata;
		};
	// -------------------------------------------------------------


	public:
		GeoViewer(void)
		{
			// sajt vltozk belltsa

			ImageBitmapInfo = (BITMAPINFO*)(new unsigned char[ sizeof(BITMAPINFO) + sizeof(RGBQUAD) * 256 ]);
			ImageOriginalColormap = new RGBQUAD[256];
			ImageBitmapInfo->bmiHeader.biSize			= sizeof(BITMAPINFOHEADER);
			ImageBitmapInfo->bmiHeader.biPlanes			= 1;
			ImageBitmapInfo->bmiHeader.biCompression	= BI_RGB;
			ImageBitmapInfo->bmiHeader.biSizeImage		= 0;
			ImageBitmapInfo->bmiHeader.biXPelsPerMeter	= 0;
			ImageBitmapInfo->bmiHeader.biYPelsPerMeter	= 0;
			ImageBitmapInfo->bmiHeader.biClrImportant	= 0;

			StrSelection			= "Kijells: {0},{1} x {2},{3}";
			StrCursorPosition		= "Pozci: {0},{1}";

			Zoom					= 100.0F;
			ScrollVisible			= true;
	        SelectionsVisible		= false;
			ImageDisplayed			= false;
			ImageDataLoaded			= false;
			MouseDownLeft			= false;
			MouseDownRight			= false;
			KeyCtrl					= false;
			SelectedFirstPoint		= false;
			ActiveBorder			= Color::LightSalmon;
			ActiveRegion			= Color::DarkOrange;
			SelectedItemColor		= Color::Maroon;
			VectorPointColor		= Color::DarkOrange;
			VectorPolylineColor		= Color::DarkOrange;
			VectorPolygonColor		= Color::DarkOrange;
			ComponentFunctionality	= IMAGE_VIEW;
			ImageNode				= nullptr;
			ImageData				= 0;
			Vector_Model            = nullptr;
			VectorDisplayed         = false;
			LeaveBitmapImageData    = false;
			LeaveVectorImageData    = false;

			InitializeComponent();

			ToolStripDropDownButtonColorMapping->Image	= ImageListColorMapping->Images[0];
			MenuItemColorMapOriginal->Image				= ImageListColorMapping->Images[0];
			MenuItemColorMapGrayScale->Image			= ImageListColorMapping->Images[1];
			MenuItemColorMapRed->Image					= ImageListColorMapping->Images[2];
			MenuItemColorMapGreen->Image				= ImageListColorMapping->Images[3];
			MenuItemColorMapBlue->Image					= ImageListColorMapping->Images[4];
			MenuItemColorMapCustom->Image				= ImageListColorMapping->Images[5];
		}

	protected:
		~GeoViewer()
		{
			FreeUpBitmapResources();
			FreeUpVectorResources();

			delete [] ImageOriginalColormap;
			delete [] ImageBitmapInfo;

			if (components) delete components;
		}

	////////////////////////
	// CONTROLS VARIABLES //
	////////////////////////

	#pragma region GUI Controls
	public:
		System::Windows::Forms::ListView^  SelectionsView;
		System::Windows::Forms::ToolStripProgressBar^  StatusProgressBar;
private: System::Windows::Forms::Label^  LabelSelectionColor;
private: System::Windows::Forms::LinkLabel^  RemoveSelectedLabel;

private: 
public: 

	private:


		System::Windows::Forms::PictureBox^  DestPicture;
		System::Windows::Forms::VScrollBar^  VSc;
		System::Windows::Forms::HScrollBar^  HSc;
		System::Windows::Forms::StatusStrip^  StatusBar;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemBehaviours;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemPan;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemPointClickSelect;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemRectangleSelection;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemPolygonSelection;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemClearSelection;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemSelections;
		System::Windows::Forms::ToolStripMenuItem^  MenuItemScrollbars;
		System::Windows::Forms::ToolStripSplitButton^  toolStripSplitButton1;
		System::Windows::Forms::ToolStripStatusLabel^  StatusLabel1;
		System::Windows::Forms::ToolStripStatusLabel^  StatusLabel2;
		System::Windows::Forms::ToolStripStatusLabel^  StatusLabel3;
		System::Windows::Forms::ImageList^  SelectionsImageList;
		System::Windows::Forms::ColorDialog^  ColorDialog;
private: System::Windows::Forms::CheckBox^  CheckBoxShowVectorPoint;
private: System::Windows::Forms::CheckBox^  CheckBoxShowVectorPolyline;
private: System::Windows::Forms::CheckBox^  CheckBoxShowVectorPolygon;
private: System::Windows::Forms::CheckBox^  CheckBoxShowRaster;
private: System::Windows::Forms::CheckBox^  CheckBoxShowSelection;
private: System::Windows::Forms::ImageList^  ImageListShowCheckBoxes;
private: System::Windows::Forms::Label^  LabelVectorPointColor;
private: System::Windows::Forms::Label^  LabelVectorPolygonColor;
private: System::Windows::Forms::Label^  LabelVectorPolylineColor;
private: System::Windows::Forms::ToolStripDropDownButton^  ToolStripDropDownButtonColorMapping;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapTitle;


private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapOriginal;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapGrayScale;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapRed;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapGreen;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapBlue;
private: System::Windows::Forms::ToolStripMenuItem^  MenuItemColorMapCustom;
private: System::Windows::Forms::ToolStripSeparator^  toolStripMenuItem1;
private: System::Windows::Forms::ImageList^  ImageListColorMapping;












		 System::ComponentModel::IContainer^  components;
	#pragma endregion


		void InitializeComponent(void)
		{
			this->components = (gcnew System::ComponentModel::Container());
			System::ComponentModel::ComponentResourceManager^  resources = (gcnew System::ComponentModel::ComponentResourceManager(GeoViewer::typeid));
			this->DestPicture = (gcnew System::Windows::Forms::PictureBox());
			this->RemoveSelectedLabel = (gcnew System::Windows::Forms::LinkLabel());
			this->LabelSelectionColor = (gcnew System::Windows::Forms::Label());
			this->SelectionsView = (gcnew System::Windows::Forms::ListView());
			this->SelectionsImageList = (gcnew System::Windows::Forms::ImageList(this->components));
			this->StatusBar = (gcnew System::Windows::Forms::StatusStrip());
			this->toolStripSplitButton1 = (gcnew System::Windows::Forms::ToolStripSplitButton());
			this->MenuItemBehaviours = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemPan = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemPointClickSelect = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemRectangleSelection = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemPolygonSelection = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemClearSelection = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemSelections = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemScrollbars = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->StatusProgressBar = (gcnew System::Windows::Forms::ToolStripProgressBar());
			this->StatusLabel1 = (gcnew System::Windows::Forms::ToolStripStatusLabel());
			this->StatusLabel2 = (gcnew System::Windows::Forms::ToolStripStatusLabel());
			this->StatusLabel3 = (gcnew System::Windows::Forms::ToolStripStatusLabel());
			this->ToolStripDropDownButtonColorMapping = (gcnew System::Windows::Forms::ToolStripDropDownButton());
			this->MenuItemColorMapTitle = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->toolStripMenuItem1 = (gcnew System::Windows::Forms::ToolStripSeparator());
			this->MenuItemColorMapOriginal = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemColorMapGrayScale = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemColorMapRed = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemColorMapGreen = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemColorMapBlue = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->MenuItemColorMapCustom = (gcnew System::Windows::Forms::ToolStripMenuItem());
			this->VSc = (gcnew System::Windows::Forms::VScrollBar());
			this->HSc = (gcnew System::Windows::Forms::HScrollBar());
			this->ColorDialog = (gcnew System::Windows::Forms::ColorDialog());
			this->CheckBoxShowVectorPoint = (gcnew System::Windows::Forms::CheckBox());
			this->ImageListShowCheckBoxes = (gcnew System::Windows::Forms::ImageList(this->components));
			this->CheckBoxShowVectorPolyline = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxShowVectorPolygon = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxShowRaster = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxShowSelection = (gcnew System::Windows::Forms::CheckBox());
			this->LabelVectorPointColor = (gcnew System::Windows::Forms::Label());
			this->LabelVectorPolygonColor = (gcnew System::Windows::Forms::Label());
			this->LabelVectorPolylineColor = (gcnew System::Windows::Forms::Label());
			this->ImageListColorMapping = (gcnew System::Windows::Forms::ImageList(this->components));
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->DestPicture))->BeginInit();
			this->StatusBar->SuspendLayout();
			this->SuspendLayout();
			// 
			// DestPicture
			// 
			this->DestPicture->Location = System::Drawing::Point(0, 0);
			this->DestPicture->Name = L"DestPicture";
			this->DestPicture->Size = System::Drawing::Size(411, 316);
			this->DestPicture->TabIndex = 1;
			this->DestPicture->TabStop = false;
			this->DestPicture->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &GeoViewer::Dest_MouseMove);
			this->DestPicture->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &GeoViewer::Dest_MouseDown);
			this->DestPicture->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &GeoViewer::OnPaint);
			this->DestPicture->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &GeoViewer::Dest_MouseUp);
			// 
			// RemoveSelectedLabel
			// 
			this->RemoveSelectedLabel->AutoSize = true;
			this->RemoveSelectedLabel->Location = System::Drawing::Point(446, 123);
			this->RemoveSelectedLabel->Name = L"RemoveSelectedLabel";
			this->RemoveSelectedLabel->Size = System::Drawing::Size(83, 13);
			this->RemoveSelectedLabel->TabIndex = 13;
			this->RemoveSelectedLabel->TabStop = true;
			this->RemoveSelectedLabel->Text = L"Kijelltek trlse";
			this->RemoveSelectedLabel->LinkClicked += gcnew System::Windows::Forms::LinkLabelLinkClickedEventHandler(this, &GeoViewer::RemoveSelectedLabel_LinkClicked);
			// 
			// LabelSelectionColor
			// 
			this->LabelSelectionColor->BackColor = System::Drawing::Color::SteelBlue;
			this->LabelSelectionColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->LabelSelectionColor->ForeColor = System::Drawing::Color::White;
			this->LabelSelectionColor->Location = System::Drawing::Point(491, 32);
			this->LabelSelectionColor->Name = L"LabelSelectionColor";
			this->LabelSelectionColor->Size = System::Drawing::Size(55, 14);
			this->LabelSelectionColor->TabIndex = 12;
			this->LabelSelectionColor->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->LabelSelectionColor->Click += gcnew System::EventHandler(this, &GeoViewer::LabelColorChoose_Click);
			// 
			// SelectionsView
			// 
			this->SelectionsView->BackColor = System::Drawing::Color::SkyBlue;
			this->SelectionsView->HeaderStyle = System::Windows::Forms::ColumnHeaderStyle::None;
			this->SelectionsView->LargeImageList = this->SelectionsImageList;
			this->SelectionsView->Location = System::Drawing::Point(446, 140);
			this->SelectionsView->Margin = System::Windows::Forms::Padding(0);
			this->SelectionsView->Name = L"SelectionsView";
			this->SelectionsView->Size = System::Drawing::Size(100, 176);
			this->SelectionsView->SmallImageList = this->SelectionsImageList;
			this->SelectionsView->TabIndex = 11;
			this->SelectionsView->UseCompatibleStateImageBehavior = false;
			this->SelectionsView->ItemSelectionChanged += gcnew System::Windows::Forms::ListViewItemSelectionChangedEventHandler(this, &GeoViewer::SelectionsView_ItemSelectionChanged);
			this->SelectionsView->KeyUp += gcnew System::Windows::Forms::KeyEventHandler(this, &GeoViewer::SelectionsView_KeyUp);
			this->SelectionsView->KeyDown += gcnew System::Windows::Forms::KeyEventHandler(this, &GeoViewer::SelectionsView_KeyDown);
			// 
			// SelectionsImageList
			// 
			this->SelectionsImageList->ColorDepth = System::Windows::Forms::ColorDepth::Depth32Bit;
			this->SelectionsImageList->ImageSize = System::Drawing::Size(64, 64);
			this->SelectionsImageList->TransparentColor = System::Drawing::Color::Transparent;
			// 
			// StatusBar
			// 
			this->StatusBar->BackColor = System::Drawing::Color::Gainsboro;
			this->StatusBar->Items->AddRange(gcnew cli::array< System::Windows::Forms::ToolStripItem^  >(6) {this->toolStripSplitButton1, 
				this->StatusProgressBar, this->StatusLabel1, this->StatusLabel2, this->StatusLabel3, this->ToolStripDropDownButtonColorMapping});
			this->StatusBar->Location = System::Drawing::Point(0, 418);
			this->StatusBar->Name = L"StatusBar";
			this->StatusBar->Size = System::Drawing::Size(638, 25);
			this->StatusBar->TabIndex = 10;
			this->StatusBar->Text = L"statusStrip1";
			// 
			// toolStripSplitButton1
			// 
			this->toolStripSplitButton1->AutoSize = false;
			this->toolStripSplitButton1->DisplayStyle = System::Windows::Forms::ToolStripItemDisplayStyle::Image;
			this->toolStripSplitButton1->DropDownItems->AddRange(gcnew cli::array< System::Windows::Forms::ToolStripItem^  >(3) {this->MenuItemBehaviours, 
				this->MenuItemSelections, this->MenuItemScrollbars});
			this->toolStripSplitButton1->Image = (cli::safe_cast<System::Drawing::Image^  >(resources->GetObject(L"toolStripSplitButton1.Image")));
			this->toolStripSplitButton1->ImageTransparentColor = System::Drawing::Color::Magenta;
			this->toolStripSplitButton1->Name = L"toolStripSplitButton1";
			this->toolStripSplitButton1->Size = System::Drawing::Size(40, 23);
			this->toolStripSplitButton1->Text = L"toolStripSplitButton1";
			this->toolStripSplitButton1->Visible = false;
			// 
			// MenuItemBehaviours
			// 
			this->MenuItemBehaviours->DropDownItems->AddRange(gcnew cli::array< System::Windows::Forms::ToolStripItem^  >(5) {this->MenuItemPan, 
				this->MenuItemPointClickSelect, this->MenuItemRectangleSelection, this->MenuItemPolygonSelection, this->MenuItemClearSelection});
			this->MenuItemBehaviours->Name = L"MenuItemBehaviours";
			this->MenuItemBehaviours->Size = System::Drawing::Size(133, 22);
			this->MenuItemBehaviours->Text = L"Behaviour";
			// 
			// MenuItemPan
			// 
			this->MenuItemPan->Name = L"MenuItemPan";
			this->MenuItemPan->Size = System::Drawing::Size(178, 22);
			this->MenuItemPan->Text = L"Pan";
			this->MenuItemPan->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemPan_Click);
			// 
			// MenuItemPointClickSelect
			// 
			this->MenuItemPointClickSelect->Name = L"MenuItemPointClickSelect";
			this->MenuItemPointClickSelect->Size = System::Drawing::Size(178, 22);
			this->MenuItemPointClickSelect->Text = L"Point click select";
			this->MenuItemPointClickSelect->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemPointClickSelect_Click);
			// 
			// MenuItemRectangleSelection
			// 
			this->MenuItemRectangleSelection->Name = L"MenuItemRectangleSelection";
			this->MenuItemRectangleSelection->Size = System::Drawing::Size(178, 22);
			this->MenuItemRectangleSelection->Text = L"Rectangle selection";
			this->MenuItemRectangleSelection->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemRectangleSelection_Click);
			// 
			// MenuItemPolygonSelection
			// 
			this->MenuItemPolygonSelection->Name = L"MenuItemPolygonSelection";
			this->MenuItemPolygonSelection->Size = System::Drawing::Size(178, 22);
			this->MenuItemPolygonSelection->Text = L"Polygon selection";
			this->MenuItemPolygonSelection->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemPolygonSelection_Click);
			// 
			// MenuItemClearSelection
			// 
			this->MenuItemClearSelection->Name = L"MenuItemClearSelection";
			this->MenuItemClearSelection->Size = System::Drawing::Size(178, 22);
			this->MenuItemClearSelection->Text = L"Clear selection";
			this->MenuItemClearSelection->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemClearSelection_Click);
			// 
			// MenuItemSelections
			// 
			this->MenuItemSelections->Name = L"MenuItemSelections";
			this->MenuItemSelections->Size = System::Drawing::Size(133, 22);
			this->MenuItemSelections->Text = L"Selections";
			this->MenuItemSelections->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemSelections_Click);
			// 
			// MenuItemScrollbars
			// 
			this->MenuItemScrollbars->Checked = true;
			this->MenuItemScrollbars->CheckState = System::Windows::Forms::CheckState::Checked;
			this->MenuItemScrollbars->Name = L"MenuItemScrollbars";
			this->MenuItemScrollbars->Size = System::Drawing::Size(133, 22);
			this->MenuItemScrollbars->Text = L"Scrollbars";
			this->MenuItemScrollbars->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemScrollbars_Click);
			// 
			// StatusProgressBar
			// 
			this->StatusProgressBar->Margin = System::Windows::Forms::Padding(1, 2, 1, 2);
			this->StatusProgressBar->Name = L"StatusProgressBar";
			this->StatusProgressBar->Size = System::Drawing::Size(100, 21);
			// 
			// StatusLabel1
			// 
			this->StatusLabel1->AutoSize = false;
			this->StatusLabel1->BackColor = System::Drawing::Color::Gainsboro;
			this->StatusLabel1->BorderSides = static_cast<System::Windows::Forms::ToolStripStatusLabelBorderSides>((((System::Windows::Forms::ToolStripStatusLabelBorderSides::Left | System::Windows::Forms::ToolStripStatusLabelBorderSides::Top) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Right) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Bottom));
			this->StatusLabel1->Margin = System::Windows::Forms::Padding(1, 1, 0, 0);
			this->StatusLabel1->Name = L"StatusLabel1";
			this->StatusLabel1->Size = System::Drawing::Size(130, 24);
			this->StatusLabel1->Text = L"Kurzor: ";
			this->StatusLabel1->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// StatusLabel2
			// 
			this->StatusLabel2->AutoSize = false;
			this->StatusLabel2->BorderSides = static_cast<System::Windows::Forms::ToolStripStatusLabelBorderSides>((((System::Windows::Forms::ToolStripStatusLabelBorderSides::Left | System::Windows::Forms::ToolStripStatusLabelBorderSides::Top) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Right) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Bottom));
			this->StatusLabel2->Margin = System::Windows::Forms::Padding(0, 1, 0, 0);
			this->StatusLabel2->Name = L"StatusLabel2";
			this->StatusLabel2->Size = System::Drawing::Size(220, 24);
			this->StatusLabel2->Text = L"Kijells:";
			this->StatusLabel2->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
			// 
			// StatusLabel3
			// 
			this->StatusLabel3->AutoSize = false;
			this->StatusLabel3->BorderSides = static_cast<System::Windows::Forms::ToolStripStatusLabelBorderSides>((((System::Windows::Forms::ToolStripStatusLabelBorderSides::Left | System::Windows::Forms::ToolStripStatusLabelBorderSides::Top) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Right) 
				| System::Windows::Forms::ToolStripStatusLabelBorderSides::Bottom));
			this->StatusLabel3->Margin = System::Windows::Forms::Padding(0, 1, 0, 0);
			this->StatusLabel3->Name = L"StatusLabel3";
			this->StatusLabel3->Size = System::Drawing::Size(45, 24);
			this->StatusLabel3->Text = L"100%";
			// 
			// ToolStripDropDownButtonColorMapping
			// 
			this->ToolStripDropDownButtonColorMapping->DisplayStyle = System::Windows::Forms::ToolStripItemDisplayStyle::Image;
			this->ToolStripDropDownButtonColorMapping->DropDownItems->AddRange(gcnew cli::array< System::Windows::Forms::ToolStripItem^  >(8) {this->MenuItemColorMapTitle, 
				this->toolStripMenuItem1, this->MenuItemColorMapOriginal, this->MenuItemColorMapGrayScale, this->MenuItemColorMapRed, this->MenuItemColorMapGreen, 
				this->MenuItemColorMapBlue, this->MenuItemColorMapCustom});
			this->ToolStripDropDownButtonColorMapping->Image = (cli::safe_cast<System::Drawing::Image^  >(resources->GetObject(L"ToolStripDropDownButtonColorMapping.Image")));
			this->ToolStripDropDownButtonColorMapping->ImageTransparentColor = System::Drawing::Color::Magenta;
			this->ToolStripDropDownButtonColorMapping->Name = L"ToolStripDropDownButtonColorMapping";
			this->ToolStripDropDownButtonColorMapping->Size = System::Drawing::Size(29, 23);
			this->ToolStripDropDownButtonColorMapping->Text = L"toolStripDropDownButton1";
			this->ToolStripDropDownButtonColorMapping->Visible = false;
			// 
			// MenuItemColorMapTitle
			// 
			this->MenuItemColorMapTitle->Enabled = false;
			this->MenuItemColorMapTitle->Name = L"MenuItemColorMapTitle";
			this->MenuItemColorMapTitle->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapTitle->Text = L"Csatorna szne";
			// 
			// toolStripMenuItem1
			// 
			this->toolStripMenuItem1->Name = L"toolStripMenuItem1";
			this->toolStripMenuItem1->Size = System::Drawing::Size(165, 6);
			// 
			// MenuItemColorMapOriginal
			// 
			this->MenuItemColorMapOriginal->Name = L"MenuItemColorMapOriginal";
			this->MenuItemColorMapOriginal->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapOriginal->Tag = L"0";
			this->MenuItemColorMapOriginal->Text = L"Eredeti";
			this->MenuItemColorMapOriginal->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// MenuItemColorMapGrayScale
			// 
			this->MenuItemColorMapGrayScale->Name = L"MenuItemColorMapGrayScale";
			this->MenuItemColorMapGrayScale->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapGrayScale->Tag = L"1";
			this->MenuItemColorMapGrayScale->Text = L"Szrkernyalatos";
			this->MenuItemColorMapGrayScale->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// MenuItemColorMapRed
			// 
			this->MenuItemColorMapRed->Name = L"MenuItemColorMapRed";
			this->MenuItemColorMapRed->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapRed->Tag = L"2";
			this->MenuItemColorMapRed->Text = L"Vrs";
			this->MenuItemColorMapRed->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// MenuItemColorMapGreen
			// 
			this->MenuItemColorMapGreen->Name = L"MenuItemColorMapGreen";
			this->MenuItemColorMapGreen->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapGreen->Tag = L"3";
			this->MenuItemColorMapGreen->Text = L"Zld";
			this->MenuItemColorMapGreen->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// MenuItemColorMapBlue
			// 
			this->MenuItemColorMapBlue->Name = L"MenuItemColorMapBlue";
			this->MenuItemColorMapBlue->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapBlue->Tag = L"4";
			this->MenuItemColorMapBlue->Text = L"Kk";
			this->MenuItemColorMapBlue->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// MenuItemColorMapCustom
			// 
			this->MenuItemColorMapCustom->Name = L"MenuItemColorMapCustom";
			this->MenuItemColorMapCustom->Size = System::Drawing::Size(168, 22);
			this->MenuItemColorMapCustom->Tag = L"5";
			this->MenuItemColorMapCustom->Text = L"Egyni";
			this->MenuItemColorMapCustom->Click += gcnew System::EventHandler(this, &GeoViewer::MenuItemColorMapping_Click);
			// 
			// VSc
			// 
			this->VSc->Enabled = false;
			this->VSc->Location = System::Drawing::Point(427, 1);
			this->VSc->Name = L"VSc";
			this->VSc->Size = System::Drawing::Size(16, 332);
			this->VSc->TabIndex = 9;
			this->VSc->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &GeoViewer::ControlScroll);
			// 
			// HSc
			// 
			this->HSc->Enabled = false;
			this->HSc->Location = System::Drawing::Point(0, 331);
			this->HSc->Name = L"HSc";
			this->HSc->Size = System::Drawing::Size(471, 16);
			this->HSc->TabIndex = 8;
			this->HSc->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &GeoViewer::ControlScroll);
			// 
			// ColorDialog
			// 
			this->ColorDialog->FullOpen = true;
			// 
			// CheckBoxShowVectorPoint
			// 
			this->CheckBoxShowVectorPoint->Checked = true;
			this->CheckBoxShowVectorPoint->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxShowVectorPoint->ImageIndex = 2;
			this->CheckBoxShowVectorPoint->ImageList = this->ImageListShowCheckBoxes;
			this->CheckBoxShowVectorPoint->Location = System::Drawing::Point(446, 49);
			this->CheckBoxShowVectorPoint->Name = L"CheckBoxShowVectorPoint";
			this->CheckBoxShowVectorPoint->Size = System::Drawing::Size(39, 25);
			this->CheckBoxShowVectorPoint->TabIndex = 14;
			this->CheckBoxShowVectorPoint->UseVisualStyleBackColor = true;
			this->CheckBoxShowVectorPoint->CheckStateChanged += gcnew System::EventHandler(this, &GeoViewer::CheckBoxShow_CheckStateChanged);
			// 
			// ImageListShowCheckBoxes
			// 
			this->ImageListShowCheckBoxes->ImageStream = (cli::safe_cast<System::Windows::Forms::ImageListStreamer^  >(resources->GetObject(L"ImageListShowCheckBoxes.ImageStream")));
			this->ImageListShowCheckBoxes->TransparentColor = System::Drawing::Color::Transparent;
			this->ImageListShowCheckBoxes->Images->SetKeyName(0, L"opt_image.bmp");
			this->ImageListShowCheckBoxes->Images->SetKeyName(1, L"opt_selection.bmp");
			this->ImageListShowCheckBoxes->Images->SetKeyName(2, L"opt_vector_point.bmp");
			this->ImageListShowCheckBoxes->Images->SetKeyName(3, L"opt_vector_polygon.bmp");
			this->ImageListShowCheckBoxes->Images->SetKeyName(4, L"opt_vector_polyline.bmp");
			// 
			// CheckBoxShowVectorPolyline
			// 
			this->CheckBoxShowVectorPolyline->Checked = true;
			this->CheckBoxShowVectorPolyline->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxShowVectorPolyline->ImageIndex = 4;
			this->CheckBoxShowVectorPolyline->ImageList = this->ImageListShowCheckBoxes;
			this->CheckBoxShowVectorPolyline->Location = System::Drawing::Point(446, 95);
			this->CheckBoxShowVectorPolyline->Name = L"CheckBoxShowVectorPolyline";
			this->CheckBoxShowVectorPolyline->Size = System::Drawing::Size(39, 25);
			this->CheckBoxShowVectorPolyline->TabIndex = 15;
			this->CheckBoxShowVectorPolyline->UseVisualStyleBackColor = true;
			this->CheckBoxShowVectorPolyline->CheckStateChanged += gcnew System::EventHandler(this, &GeoViewer::CheckBoxShow_CheckStateChanged);
			// 
			// CheckBoxShowVectorPolygon
			// 
			this->CheckBoxShowVectorPolygon->Checked = true;
			this->CheckBoxShowVectorPolygon->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxShowVectorPolygon->ImageIndex = 3;
			this->CheckBoxShowVectorPolygon->ImageList = this->ImageListShowCheckBoxes;
			this->CheckBoxShowVectorPolygon->Location = System::Drawing::Point(446, 72);
			this->CheckBoxShowVectorPolygon->Name = L"CheckBoxShowVectorPolygon";
			this->CheckBoxShowVectorPolygon->Size = System::Drawing::Size(39, 25);
			this->CheckBoxShowVectorPolygon->TabIndex = 16;
			this->CheckBoxShowVectorPolygon->UseVisualStyleBackColor = true;
			this->CheckBoxShowVectorPolygon->CheckStateChanged += gcnew System::EventHandler(this, &GeoViewer::CheckBoxShow_CheckStateChanged);
			// 
			// CheckBoxShowRaster
			// 
			this->CheckBoxShowRaster->Checked = true;
			this->CheckBoxShowRaster->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxShowRaster->ImageIndex = 0;
			this->CheckBoxShowRaster->ImageList = this->ImageListShowCheckBoxes;
			this->CheckBoxShowRaster->Location = System::Drawing::Point(446, 3);
			this->CheckBoxShowRaster->Name = L"CheckBoxShowRaster";
			this->CheckBoxShowRaster->Size = System::Drawing::Size(39, 25);
			this->CheckBoxShowRaster->TabIndex = 17;
			this->CheckBoxShowRaster->UseVisualStyleBackColor = true;
			this->CheckBoxShowRaster->CheckStateChanged += gcnew System::EventHandler(this, &GeoViewer::CheckBoxShow_CheckStateChanged);
			// 
			// CheckBoxShowSelection
			// 
			this->CheckBoxShowSelection->Checked = true;
			this->CheckBoxShowSelection->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxShowSelection->ImageIndex = 1;
			this->CheckBoxShowSelection->ImageList = this->ImageListShowCheckBoxes;
			this->CheckBoxShowSelection->Location = System::Drawing::Point(446, 27);
			this->CheckBoxShowSelection->Name = L"CheckBoxShowSelection";
			this->CheckBoxShowSelection->Size = System::Drawing::Size(39, 25);
			this->CheckBoxShowSelection->TabIndex = 18;
			this->CheckBoxShowSelection->UseVisualStyleBackColor = true;
			this->CheckBoxShowSelection->CheckStateChanged += gcnew System::EventHandler(this, &GeoViewer::CheckBoxShow_CheckStateChanged);
			// 
			// LabelVectorPointColor
			// 
			this->LabelVectorPointColor->BackColor = System::Drawing::Color::OrangeRed;
			this->LabelVectorPointColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->LabelVectorPointColor->Location = System::Drawing::Point(491, 54);
			this->LabelVectorPointColor->Name = L"LabelVectorPointColor";
			this->LabelVectorPointColor->Size = System::Drawing::Size(55, 14);
			this->LabelVectorPointColor->TabIndex = 19;
			this->LabelVectorPointColor->Click += gcnew System::EventHandler(this, &GeoViewer::LabelColorChoose_Click);
			// 
			// LabelVectorPolygonColor
			// 
			this->LabelVectorPolygonColor->BackColor = System::Drawing::Color::DarkOrange;
			this->LabelVectorPolygonColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->LabelVectorPolygonColor->Location = System::Drawing::Point(491, 77);
			this->LabelVectorPolygonColor->Name = L"LabelVectorPolygonColor";
			this->LabelVectorPolygonColor->Size = System::Drawing::Size(55, 14);
			this->LabelVectorPolygonColor->TabIndex = 20;
			this->LabelVectorPolygonColor->Click += gcnew System::EventHandler(this, &GeoViewer::LabelColorChoose_Click);
			// 
			// LabelVectorPolylineColor
			// 
			this->LabelVectorPolylineColor->BackColor = System::Drawing::Color::SaddleBrown;
			this->LabelVectorPolylineColor->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->LabelVectorPolylineColor->Location = System::Drawing::Point(491, 100);
			this->LabelVectorPolylineColor->Name = L"LabelVectorPolylineColor";
			this->LabelVectorPolylineColor->Size = System::Drawing::Size(55, 14);
			this->LabelVectorPolylineColor->TabIndex = 21;
			this->LabelVectorPolylineColor->Click += gcnew System::EventHandler(this, &GeoViewer::LabelColorChoose_Click);
			// 
			// ImageListColorMapping
			// 
			this->ImageListColorMapping->ImageStream = (cli::safe_cast<System::Windows::Forms::ImageListStreamer^  >(resources->GetObject(L"ImageListColorMapping.ImageStream")));
			this->ImageListColorMapping->TransparentColor = System::Drawing::Color::Transparent;
			this->ImageListColorMapping->Images->SetKeyName(0, L"colmap_original.bmp");
			this->ImageListColorMapping->Images->SetKeyName(1, L"colmap_grayscale.bmp");
			this->ImageListColorMapping->Images->SetKeyName(2, L"colmap_red.bmp");
			this->ImageListColorMapping->Images->SetKeyName(3, L"colmap_green.bmp");
			this->ImageListColorMapping->Images->SetKeyName(4, L"colmap_blue.bmp");
			this->ImageListColorMapping->Images->SetKeyName(5, L"colmap_custom.bmp");
			// 
			// GeoViewer
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->Controls->Add(this->LabelSelectionColor);
			this->Controls->Add(this->LabelVectorPolylineColor);
			this->Controls->Add(this->LabelVectorPolygonColor);
			this->Controls->Add(this->LabelVectorPointColor);
			this->Controls->Add(this->CheckBoxShowSelection);
			this->Controls->Add(this->CheckBoxShowRaster);
			this->Controls->Add(this->CheckBoxShowVectorPolygon);
			this->Controls->Add(this->CheckBoxShowVectorPolyline);
			this->Controls->Add(this->CheckBoxShowVectorPoint);
			this->Controls->Add(this->RemoveSelectedLabel);
			this->Controls->Add(this->SelectionsView);
			this->Controls->Add(this->StatusBar);
			this->Controls->Add(this->VSc);
			this->Controls->Add(this->HSc);
			this->Controls->Add(this->DestPicture);
			this->Name = L"GeoViewer";
			this->Size = System::Drawing::Size(638, 443);
			this->Load += gcnew System::EventHandler(this, &GeoViewer::GeoViewer_Load);
			this->Resize += gcnew System::EventHandler(this, &GeoViewer::FormResize);
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->DestPicture))->EndInit();
			this->StatusBar->ResumeLayout(false);
			this->StatusBar->PerformLayout();
			this->ResumeLayout(false);
			this->PerformLayout();

		}

	private: 
		void GeoViewer_Load(System::Object^  sender, System::EventArgs^  e) 
		{
			FormReSize();
		}

	
	// -------------------------------
	public: // INTERFACE FUNCTIONS -->
		void  SetVectorImage(xml_node^ image_desc)
		{
			FreeUpVectorResources();
			VectorModel^ v = gcnew VectorModel();

			int     iTmp = 0;
			String^ sTmp = "";
			double  l, t, r, b;

			try 
			{
				// BOUNDING BOX
				image_desc->XmlGetChildValueByName("BoundingBoxLeft",   sTmp = "");
				l = double::Parse( sTmp );
				image_desc->XmlGetChildValueByName("BoundingBoxTop",    sTmp = "");
				t = double::Parse( sTmp );
				image_desc->XmlGetChildValueByName("BoundingBoxRight",  sTmp = "");
				r = double::Parse( sTmp );
				image_desc->XmlGetChildValueByName("BoundingBoxBottom", sTmp = "");
				b = double::Parse( sTmp );
				v->box = RectangleF(l, t, r - l, b - t);

				// POINTS
				image_desc->XmlGetChildValueByName("PointsCount",  iTmp = 0);
				v->PointsCount      = iTmp;
				image_desc->XmlGetChildValueByName("PointsXArray", iTmp = 0);
				v->PointsXArray     = (double*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PointsYArray", iTmp = 0);
				v->PointsYArray     = (double*)(void*)iTmp;

				// POLYLINES
				image_desc->XmlGetChildValueByName("PolylinesPointCount",      iTmp = 0);
				v->PolylinesPointCount      = iTmp;
				image_desc->XmlGetChildValueByName("PolylinesPartCount",       iTmp = 0);
				v->PolylinesPartCount       = iTmp;
				image_desc->XmlGetChildValueByName("PolylinesStartArray",      iTmp = 0);
				v->PolylinesStartArray      = (int*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolylinesPartLengthArray", iTmp = 0);
				v->PolylinesPartLengthArray = (int*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolylinesXArray",          iTmp = 0);
				v->PolylinesXArray          = (double*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolylinesYArray",          iTmp = 0);
				v->PolylinesYArray          = (double*)(void*)iTmp;

				// POLYGONS
				image_desc->XmlGetChildValueByName("PolygonsPointCount",       iTmp = 0);
				v->PolygonsPointCount      = iTmp;
				image_desc->XmlGetChildValueByName("PolygonsPartCount",        iTmp = 0);
				v->PolygonsPartCount       = iTmp;
				image_desc->XmlGetChildValueByName("PolygonsStartArray",       iTmp = 0);
				v->PolygonsStartArray      = (int*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolygonsPartLengthArray",  iTmp = 0);
				v->PolygonsPartLengthArray = (int*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolygonsXArray",           iTmp = 0);
				v->PolygonsXArray          = (double*)(void*)iTmp;
				image_desc->XmlGetChildValueByName("PolygonsYArray",           iTmp = 0);
				v->PolygonsYArray          = (double*)(void*)iTmp;
			} 
			catch (Exception^ ex)
			{
				v->leavedata = true;
				delete v;
				v = nullptr;
				MessageBox::Show("Vector_Model load error: " + ex->Message);
			}

			if (!ImageDisplayed)
			{
				SrcWidth  = DestPicture->Width;
				SrcHeight = DestPicture->Height;
			}
			ImageGeoreference = v->box;
			Vector_Model = v;
			VectorDisplayed = true;
			this->Refresh();
		}

		void  FreeUpVectorResources()
		{
			VectorDisplayed = false;
			if (Vector_Model == nullptr) return;
			Vector_Model->leavedata = LeaveVectorImageData;
			delete Vector_Model;
			Vector_Model = nullptr;
		}

		void  SetVectorImageLiveOnFreeUp(bool b){ LeaveVectorImageData = b; }


		void  SetBitmapImage(xml_node^ image_desc)
		{
			FreeUpBitmapResources();

			if (image_desc == nullptr) return;

			xml_node ^t;
			int     datatmp, width, height, channels, iptmp;
			String^ strtmp;
			ImageBitsPerPixel = 0;

			// WIDTH
			if ( ! image_desc->XmlGetChildValueByName("Width", width) ) return;
			ImageWidth = width;
			ImageWidthBytes = ((ImageWidth * 3 + 3) >> 2) << 2;

			// HEIGHT
			if ( ! image_desc->XmlGetChildValueByName("Height", height) ) return;
			ImageHeight = height;

			// CHANNELCOUNT
			if ( ! image_desc->XmlGetChildValueByName("ChannelCount", channels) ) return;

			// DATA
			if ( ! image_desc->XmlGetChildValueByName("Data", iptmp) ) return;
			ImageData = (byte*)IntPtr(iptmp).ToPointer();

			// CHANNELSBITALLOCATION
			if ( ! image_desc->XmlGetChildByName(t, "ChannelsBitAllocation") ) return;

			bool l = true;
			for (int i = 0; i < channels; ++i)
			{
				if ( ! t->XmlGetChildValueByName( "Channel" + i.ToString(), datatmp ) ) return;
				ImageBitsPerPixel += datatmp;
				if (datatmp != 8) l = false;
			}
			ImageDataLoaded = true;

			if (l && ImageBitsPerPixel == 24 || ImageBitsPerPixel == 32) ImageDisplayed = true;

			ImageBitmapInfo->bmiHeader.biWidth		= ImageWidth;
			ImageBitmapInfo->bmiHeader.biHeight		= ImageHeight;
			ImageBitmapInfo->bmiHeader.biBitCount	= ImageBitsPerPixel;
			ImageBitmapInfo->bmiHeader.biClrUsed	= 0;

			bool col_mapping = false;

			if (channels == 1 && (ImageBitsPerPixel == 1 || ImageBitsPerPixel == 4 || ImageBitsPerPixel == 8))
			{
				ImageBitmapInfo->bmiHeader.biClrUsed = (1 << ImageBitsPerPixel);
				bool grayscale_color_mapping = false;
				col_mapping = true;

				if ( ! t->XmlGetChildValueByName("ColorMap", strtmp ) ) grayscale_color_mapping = true;

				if (grayscale_color_mapping)
					for (int i = 0; i < 256; i++) ImageBitmapInfo->bmiColors[i].rgbBlue = ImageBitmapInfo->bmiColors[i].rgbGreen = ImageBitmapInfo->bmiColors[i].rgbRed = i;
				else
				{
					unsigned char* ucdata;
					int            dlen;
					base64(StringTostring(strtmp), ucdata, dlen);
					memcpy(ImageBitmapInfo->bmiColors, ucdata, dlen);
					delete [] ucdata;
				}

				memcpy(ImageOriginalColormap, ImageBitmapInfo->bmiColors, 256 * sizeof(RGBQUAD));

				ImageDisplayed = true;
				ToolStripDropDownButtonColorMapping->Visible = true;
			}
			else
			{
				ToolStripDropDownButtonColorMapping->Visible = false;
			}

			SrcWidth  = ImageWidth;
            SrcHeight = ImageHeight;

			// COPY XML_Tree into myBitmap
			Byte* btmp = image_desc->XML_Tree_Marshal();
			ImageNode = gcnew xml_node( btmp );
			delete [] btmp;

			if (col_mapping)
			{
				ImageNode->XmlGetChildByName(t, "ChannelsBitAllocation");
				String^ colMap = stringToString(base64((unsigned char*)(ImageBitmapInfo->bmiColors), ImageBitmapInfo->bmiHeader.biClrUsed * sizeof(RGBQUAD)));
				t->XmlSetChildValueByName("ColorMap", colMap, false);
			}

            InitThumbnail();

            refreshSliders();
            this->Refresh();
		}
		void  RebindBitmap()
		{
			if (!ImageDisplayed) return;
            InitThumbnail();
			this->Refresh();
		}

		void  FreeUpBitmapResources()
		{
			ImageDisplayed = false;
			ImageDataLoaded = false;

			if (ImageData && !LeaveBitmapImageData) delete [] ImageData;
			ImageData = 0;

			if (ImageNode != nullptr) delete ImageNode;
			ImageNode = nullptr;
		}

		void  SetBitmapImageLiveOnFreeUp(bool b){ LeaveBitmapImageData = b; }

		bool  IsImageDataLoaded()
		{
			return ImageDataLoaded;
		}		

		bool  IsVectorDataLoaded()
		{
			return (Vector_Model != nullptr);
		}		

		void  SetGeoreference(RectangleF r){ if (Vector_Model != nullptr) Vector_Model->box = r; } // rewrite
		void  SetZoom(float zoom)
		{
			float Scaler = Zoom / 100.0F;
			float Harany, Varany;
			float DestWidth, DestHeight;
			int newValue;

			DestWidth = (float)(DestPicture->Width);
			DestHeight = (float)(DestPicture->Height);

			Harany = ((float)HSc->Value + DestWidth / 2) / ((float)SrcWidth * Scaler);
			Varany = ((float)VSc->Value + DestHeight / 2) / ((float)SrcHeight * Scaler);

			if (!HSc->Enabled) Harany = 0.5F;
			if (!VSc->Enabled) Varany = 0.5F;

			// -----------------

			Zoom = zoom;
			Scaler = Zoom / 100.0F;

			// -----------------

			if ((float)SrcWidth * Scaler < DestWidth)
			{
				HSc->Enabled = false;
				HSc->Maximum = 0;
				HSc->Value = 0;
			}
			else
			{
				HSc->Enabled = true;
				HSc->Maximum = (int)((float)SrcWidth * Scaler - DestWidth);

				newValue = (int)(Harany * (float)SrcWidth * Scaler - DestWidth / 2.0F);

				if (newValue < 0)
					HSc->Value = 0;
				else if (HSc->Maximum < newValue)
					HSc->Value = HSc->Maximum;
				else
					HSc->Value = newValue;
			}

			// -----------------

			if ((float)SrcHeight * Scaler < DestHeight)
			{
				VSc->Enabled = false;
				VSc->Maximum = 0;
				VSc->Value = 0;
			}
			else
			{
				VSc->Enabled = true;
				VSc->Maximum = (int)((float)SrcHeight * Scaler - DestHeight);

				newValue = (int)(Varany * (float)SrcHeight * Scaler - DestHeight / 2.0F);

				if (newValue < 0)
					VSc->Value = 0;
				else if (VSc->Maximum < newValue)
					VSc->Value = VSc->Maximum;
				else
					VSc->Value = newValue;
			}

			// -----------------

			Graphics^ g = DestPicture->CreateGraphics();
			g->Clear(Color::Gainsboro);
			this->Refresh();
			StatusLabel3->Text = ((int)zoom) + "%";
		}
		int   GetZoom()
		{
			return (int)Zoom;
		}
		void  SetFunctionality(int func)
		{
			ComponentFunctionality = func;
			TempSelectionPolygon.Clear();
		}
		int   GetFunctionality()
		{
			return (int)ComponentFunctionality;
		}
		bool  GetScrollVisibility()
		{
			return ScrollVisible;
		}
		void  SetScrollVisibility(bool visible)
		{
			MenuItemScrollbars->Checked = ScrollVisible = visible;
			FormReSize();
		}
		bool  GetSelectionsVisibility()
		{
			return SelectionsVisible;
		}
		void  SetSelectionsVisibility(bool visible)
		{
			MenuItemSelections->Checked = SelectionsVisible = visible;
			FormReSize();
		}		

		void  ClearSelections()
        {
            SelectionsView->Items->Clear();
            SelectionsImageList->Images->Clear();
            FreeThumbnailIndexes.Clear();
            this->Refresh();
        }

	public:
	// -------------------------------
		void SetInterfaceLanguage(xml_node^ lang)
		{
			RemoveSelectedLabel->Text		= lang->XmlGetChildStringValueByName("GeoViewerRemoveSelection");
			StrSelection					= lang->XmlGetChildStringValueByName("GeoViewerSelection");
			StrCursorPosition				= lang->XmlGetChildStringValueByName("GeoViewerPosition");
			MenuItemColorMapTitle->Text		= lang->XmlGetChildStringValueByName("GeoViewerColormappingTitle");
			MenuItemColorMapOriginal->Text	= lang->XmlGetChildStringValueByName("GeoViewerColormappingOriginal");
			MenuItemColorMapGrayScale->Text	= lang->XmlGetChildStringValueByName("GeoViewerColormappingGrayscale");
			MenuItemColorMapRed->Text		= lang->XmlGetChildStringValueByName("GeoViewerColormappingRed");
			MenuItemColorMapGreen->Text		= lang->XmlGetChildStringValueByName("GeoViewerColormappingGreen");
			MenuItemColorMapBlue->Text		= lang->XmlGetChildStringValueByName("GeoViewerColormappingBlue");
			MenuItemColorMapCustom->Text	= lang->XmlGetChildStringValueByName("GeoViewerColormappingCustom");
		}



	// -------------------------------
	private: // HELPER FUNCTIONS -->
		char* StringToCharstring(String^ s)
		{
			int l = s->Length;
			char* c = new char[l + 1];
			for (int i = 0; i < l; ++i) c[i] = (char)s[i];
			c[l] = '\0';
			return c;
		}



		Point SourceToRectangleClippedPoint(Point p, System::Drawing::Rectangle r)
        {
            p.X = p.X * r.Width / ImageWidth + r.X;
            p.Y = p.Y * r.Height / ImageHeight + r.Y;
            return p;
        }

        Point ClientToSourcePoint(Point p)
        {
            float Scaler = Zoom / 100.0F;
            p.X = (int)((float)(p.X + HSc->Value) / Scaler);
            p.Y = (int)((float)(p.Y + VSc->Value) / Scaler);
            return p;
        }
        Point SourceToClientPoint(Point p)
        {
            float Scaler = Zoom / 100.0F;
            p.X = (int)((p.X * Scaler) - HSc->Value);
            p.Y = (int)((p.Y * Scaler) - VSc->Value);
            return p;
        }
		void  ClientToSourceRectangle(System::Drawing::Rectangle& r)
        {
            float Scaler = Zoom / 100.0F;
            r.Width = (int)(r.Width / Scaler);
            r.Height = (int)(r.Height / Scaler);
            r.X = (int)((r.X + HSc->Value) / Scaler);
            r.Y = (int)((r.Y + VSc->Value) / Scaler);
        }
        void  SourceToClientRectangle(System::Drawing::Rectangle& r)
        {
            float Scaler = Zoom / 100.0F;
            r.Width = (int)(r.Width * Scaler);
            r.Height = (int)(r.Height * Scaler);
            r.X = (int)((r.X * Scaler) - HSc->Value);
            r.Y = (int)((r.Y * Scaler) - VSc->Value);
        }
        Point ValidatePoint(Point p)
        {
            if (p.X < 0) p.X = 0;
            if (p.Y < 0) p.Y = 0;
            if (p.X > ImageWidth - 1) p.X = ImageWidth - 1;
            if (p.Y > ImageHeight - 1) p.Y = ImageHeight - 1;
            return p;
        }
        Point SortRectangleCorners(Point p0, Point p1, int i)
        {
            Point p;
            p.X = (p0.X > p1.X) ? p0.X : p1.X;
            p.Y = (p0.Y > p1.Y) ? p0.Y : p1.Y;

            if (i == 0)
            {
                p.X = (p0.X < p1.X) ? p0.X : p1.X;
                p.Y = (p0.Y < p1.Y) ? p0.Y : p1.Y;
            }
            return p;
        }
		System::Drawing::Rectangle CreateRectangleFromPoints(Point p0, Point p1)
        {
            p0 = ValidatePoint(p0);
            p1 = ValidatePoint(p1);
            Point p00 = SortRectangleCorners(p0, p1, 0);
            Point p11 = SortRectangleCorners(p0, p1, 1);
            p11.X -= p00.X;
            p11.Y -= p00.Y;
			System::Drawing::Rectangle r = System::Drawing::Rectangle(p00, System::Drawing::Size(p11));
            return r;
        }
        Point IntersectLines(Point a0, Point a1, Point b0, Point b1)
        {
            int x1 = a0.X;
            int x2 = a1.X;
            int x3 = b0.X;
            int x4 = b1.X;
            int y1 = a0.Y;
            int y2 = a1.Y;
            int y3 = b0.Y;
            int y4 = b1.Y;

            double denom = (y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1);
            double ua = (x4 - x3) * (y1 - y3) - (y4 - y3) * (x1 - x3);
            double ub = (x2 - x1) * (y1 - y3) - (y2 - y1) * (x1 - x3);

            if (denom == 0.0f)
            {
                if (ua == 0.0f && ub == 0.0f)
                {
                    return Point(-1, -1); // Egybevg
                }
                return Point(-1, -1); // Prhuzamos
            }

            ua /= (double)denom;
            ub /= (double)denom;

            if (ua > 0.0f && ua < 1.0f && ub > 0.0f && ub < 1.0f)
            {
                int x = (int)(x1 + ua * (x2 - x1));
                int y = (int)(y1 + ua * (y2 - y1));
                return Point(x, y); // Metsz szakaszok
            }
            
            return Point(-1, -1); // Nem metsz szakaszok
        }
        bool IsPolygonSelfIntersected()
        {
            Graphics^ g = this->DestPicture->CreateGraphics();
            Point a0, a1, b0, b1;

            int count = TempSelectionPolygon.Count;

            for (int i = 0; i < count - 1; i++)
            {
                a0 = TempSelectionPolygon[i];
                a1 = TempSelectionPolygon[i + 1];
                
                for (int j = i + 1; j < count - 1; j++) // i s i+2 egyenes, mert szomszdosakat rtelmetlen vizsglni...
                {
                    b0 = TempSelectionPolygon[j];
                    b1 = TempSelectionPolygon[j + 1];
                    Point p = IntersectLines(a0, a1, b0, b1);

                    if ( p != Point(-1, -1)  )
                    {
                        // A rosszalkod vonalak
						g->DrawLine(gcnew Pen(Color::FromArgb(255, 0, 0)), a0, a1);
						g->DrawLine(gcnew Pen(Color::FromArgb(255, 0, 0)), b0, b1);

						Brush^ b = gcnew SolidBrush(Color::White);
                        g->FillRectangle(b, p.X - 5, p.Y - 5, 11, 11);

						MessageBox::Show(p.X + "," + p.Y);

                        return true;
                    }
                }
            }
            return false;
        }
        bool IsPointInPolygon(Point p, Object^ polygonn)
        {
            SelectionStruct^ polygon = (SelectionStruct^)polygonn;

            if (p.X < polygon->rectangle.X || p.Y < polygon->rectangle.Y) return false;
            if (p.X >= polygon->rectangle.X + polygon->rectangle.Width || p.Y >= polygon->rectangle.Y + polygon->rectangle.Height) return false;

			if (polygon->selectiontype == RECTANGLE) return true;

            int node_count = polygon->polygonpoints.Count;
            int edge_count = 0;
            int *edges     = new int[node_count];
            int ymin, ymax, y0, y1, x0, x1;

            // Els s utols pont azonos kell legyen
            for (int i = 0; i < node_count - 1; ++i)
            {
                x0 = polygon->polygonpoints[i].X;
                x1 = polygon->polygonpoints[i + 1].X;
                y0 = polygon->polygonpoints[i].Y;
                y1 = polygon->polygonpoints[i + 1].Y;
                ymin = (y0 < y1) ? y0 : y1;
                ymax = (y0 < y1) ? y1 : y0;

                if (p.Y < ymin || p.Y > ymax) continue; // A fggleges intervallumban van a pont?

                if (ymin == ymax) // Vzszintes vonal
                {
                    if (ymin != p.Y) continue; // Ez az y koordintja?

                    ymin = (x0 < x1) ? x0 : x1;
                    ymax = (x0 < x1) ? x1 : x0;

                    if (p.X < ymin || p.X > ymax) continue; // A szakaszon van a pont?

                    edges[edge_count++] = ymin;
                    continue;
                }

                int x = (int)( (double)((x1 - x0) * (p.Y - y0)) / (double)(y1 - y0) ) + x0;

                // Egy vgpontot csak 1x vehetnk listba
                if (x0 != p.X || y0 != p.Y) edges[edge_count++] = x;
            }

            // A pont bal oldaln pratlan szm hatrol l van
            int k = 0;
            for (int i = 0; i < edge_count; ++i) if (edges[i] <= p.X) k++;

			delete [] edges;

            return ((k & 1) != 0);
        }


		void InitThumbnail()
		{
			if (!ImageDisplayed) return;

            int bw = SelectionsImageList->ImageSize.Width;
            int bh = SelectionsImageList->ImageSize.Height;
            int w = (ImageWidth < ImageHeight) ? (bw * ImageWidth) / ImageHeight : bw;
            int h = (ImageWidth > ImageHeight) ? (bh * ImageHeight) / ImageWidth : bh;

			Thumbnail = gcnew Bitmap(bw, bh, PixelFormat::Format32bppArgb);
			ThumbnailClipper = System::Drawing::Rectangle((bw - w) / 2, (bh - h) / 2, w - 1, h - 1);

			Graphics^ g = Graphics::FromImage(Thumbnail);

            IntPtr hdc = g->GetHdc();
            SetStretchBltMode((HDC)(void*)hdc, 3);
			StretchDIBits( (HDC)(void*)hdc, ThumbnailClipper.Left, ThumbnailClipper.Y + ThumbnailClipper.Height-1, ThumbnailClipper.Width, -ThumbnailClipper.Height, 0, 0, ImageWidth, ImageHeight, (void*)ImageData, ImageBitmapInfo, DIB_RGB_COLORS, SRCCOPY);
            g->ReleaseHdc(hdc);

			g->DrawRectangle(gcnew Pen(Color::DarkSlateBlue), ThumbnailClipper);
		}
		void SelectionViewAddCurrentSelection()
		{
			if (!ImageDisplayed) return;
            Bitmap^ bmp = gcnew Bitmap(Thumbnail);
			Graphics^ gr = Graphics::FromImage(bmp);
            SelectionStruct^ ss = gcnew SelectionStruct();
            ss->color = LabelSelectionColor->BackColor;
			Brush^ brush = gcnew SolidBrush(Color::FromArgb(96, ss->color));
            Pen^ pen = gcnew Pen(ss->color);

			if (ComponentFunctionality == MULTIRECTANGLE_SELECT)
            {
                ss->selectiontype = RECTANGLE;
				System::Drawing::Rectangle rect = TempSelectionRectangle;
                ss->rectangle = rect;

                rect.Location = SourceToRectangleClippedPoint(rect.Location, ThumbnailClipper);
				rect.Size = (System::Drawing::Size)SourceToRectangleClippedPoint((Point)rect.Size, ThumbnailClipper);
                rect.Width -= ThumbnailClipper.X;
                rect.Height -= ThumbnailClipper.Y;

                gr->FillRectangle(brush, rect);
                gr->DrawRectangle(pen, rect);
            }
            else if (ComponentFunctionality == MULTIPOLIGON_SELECT)
            {
                ss->selectiontype = POLIGON;

                int count = TempSelectionPolygon.Count;
				cli::array<Point, 1>^ points;
				points->Resize(points, count);
                int xmin = TempSelectionPolygon[0].X;
                int xmax = xmin;
                int ymin = TempSelectionPolygon[0].Y;
                int ymax = ymin;

                for (int j = 0; j < count; ++j)
                {
                    points[j] = SourceToRectangleClippedPoint(TempSelectionPolygon[j], ThumbnailClipper);
                    ss->polygonpoints.Add(TempSelectionPolygon[j]);

                    if (TempSelectionPolygon[j].X < xmin) xmin = TempSelectionPolygon[j].X;
                    if (TempSelectionPolygon[j].Y < ymin) ymin = TempSelectionPolygon[j].Y;
                    if (TempSelectionPolygon[j].X > xmax) xmax = TempSelectionPolygon[j].X;
                    if (TempSelectionPolygon[j].Y > ymax) ymax = TempSelectionPolygon[j].Y;
                }
				ss->rectangle = System::Drawing::Rectangle(xmin, ymin, xmax - xmin + 1, ymax - ymin + 1);

				gr->FillPolygon(brush, points, System::Drawing::Drawing2D::FillMode::Alternate);
                gr->DrawPolygon(pen, points);

                TempSelectionPolygon.Clear();
            }
			
            if (FreeThumbnailIndexes.Count > 0) // Mr hasznlt, nem hasznlatban lv helyet jrahasznostjuk
            {
                SelectionsImageList->Images[FreeThumbnailIndexes[0]] = bmp;
                SelectionsView->Items->Add("", FreeThumbnailIndexes[0]);
                FreeThumbnailIndexes.Remove(FreeThumbnailIndexes[0]);
            }
            else
            {
                SelectionsImageList->Images->Add(bmp);
                SelectionsView->Items->Add("", SelectionsImageList->Images->Count - 1);
            }
            SelectionsView->Items[SelectionsView->Items->Count - 1]->Tag = (Object^)(ss);
		}
		void RemoveSelectedItems()
        {
            if (SelectionsView->SelectedItems->Count > 0)
            {
                for (int i = SelectionsView->SelectedItems->Count - 1; 0 <= i; --i)
                {
                    SelectionsView->SelectedItems[i]->Remove();
                    FreeThumbnailIndexes.Add(SelectionsView->SelectedItems->Count - i - 1);
                }
                if (SelectionsView->SelectedItems->Count == 0) FreeThumbnailIndexes.Clear();
                this->Refresh();
            }
        }

	private: // HELPER FUNCTIONS -->
		void FormReSize()
		{
			DestPicture->Width = this->Width;
			DestPicture->Height = this->Height - StatusBar->Height;

			SelectionsView->Width = 140;

			if (SelectionsVisible)
			{
				DestPicture->Width -= SelectionsView->Width;
				SelectionsView->Height = DestPicture->Height - SelectionsView->Top;
			}
			SelectionsView->Left      = DestPicture->Width + 2;
			RemoveSelectedLabel->Left = DestPicture->Width + 2;

			CheckBoxShowRaster->Left         = DestPicture->Width + 2;
			CheckBoxShowSelection->Left      = DestPicture->Width + 2;
			CheckBoxShowVectorPoint->Left    = DestPicture->Width + 2;
			CheckBoxShowVectorPolyline->Left = DestPicture->Width + 2;
			CheckBoxShowVectorPolygon->Left  = DestPicture->Width + 2;

			LabelSelectionColor->Left      = CheckBoxShowSelection->Left + CheckBoxShowSelection->Width + 6;
			LabelVectorPointColor->Left    = LabelSelectionColor->Left;
			LabelVectorPolylineColor->Left = LabelSelectionColor->Left;
			LabelVectorPolygonColor->Left  = LabelSelectionColor->Left;

			if (ScrollVisible)
			{
				DestPicture->Width -= 16;
				DestPicture->Height -= 16;
			}
			SelectionsView->Height = DestPicture->Height - SelectionsView->Top;

			HSc->Top = DestPicture->Height;
			HSc->Width = DestPicture->Width;

			VSc->Left = DestPicture->Width;
			VSc->Height = DestPicture->Height;

			if (!ImageDisplayed && VectorDisplayed)
			{
				if (Vector_Model->box.Width == 0 || Vector_Model->box.Height == 0)
				{
					SrcWidth = DestPicture->Width - 1;
					SrcHeight = DestPicture->Height - 1;
				}
				else
				{
					double ratio = Vector_Model->box.Width / Vector_Model->box.Height;
					double dw = DestPicture->Width - 1;
					double dh = DestPicture->Height - 1;

					if (dw / Vector_Model->box.Width < dh / Vector_Model->box.Height)
					{
						SrcWidth = DestPicture->Width - 1;
						SrcHeight = SrcWidth / ratio;
					}
					else
					{
						SrcHeight = DestPicture->Height - 1;
						SrcWidth = SrcHeight * ratio;
					}
				}
			}
			if (ImageDisplayed || VectorDisplayed)
			{
				refreshSliders();
				this->Refresh();
			}
		}    

		void refreshSliders()
		{
			float Scaler = Zoom / 100.0F;
			int DestWidth = this->Width - ((ScrollVisible) ? 16 : 0);
			int DestHeight = this->Height - ((ScrollVisible) ? 16 : 0);

			if (!ImageDisplayed && !VectorDisplayed) return;

			if (DestPicture->Width < (float)SrcWidth * Scaler)
			{
				HSc->Enabled = true;
				HSc->Maximum = (int)((float)SrcWidth * Scaler - DestPicture->Width);
			}
			else
			{
				HSc->Enabled = false;
				HSc->Maximum = 0;
			}

			if (DestPicture->Height < (float)SrcHeight * Scaler)
			{
				VSc->Enabled = true;
				VSc->Maximum = (int)((float)SrcHeight * Scaler - DestPicture->Height);
			}
			else
			{
				VSc->Enabled = false;
				VSc->Maximum = 0;
			}
		}

	// -------------------------------
	private: // GUI_EVENTHANDLERS -->
		void DrawSelections(System::Drawing::Graphics^ g, bool border_instead_region)
		{
			if (!CheckBoxShowSelection->Checked) return;

			Brush^ brush = gcnew SolidBrush(Color::FromArgb(96, Color::Black)); // Mindegy
			Pen^ pen = gcnew Pen(Color::Red); // Mindegy
            Color color;

            for (int i = 0; i < SelectionsView->Items->Count; ++i)
            {
                SelectionStruct^ s = (SelectionStruct^)SelectionsView->Items[i]->Tag;
                color = (SelectionsView->Items[i]->Selected) ? SelectedItemColor : s->color;
				((SolidBrush^)brush)->Color = Color::FromArgb(96, color);
                pen->Color = color;

				if (s->selectiontype == RECTANGLE)
                {
					System::Drawing::Rectangle r = s->rectangle;
                    SourceToClientRectangle(r);

                    if (border_instead_region)
                        g->DrawRectangle(pen, r);
                    else
                        g->FillRectangle(brush, r);
                }
                else if (s->selectiontype == POLIGON)
                {
                    int count = s->polygonpoints.Count;
					cli::array<Point, 1>^ points;
					points->Resize(points, count);
                    for (int j = 0; j < count; ++j) points[j] = SourceToClientPoint(s->polygonpoints[j]);

                    if (border_instead_region)
                        g->DrawPolygon(pen, points);
                    else
						g->FillPolygon(brush, points, System::Drawing::Drawing2D::FillMode::Alternate);
					points->Resize(points, 0);
                }
            }

			
			((SolidBrush^)brush)->Color = Color::FromArgb(96, ActiveRegion);
            pen->Color = ActiveBorder;
			
			if (ComponentFunctionality == MULTIPOLIGON_SELECT && TempSelectionPolygon.Count > 0)
            {
				cli::array<Point, 1>^ points;
				points->Resize(points, TempSelectionPolygon.Count + 2);

                int j = 0;
                for (j = 0; j < TempSelectionPolygon.Count; ++j) points[j] = SourceToClientPoint(TempSelectionPolygon[j]);
                points[j++] = SourceToClientPoint(Last);
                points[j] = points[0];

                if (border_instead_region)
                    g->DrawPolygon(pen, points);
                else
					g->FillPolygon(brush, points, System::Drawing::Drawing2D::FillMode::Alternate);

				points->Resize(points, 0);
            }
			
            if (ComponentFunctionality == MULTIRECTANGLE_SELECT && MouseDownLeft)
            {
				System::Drawing::Rectangle r = TempSelectionRectangle;
                SourceToClientRectangle(r);

                if (border_instead_region)
                    g->DrawRectangle(pen, r);
                else
                    g->FillRectangle(brush, r);
            }
		}


		void DrawVectorImage(System::Drawing::Graphics^ g)
		{
			if (Vector_Model == nullptr) return;

			int    hscv   = HSc->Value;
			int    vscv   = VSc->Value;
			double l      = ImageGeoreference.Left;
			double t      = ImageGeoreference.Top;
			double b      = ImageGeoreference.Bottom;
			double w      = ((double)SrcWidth)  / ImageGeoreference.Width;
			double h      = ((double)SrcHeight) / ImageGeoreference.Height;
			int    px0_, py0_;
			float  Scaler = Zoom / 100.0F;

			// POINT
			if (Vector_Model->PointsCount && CheckBoxShowVectorPoint->Checked)
			{
				int     count = Vector_Model->PointsCount;
				double* px    = Vector_Model->PointsXArray;
				double* py    = Vector_Model->PointsYArray;
				Brush^  brush = gcnew SolidBrush(Color::FromArgb(255, LabelVectorPointColor->BackColor)); // Mindegy

				for (int i = 0; i < count; i++)
				{
					double px0 = (px[i] - l) * w;
					double py0 = (b - py[i]) * h;
					px0_       = (int)((px0 * Scaler) - hscv);
					py0_       = (int)((py0 * Scaler) - vscv);

					g->FillRectangle(brush, RectangleF(px0_ - 1, py0_ - 1, 3, 3));
				}
				delete brush;
			}

			// POLYLINE
			if (Vector_Model->PolylinesPointCount && CheckBoxShowVectorPolyline->Checked)
			{
				int     count = Vector_Model->PolylinesPartCount;
				double* px    = Vector_Model->PolylinesXArray;
				double* py    = Vector_Model->PolylinesYArray;
				Brush^  brush = gcnew SolidBrush(Color::FromArgb(255, LabelVectorPolylineColor->BackColor)); // Mindegy
				Pen^    pen   = gcnew Pen(LabelVectorPolylineColor->BackColor, 1.0F); // Mindegy
				cli::array<Point, 1>^ points;

				for (int i = 0; i < count; i++)
				{
					int nodes = Vector_Model->PolylinesPartLengthArray[i];
					int start = Vector_Model->PolylinesStartArray[i];
					points->Resize(points, nodes);

					if (nodes > 1)
					{
						for (int j = 0; j < nodes; j++)
						{
							double px0 = (px[start + j] - l) * w;
							double py0 = (b - py[start + j]) * h;
							px0_       = (int)((px0 * Scaler) - hscv);
							py0_       = (int)((py0 * Scaler) - vscv);
							points[j]  = Point(px0_, py0_);
						}
						g->DrawLines(pen, points);
					}
					else
					{
						double px0 = (px[start] - l) * w;
						double py0 = (b - py[start]) * h;
						px0_       = (int)((px0 * Scaler) - hscv);
						py0_       = (int)((py0 * Scaler) - vscv);
						g->FillRectangle(brush, RectangleF(px0_ - 1, py0_ - 1, 3, 3));
					}
				}
				if (points != nullptr) delete points;
				delete pen;
				delete brush;
			}

			// POLYGON
			if (Vector_Model->PolygonsPointCount && CheckBoxShowVectorPolygon->Checked)
			{
				int     count = Vector_Model->PolygonsPartCount;
				double* px    = Vector_Model->PolygonsXArray;
				double* py    = Vector_Model->PolygonsYArray;
				Pen^    pen   = gcnew Pen(LabelVectorPolygonColor->BackColor, 1.0F); // Mindegy
				cli::array<Point, 1>^ points;

				for (int i = 0; i < count; i++)
				{
					int nodes = Vector_Model->PolygonsPartLengthArray[i];
					int start = Vector_Model->PolygonsStartArray[i];
					points->Resize(points, nodes);

					for (int j = 0; j < nodes; j++)
					{
						double px0 = (px[start + j] - l) * w;
						double py0 = (b - py[start + j]) * h;
						px0_       = (int)((px0 * Scaler) - hscv);
						py0_       = (int)((py0 * Scaler) - vscv);
						points[j]  = Point(px0_, py0_);
					}

					g->DrawPolygon(pen, points);
				}
				if (points != nullptr) delete points;
				delete pen;
			}

		}

		void OnPaint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) 
		{
			if (!ImageDisplayed && !VectorDisplayed) return;

            double Scaler = Zoom / 100.0F;

            double SourceWidth, SourceHeight, SourceLeft, SourceTop;
            int DestWidth, DestHeight, DestLeft, DestTop;

            // ImageSize calculations
            // ---------------

            DestWidth = DestPicture->Width;
            SourceWidth = (double)DestWidth / Scaler;
            if (SrcWidth < SourceWidth)
            {
                SourceWidth = SrcWidth;
                DestWidth = (int)(SourceWidth * Scaler);
                SourceLeft = 0;
                DestLeft = 0;
            }
            else
            {
                SourceLeft = SrcWidth * (HSc->Value / ((double)SrcWidth * Scaler));
                DestLeft = (DestPicture->Width - DestWidth) / 2;
            }

            // ---------------

            DestHeight = DestPicture->Height;
            SourceHeight = (double)DestHeight / Scaler;
            if (SrcHeight < SourceHeight)
            {
                SourceHeight = SrcHeight;
                DestHeight = (int)(SourceHeight * Scaler);
                SourceTop = 0;
                DestTop = 0;
            }
            else
            {
                SourceTop = SrcHeight * (VSc->Value / ((double)SrcHeight * Scaler));
                DestTop = (DestPicture->Height - DestHeight) / 2;
            }

			Graphics^ clientDC = e->Graphics;

			if (ImageDisplayed)
			{
				IntPtr hdc = clientDC->GetHdc();
				SetStretchBltMode((HDC)(void*)hdc, 3);
				if (CheckBoxShowRaster->Checked) StretchDIBits( (HDC)(void*)hdc, 0, DestHeight-1, DestWidth, -DestHeight, (int)SourceLeft, (int)SourceTop, (int)SourceWidth, (int)SourceHeight, (void*)ImageData, ImageBitmapInfo, DIB_RGB_COLORS, SRCCOPY);
				clientDC->ReleaseHdc(hdc);
				DrawSelections(clientDC, false);
				DrawSelections(clientDC, true);
			}

			if (VectorDisplayed)
			{
				DrawVectorImage(clientDC);
			}
		}

		void SelectionsView_KeyDown(System::Object^  sender, System::Windows::Forms::KeyEventArgs^  e) 
		{
			if (e->KeyCode == Keys::ControlKey) KeyCtrl = true;
			if (e->KeyCode == Keys::Delete) RemoveSelectedItems();
		}
		void SelectionsView_KeyUp(System::Object^  sender, System::Windows::Forms::KeyEventArgs^  e) 
		{
			if (e->KeyCode == Keys::ControlKey) KeyCtrl = false;
		}

		void Dest_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (!ImageDisplayed && !VectorDisplayed) return;
            float Scaler = Zoom / 100.0F;
			if (e->Button == Windows::Forms::MouseButtons::Left) MouseDownLeft = true;

            switch (ComponentFunctionality)
            {
                case IMAGE_VIEW:
                    Last = e->Location;
                    break;
                case MULTIRECTANGLE_SELECT:
                    if (e->Button == Windows::Forms::MouseButtons::Right)
                    {
                        MouseDownLeft = false;
                        this->Refresh();
                    }
                    else if (e->Button == Windows::Forms::MouseButtons::Left)
                    {
                        Last = ValidatePoint(ClientToSourcePoint(e->Location));
                        TempSelectionRectangle.Location = Last;
						TempSelectionRectangle.Size = System::Drawing::Size(0, 0);
                        MouseDownLeft = true;
                    }
                    break;

                case MULTIPOLIGON_SELECT:
                    // Belltjuk melyik gomb lett lenyomva, deeee
                    // ha msik gomb volt lenyomva, mindkett rvnytelenedik
                    if (e->Button == Windows::Forms::MouseButtons::Left)
                    {
                        MouseDownLeft = !MouseDownRight;
                        MouseDownRight = false;
                    }
                    if (e->Button == Windows::Forms::MouseButtons::Right)
                    {
                        MouseDownRight = !MouseDownLeft;
                        MouseDownLeft = false;
                    }
                    break;
                default:
                    break;
            }
		}
		void Dest_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
            if (!ImageDisplayed && !VectorDisplayed) return;

            int XX, YY;
            Point p = ValidatePoint(ClientToSourcePoint(e->Location));

            switch (ComponentFunctionality)
            {
                case IMAGE_VIEW:
                    if (MouseDownLeft)
                    {
                        XX = e->X - Last.X;
                        if (HSc->Value - XX < 0)
                            HSc->Value = 0;
                        else if (HSc->Maximum < HSc->Value - XX)
                            HSc->Value = HSc->Maximum;
                        else
                            HSc->Value = HSc->Value - XX;

                        YY = e->Y - Last.Y;
                        if (VSc->Value - YY < 0)
                            VSc->Value = 0;
                        else if (VSc->Maximum < VSc->Value - YY)
                            VSc->Value = VSc->Maximum;
                        else
                            VSc->Value = VSc->Value - YY;

                        Last = e->Location;

                        this->Refresh();
                    }
                    break;

                case MULTIPOLIGON_SELECT:
                    if (SelectedFirstPoint)
                    {
                        Last = p;
                        this->Refresh();
                    }
                    break;

                case MULTIRECTANGLE_SELECT:

                    if (MouseDownLeft)
                    {
                        TempSelectionRectangle = CreateRectangleFromPoints(Last, p);
						StatusLabel2->Text = String::Format(StrSelection, TempSelectionRectangle.X, TempSelectionRectangle.Y, TempSelectionRectangle.Width, TempSelectionRectangle.Height);
                        this->Refresh();
                    }
                    break;

                default:
                    break;
            }

			StatusLabel1->Text = String::Format(StrCursorPosition, p.X, p.Y);
		}
		void Dest_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
            if (!ImageDisplayed && !VectorDisplayed) return;

            Point p = ValidatePoint(ClientToSourcePoint(e->Location));

            switch (ComponentFunctionality)
            {
                case IMAGE_VIEW:
					if (e->Button == Windows::Forms::MouseButtons::Right) MouseDownLeft = false;
                    break;

                case MULTIRECTANGLE_SELECT:
                    if (MouseDownLeft && e->Button == Windows::Forms::MouseButtons::Left)
                    {
                        TempSelectionRectangle = CreateRectangleFromPoints(Last, p);
                        SelectionViewAddCurrentSelection();
                    }
                    MouseDownLeft = false;
                    break;

                case MULTIPOLIGON_SELECT:
                    // Bal klikk
                    if (MouseDownLeft && e->Button == Windows::Forms::MouseButtons::Left)
                    {
                        if (!SelectedFirstPoint) SelectedFirstPoint = true;
                        TempSelectionPolygon.Add(p);
                    }
                    // Jobb klikk
                    else if (MouseDownRight && e->Button == Windows::Forms::MouseButtons::Right) // utols pont kezelse: 
                    {
                        if (SelectedFirstPoint)
                        {
                            if (TempSelectionPolygon.Count > 2) // legalbb 3szg legyen (mr 2 pont van)
                            {
                                //TempSelectionPolygon.Add(p);
                                TempSelectionPolygon.Add(TempSelectionPolygon[0]);

                                // A poligon nem lehet nmagt tlapol
                                if (IsPolygonSelfIntersected())
                                {
									MessageBox::Show("A poligon hatrai nem metszhetik egymst!\nA kijells trlsre kerl...", "Kijells", MessageBoxButtons::OK, MessageBoxIcon::Warning);
                                    TempSelectionPolygon.Clear();
                                }
                                else
                                {
                                    SelectionViewAddCurrentSelection();
                                }
                            }
                            else // klnben trljk
                            {
                                TempSelectionPolygon.Clear();
                            }
                        }
                        SelectedFirstPoint = false;
                    }
                    MouseDownLeft = false;
                    MouseDownRight = false;

                    break;

                case CLICK_SELECT:
					for (int i = 0; i < SelectionsView->Items->Count; ++i)
                        SelectionsView->Items[i]->Selected = IsPointInPolygon(p, SelectionsView->Items[i]->Tag);
                    break;

                default:
                    break;
            }

            // Biztos ami biztos, ha felengedtk, fel van engedve az a gomb!!!
            // --- Csoda kizrsnak elve :) ---
            if (e->Button == Windows::Forms::MouseButtons::Right) MouseDownRight = false;
            if (e->Button == Windows::Forms::MouseButtons::Left) MouseDownLeft = false;
            StatusLabel2->Text = "";
            this->Refresh();
		}

		void FormResize(System::Object^  sender, System::EventArgs^  e) 
		{
			FormReSize();
		}
		void ControlScroll(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) 
		{
			this->Refresh();
		}


	// -------------------------------
	private: // EVENT_HANDLERS -->
		void MenuItemScrollbars_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetScrollVisibility(!MenuItemScrollbars->Checked);
		}
		void MenuItemSelections_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetSelectionsVisibility(!MenuItemSelections->Checked);
		}
		void MenuItemPan_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetFunctionality((int)IMAGE_VIEW);
		}
		void MenuItemPointClickSelect_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetFunctionality((int)CLICK_SELECT);
		}
		void MenuItemRectangleSelection_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetFunctionality((int)MULTIRECTANGLE_SELECT);
		}
		void MenuItemPolygonSelection_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			SetFunctionality((int)MULTIPOLIGON_SELECT);
		}
		void MenuItemClearSelection_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			ClearSelections();
		}
		void SelectionsView_ItemSelectionChanged(System::Object^  sender, System::Windows::Forms::ListViewItemSelectionChangedEventArgs^  e) 
		{
			this->Refresh();
		}
		void LabelColorChoose_Click(System::Object^  sender, System::EventArgs^  e) 
		{
            ColorDialog->Color = ((Label^)(sender))->BackColor;
            DialogResult res = ColorDialog->ShowDialog();
			if (res == DialogResult::OK) ((Label^)(sender))->BackColor = ColorDialog->Color;
			this->Refresh();
		}
		void RemoveSelectedLabel_LinkClicked(System::Object^  sender, System::Windows::Forms::LinkLabelLinkClickedEventArgs^  e) 
		{
			RemoveSelectedItems();
		}
		void CheckBoxShow_CheckStateChanged(System::Object^  sender, System::EventArgs^  e) 
		{
			this->Refresh();
		}

		void MenuItemColorMapping_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			ToolStripMenuItem^ mi = (ToolStripMenuItem^)(sender);
			int choice = int::Parse(mi->Tag->ToString());
			int colors = ImageBitmapInfo->bmiHeader.biClrUsed;
			Color col;

			switch (choice)
			{
			case 0: // original
				memcpy(&(ImageBitmapInfo->bmiColors[0]), ImageOriginalColormap, 256 * sizeof(RGBQUAD));
				this->Refresh();
				return;
			case 1: col = Color::FromArgb(255,255,255); break; // grayscale
			case 2: col = Color::FromArgb(255,  0,  0); break; // red
			case 3: col = Color::FromArgb(  0,255,  0); break; // green
			case 4: col = Color::FromArgb(  0,  0,255); break; // blue
			case 5: // custom
				System::Windows::Forms::ColorDialog^ coldg = gcnew System::Windows::Forms::ColorDialog();
				coldg->Color = Color::Red;
				if (coldg->ShowDialog() == System::Windows::Forms::DialogResult::Cancel) return;
				col = coldg->Color;
				break;
			}

			int rgbr = col.R;
			int rgbg = col.G;
			int rgbb = col.B;
			for (int i = 0; i < colors; i++)
			{
				ImageBitmapInfo->bmiColors[i].rgbRed   = (i * rgbr) / (colors - 1);
				ImageBitmapInfo->bmiColors[i].rgbGreen = (i * rgbg) / (colors - 1);
				ImageBitmapInfo->bmiColors[i].rgbBlue  = (i * rgbb) / (colors - 1);
			}
			this->Refresh();
		}


	// -------------------------------
	public: // CLASS_VARIABLES -->
		int							ComponentFunctionality;
		System::Drawing::Rectangle	TempSelectionRectangle;
		List<Point>					TempSelectionPolygon;
	public: // CLASS_VARIABLES -->
		// Image model

		BITMAPINFO*					ImageBitmapInfo;
		RGBQUAD*					ImageOriginalColormap;
		bool						ImageDisplayed;
		int							ImageWidth, ImageHeight, ImageWidthBytes, ImageBitsPerPixel;
		Byte*						ImageData;
		xml_node^					ImageNode;
		RectangleF					ImageGeoreference;

		// Vector model
		bool						VectorDisplayed;
		VectorModel^				Vector_Model;
		xml_node^					VectorNode(){ if (Vector_Model != nullptr) return Vector_Model->GetXmlDescription(); return nullptr; }

		// Do we have to leave images in memory on exit or changing the data?
		bool						LeaveBitmapImageData;
		bool						LeaveVectorImageData;

	private:
		int							SrcWidth, SrcHeight;
		bool						ScrollVisible, SelectionsVisible;
		bool						ImageDataLoaded;
		bool						MouseDownLeft, MouseDownRight, KeyCtrl;
		bool						SelectedFirstPoint;
		float						Zoom;
		Point						Last;
		Color						ActiveBorder, ActiveRegion, SelectedItemColor;
		Color						VectorPointColor, VectorPolylineColor, VectorPolygonColor;
		System::Drawing::Bitmap^    Thumbnail;
		System::Drawing::Rectangle  ThumbnailClipper;
		List<int> FreeThumbnailIndexes;

		String						^StrSelection, ^StrCursorPosition;

};

}
