#ifndef FAST_BIT_OPERATION_MACROS__
#define FAST_BIT_OPERATION_MACROS__

#ifndef byte
typedef unsigned char byte;
#endif

#ifndef uint
typedef unsigned int  uint;
#endif

#ifndef LE_TO_BE_INT
#define LE_TO_BE_INT(x) (((x) >> 24) | ((x) << 24) | (((x) >> 8) & 0xff00) | (((x) << 8) & 0xff0000))
#endif

#ifndef BE_TO_LE_INT
#define BE_TO_LE_INT(x) LE_TO_BE_INT(x)
#endif

#ifndef IS_POWER_OF_2
#define IS_POWER_OF_2(v) (((v) & ((v) - 1)) == 0)
#endif

#ifndef LIMITER
#define LIMITER(v, a, b) (((v) < (a)) ? (a) : (((v) > (b)) ? (b) : (v)))
#endif

#ifndef REVERSE_BITS
#define REVERSE_BITS(b) (((((b) * 0x0802LU & 0x22110LU) | ((b) * 0x8020LU & 0x88440LU)) * 0x10101LU >> 16) & 0xff)
#endif

#ifndef GET_BITPIXEL_OF_ROW
#define GET_BITPIXEL_OF_ROW(p,x) (((p)[(x) >> 3] >> (((x) & 7) ^ 7)) & 1)
#endif

#ifndef SET_BITPIXEL_OF_ROW
#define SET_BITPIXEL_OF_ROW(p,x) {(p)[(x) >> 3] |= 1 << (((x) & 7) ^ 7);}
#endif

#ifndef SET_WHITEBITPIXEL_OF_ROW
#define SET_WHITEBITPIXEL_OF_ROW(p,x) {(p)[(x) >> 3] |= 1 << (((x) & 7) ^ 7);}
#endif

#ifndef SET_BLACKBITPIXEL_OF_ROW
#define SET_BLACKBITPIXEL_OF_ROW(p,x) {(p)[(x) >> 3] &= ~(1 << (((x) & 7) ^ 7));}
#endif

#ifndef LO_SUINT
#define LO_SUINT(x) ((x) & 0xffff)
#endif

#ifndef HI_SUINT
#define HI_SUINT(x) (((x) >> 16) & 0xffff)
#endif

#ifndef coord_struct
#define coord_struct
struct coord{
	int x, y;
	coord(){}
	coord(int mx, int my) : x(mx), y(my) {}
};
#endif

#ifndef REMOVE_POINT
#define REMOVE_POINT(img, w, x, y) *((img) + ((w) * (y) + (x))) = 0 
#endif


// az rdekes pixel rtke 1 lesz, a httr 0
// bitplane esetben a fekete pixelek lesznek rdekesek
#define GET_NEIGHBOURS_BITPLANE(IMG, X, Y, W, H, M_S, M_RES) \
{ \
	unsigned int RES = 0; \
	for (int M_Y = -1; M_Y <= 1; M_Y++) \
	{ \
		int Y_POS = LIMITER(Y + M_Y, 0, H - 1); \
		if (Y_POS != Y + M_Y) \
		{ \
			RES = (RES << 3) | 7; \
			continue; \
		} \
		byte* ROW = IMG + Y_POS * M_S; \
		for (int M_X = -1; M_X <= 1; M_X++) \
		{ \
			RES <<= 1; \
			int X_POS = LIMITER(X + M_X, 0, W - 1); \
			if (X_POS != X + M_X) RES |= 1; \
			else			      RES |= ((ROW[X_POS >> 3] >> ((X_POS & 7) ^ 7)) & 1); \
		} \
	} \
	M_RES = ~(((RES >> 1) & 0xf0) | (RES & 0x0f)); \
}


// Az rdekes pixelek rtke 1 lesz, a tbbiek pedig 0 lesz
#define GET_NEIGHBOURS_BYTEPLANE(IMG, W, H, X, Y, M_S, FOREGROUND, M_RES) \
{ \
	unsigned int RES = 0; \
	for (int M_Y = -1; M_Y <= 1; M_Y++) \
	{ \
		int Y_POS = LIMITER(Y + M_Y, 0, H - 1); \
		if (Y_POS != Y + M_Y) \
		{ \
			RES <<= 3; \
			continue; \
		} \
		byte* row = IMG + Y_POS * M_S; \
		for (int M_X = -1; M_X <= 1; M_X++) \
		{ \
			RES <<= 1; \
			int X_POS = LIMITER(X + M_X, 0, W - 1); \
			if (X_POS != X + M_X) continue; \
			RES |= (row[X_POS] == FOREGROUND) ? 1 : 0; \
		} \
	} \
	M_RES = ((RES >> 1) & 0xf0) | (RES & 0x0f); \
}



// INVERTLT BITRTKEKET AD VISSZA, S A HATRPONTOKAT MSOLJA A HATRON KVLI HELYEKRE
#define NEIGHBORHOOD_QUERY( IMG, X, Y, mW, mH, mRes ) \
{ \
	int            mX     = X - 1; \
	int            mY     = Y - 1; \
	unsigned int   mB, mN = 0; \
	unsigned char* pIMG   = IMG + LIMITER(mY, 0, mH - 1) * s; \
	unsigned char  mVALUE; \
	\
	if ( mX < 0 || mX > mW - 3 ) /* A BAL S JOBB OLDALI SZLEK FELDOLGOZSA */ \
	{ \
		for (int mI = 0; mI < 3; mI++) \
		{ \
			mVALUE = 0; \
			mN  <<= 3; \
			\
			if (mX < 0) /* KZPPONT BAL SZLEN VAN */ \
			{ \
				mVALUE = ((0xff * (pIMG[0] >> 7)) | pIMG[0]) >> (8 + mX); \
			} \
			else if (mX == mW - 2) /* KZPPONT JOBB SZLEN VAN */ \
			{ \
				mVALUE  = ((pIMG[mX >> 3] >> (7 - (mX & 7))) << 1) | (pIMG[(mX + 1) >> 3] >> (7 - ((mX + 1) & 7))); \
				mVALUE |= ((mVALUE << 1) & 4); \
			} \
			else if (mX == mW - 1) /* KZPPONT JOBB SZLTL EGGYEL JOBBRA VAN */ \
			{ \
				mVALUE  = pIMG[mX >> 3] >> (7 - (mX & 7)); \
				mVALUE |= (mVALUE << 1) | (mVALUE << 2); \
			} \
			mN |= mVALUE; \
			if ((mI == 0 && mY >= 0) || (mI == 1 && mY < mH - 2)) pIMG += s; /* ALS/FELS HATRON NINCS LPTETS */ \
		} \
	} \
	else \
	{ \
		pIMG += (mX >> 3); \
		for (int mI = 0; mI < 3; mI++) \
		{ \
			/* BITEK SORRENDJT MEGCSERLJK, BJTOK HELYIRTKT FELCSERLJK */ \
			mB = ((REVERSE_BITS(pIMG[0]) | (((mX & 7) > 5) ? (REVERSE_BITS(pIMG[1]) << 8) : 0)) >> (mX & 7)) & 0x7; \
			mN |= mB << (mI * 3); \
			if ((mI == 0 && mY >= 0) || (mI == 1 && mY < mH - 2)) pIMG += s; /* ALS/FELS HATRON NINCS LPTETS */   \
		} \
	} \
	mN   = (((mN >> 1) & 0xf0) | (mN & 0x0f)); /* A 9 BITBL A KZPST KIDOBJUK, 1<->0 CSERE */ \
	mRes = ~(REVERSE_BITS(mN)); /* mRes should be a byte type */ \
}



#endif


