
// nCoreAPI VERSION 1.3.1
// RELEASE DATE: 2010.11.25.
// MAJOR FIXES: SUPPORTS ANY INPUT AND OUTPUT PIXEL- AND IMAGESIZES FOR PARALLELIZATION
// NON-INTERLEAVED AND NON-PACKED IMAGES ARE NOT SUPPORTED! (THEY SHOULD BE USED WITHOUT PARALLELIZATION)

#ifndef NCOREAPI_H
#define NCOREAPI_H

#pragma once
#pragma warning (disable:4244)
#pragma warning (disable:4211) // pointer truncation from 'void *' to 'int'
#pragma warning (disable:4212) // conversion from 'int' to 'void *' of greater size


using namespace System;
using namespace System::Threading;
#include <string.h>
#include <stdlib.h>
#include <math.h>

#ifndef MIN_CPUCORE_USAGE_IN_PIXEL
#define MIN_CPUCORE_USAGE_IN_PIXEL 10000
#endif

#define IMAGEFILTER_API_VERSION    VER_GEN(1,2,0,0)
#define IS_SET_BIT(x,b) (((x) & (1 << (b))) ? 1 : 0)

//#define NCORE_API_DEBUG_INFOS

namespace FilterSpace {

	struct polygon_node { int x, y; };

	class ImageFilterAccelerator;

	bool FilterCore(filtersettings*, int, int, int, int, Semaphore^);
	typedef bool (*filter_core_func)(filtersettings*, int, int, int, int, Semaphore^);

	

	public ref class nCoreProcessor
	{
	public:
		nCoreProcessor(Semaphore^ synchronizer, Semaphore^ updater, filter_core_func _func, filtersettings* _fset, int _x, int _y, int _w, int _h) 
			: sync(synchronizer), upd(updater), func(_func), fset(_fset), x(_x), y(_y), w(_w), h(_h)
		{
			thread = gcnew Thread(gcnew ThreadStart( this, &nCoreProcessor::ThreadEntryPoint ) );
			thread->IsBackground = true;
			thread->Start();
		}

		~nCoreProcessor(){ delete thread; }

	private:
		void ThreadEntryPoint() 
		{ 
			func(fset, x, y, w, h, upd); 
			sync->Release(1);
		}

		filter_core_func func;
		filtersettings*  fset;
		int              x, y, w, h;
		Semaphore^       sync;
		Semaphore^       upd;
		Thread^          thread;
	};

	public class MultiThreadTaskVariables
	{
	public:
		MultiThreadTaskVariables(filtersettings* f, bool box_filtering) : box_filter(box_filtering), fset(f)
		{
			cpu_cores	= MIN(System::Environment::ProcessorCount, (f->width * f->height) / MIN_CPUCORE_USAGE_IN_PIXEL);
			width		= f->width;
			height		= f->height;
			stride		= f->strideout;
			img			= f->datain;
			Init_Filter();

			#ifdef NCORE_API_DEBUG_INFOS
			String^ s = "";
			s += "radius_width:  " + radius_width.ToString()  + "\n";
			s += "radius_height: " + radius_height.ToString() + "\n";
			s += "block_macroblock_width:  " + block_macroblock_width.ToString()  + "\n";
			s += "block_macroblock_height: " + block_macroblock_height.ToString() + "\n";
			s += "block_microblock_width:  " + block_microblock_width.ToString()  + "\n";
			s += "block_microblock_height: " + block_microblock_height.ToString() + "\n";
			s += "block_macroblock_width_bytesize_in:  " + block_macroblock_width_bytesize_in.ToString()  + "\n";
			s += "block_macroblock_width_bytesize_out: " + block_macroblock_width_bytesize_out.ToString() + "\n";
			s += "block_microblock_width_bytesize_in:  " + block_microblock_width_bytesize_in.ToString()  + "\n";
			s += "block_microblock_width_bytesize_out: " + block_microblock_width_bytesize_out.ToString() + "\n";

			s += "\nSpan Columns(" + columns.ToString() + "):\n";
			for (int i = 0; i < columns - 1; i++) s += "[" + SpanColumnPixelPos[i].ToString()  + "," + BlockColumnPixelPos[i+1].ToString() + ")\n";

			s += "\nBlock Columns(" + columns.ToString() + "):\n";
			for (int i = 0; i < columns; i++)     s += "[" + BlockColumnPixelPos[i].ToString() + "," + SpanColumnPixelPos[i].ToString() + ")\n";

			s += "\nSpan Rows(" + rows.ToString() + "):\n";
			for (int i = 0; i < rows    - 1; i++) s += "[" + SpanRowPixelPos[i].ToString()     + "," + BlockRowPixelPos[i+1].ToString() + ")\n";

			s += "\nBlock Rows(" + rows.ToString() + "):\n";
			for (int i = 0; i < rows; i++)        s += "[" + BlockRowPixelPos[i].ToString()    + "," + SpanRowPixelPos[i].ToString() + ")\n";

			System::Windows::Forms::MessageBox::Show(s);
			#endif
		}

		~MultiThreadTaskVariables()
		{
			if (box_filter)
			{
				delete [] FirstPassResultRows;
				delete [] FirstPassResultColumns;
				delete [] BlockColumnBytePos;
				delete [] BlockRowBytePos;
				delete [] SpanColumnBytePos;
				delete [] SpanColumnPixelPos;
				delete [] SpanRowBytePos;
				delete [] SpanRowPixelPos;
			}
			delete [] CpuCoresSettings;
			delete [] BlockColumnPixelPos;
			delete [] BlockRowPixelPos;
		}

		void Init_BoxColumnTasks()
		{
			if (!box_filter) return;
			for (int j = 0; j < columns - 1; j++)
			{
				memcpy(&(CpuCoresSettings[rows + j]), fset, sizeof(filtersettings));
				CpuCoresSettings[rows + j].dataout       = FirstPassResultColumns + j * spx;
				CpuCoresSettings[rows + j].strideout     = sps;
				CpuCoresSettings[rows + j].nCoreThreadId = j;
			}
		}

		void Init_BoxRowTasks()
		{
			if (!box_filter) return;
			for (int i = 0; i < rows - 1; i++)
			{
				memcpy(&(CpuCoresSettings[i]), fset, sizeof(filtersettings));
				CpuCoresSettings[i].dataout       = FirstPassResultRows + i * radius_height * stride;
				CpuCoresSettings[i].nCoreThreadId = i;
			}
		}

		void Init_BoxMatrixTasks()
		{
			if (!box_filter) return;
			for (int i = 0; i < rows; i++)
			for (int j = 0; j < columns; j++)
			{
				memcpy(&(CpuCoresSettings[i * columns + j]), fset, sizeof(filtersettings));
				CpuCoresSettings[i * columns + j].dataout += BlockRowBytePos[i] + BlockColumnBytePos[j];
				CpuCoresSettings[i * columns + j].nCoreThreadId = i * columns + j;
			}
		}

		void FinalizeBoxFilter()
		{
			Init_BoxRowTasks();
			Init_BoxColumnTasks();
			
			for (int i = 0; i < rows - 1; i++) 
				memcpy(fset->dataout + stride * SpanRowPixelPos[i], FirstPassResultRows + i * radius_height * stride, stride * radius_height);
			
			for (int i = 0; i < columns - 1; i++) 
				for (int j = 0; j < height; j++) 
					memcpy(fset->dataout + j * stride + SpanColumnBytePos[i], FirstPassResultColumns + j * sps + i * spx, spx);
		}

		void Init_BlockMatrixTasks()
		{
			if (box_filter) return;
			for (int y = 0; y < rows; y++)
			{
				int y_start = block_macroblock_height *  y;
				for (int x = 0; x < columns; x++)
				{
					memcpy(&(CpuCoresSettings[y * columns + x]), fset, sizeof(filtersettings));
					CpuCoresSettings[y * columns + x].dataout      += stride * y_start + x * block_macroblock_width_bytesize_out;
					CpuCoresSettings[y * columns + x].nCoreThreadId = y * columns + x;
				}
			}
		}


	private:
		int CalculateLKKT(int a, int b)
		{
			int a0 = a, b0 = b;
			while (a0 != b0)
			{
				if (a0 < b0) a0 += a;
				else         b0 += b;
			}
			return a0;
		}

		void Init_Filter()
		{
			int n_min, blocks_in_row, blocks_in_col, radius, blocks_in_rowspan, min_area_size, curr_area_size, span_x;

			if (box_filter) // BOX FILTER
			{
				block_microblock_height = radius	= fset->nCoreExecutionSize;  // radius
				block_microblock_width_bytesize_in  = fset->pixelsizein;         // bits size of an "in"  pixel
				block_microblock_width_bytesize_out = fset->pixelsizeout;        // bits size of an "out" pixel
				min_area_size						= fset->width * fset->height + 1;
				// LKKT w/ 8 (byte align)
				while (block_microblock_width_bytesize_in  & 0x7) block_microblock_width_bytesize_in  <<= 1;  
				while (block_microblock_width_bytesize_out & 0x7) block_microblock_width_bytesize_out <<= 1;
				// combined byte aligned pixel amount
				block_microblock_width				= CalculateLKKT( (block_microblock_width_bytesize_in / fset->pixelsizein), (block_microblock_width_bytesize_out / fset->pixelsizeout) );
				block_microblock_width_bytesize_in  = (block_microblock_width * fset->pixelsizein)  >> 3;
				block_microblock_width_bytesize_out = (block_microblock_width * fset->pixelsizeout) >> 3;
				
				// block in row, column and rowspan
				blocks_in_rowspan = (radius + block_microblock_width - 1) / block_microblock_width; // pixelwidth >= radius for vertical spans
				blocks_in_row     = width  / block_microblock_width;
				blocks_in_col     = height / radius;

				// byte size of a "span"
				span_x            = blocks_in_rowspan * block_microblock_width_bytesize_out;

				// TRY TO USE ALL EFFECTIVELY USABLE CPU CORES
				
				for (columns = cpu_cores; columns >= 1; --columns)
				{
					rows = cpu_cores / columns;
					 // width of block in pixel
					curr_area_size  = ((blocks_in_row - (columns - 1) * blocks_in_rowspan) / columns) * block_microblock_width;
					 // height of block for cpu core in pixel
					curr_area_size *= (height - (rows - 1) * radius) / rows;
					curr_area_size += (columns - 1) * blocks_in_rowspan;
					curr_area_size += (rows - 1)    * radius;
					if (curr_area_size < min_area_size)
					{
						min_area_size = curr_area_size;
						n_min         = columns;
					}
				}

				columns = n_min;
				rows    = cpu_cores / columns;

				// block sizes in pixels for blockwidth, blockheight, spanwidth

				CpuCoresSettings					= new filtersettings[rows * columns];
				block_macroblock_width				= ((blocks_in_row - (columns - 1) * blocks_in_rowspan) / columns) * block_microblock_width;
				block_macroblock_width_bytesize_in  = (block_macroblock_width / block_microblock_width) * block_microblock_width_bytesize_in;
				block_macroblock_width_bytesize_out = (block_macroblock_width / block_microblock_width) * block_microblock_width_bytesize_out;
				block_macroblock_height				= ((blocks_in_col - (rows - 1) * radius) / rows);
				radius_width						= blocks_in_rowspan * block_microblock_width;
				radius_height						= radius;
				spx									= span_x;
				sps									= (columns - 1)   * span_x;
				FirstPassResultColumns				= new byte[sps    * height];
				FirstPassResultRows					= new byte[stride * (rows - 1) * radius_height];
				BlockColumnBytePos					= new int[columns];
				BlockColumnPixelPos					= new int[columns];
				BlockRowBytePos						= new int[rows];
				BlockRowPixelPos					= new int[rows];
				SpanColumnBytePos					= new int[columns];
				SpanColumnPixelPos					= new int[columns];
				SpanRowBytePos						= new int[rows];
				SpanRowPixelPos						= new int[rows];
				SpanColumnPixelPos[columns - 1]		= width;  // VIRTUAL SPAN, USED AS END POINT TO THE LAST BLOCK
				SpanRowPixelPos[   rows    - 1]		= height;

				// FOR FAST SPAN AND BLOCK INDEXING
				for (int i = 0; i < columns - 1; i++) 
				{
					SpanColumnPixelPos[i] = (i + 1) * block_macroblock_width + i * radius_width;
					SpanColumnBytePos[i]  = (SpanColumnPixelPos[i] * fset->pixelsizeout) >> 3;
				}
				for (int i = 0; i < rows - 1; i++) 
				{
					SpanRowPixelPos[i] = (i + 1) * block_macroblock_height + i * radius_height;
					SpanRowBytePos[i]  = SpanRowPixelPos[i] * stride;
				}
				for (int i = 0; i < columns; i++)
				{
					BlockColumnPixelPos[i] = i * (block_macroblock_width + radius_width);
					BlockColumnBytePos[i]  = (BlockColumnPixelPos[i] * fset->pixelsizeout) >> 3;
				}
				for (int i = 0; i < rows; i++)     
				{
					BlockRowPixelPos[i] = i * (block_macroblock_height + radius_height);
					BlockRowBytePos[i]  = BlockRowPixelPos[i] * stride;
				}
			}
			else // PIXELWISE / BLOCK FILTER
			{
				block_microblock_height				= fset->nCoreExecutionSize;
				block_microblock_width_bytesize_in  = block_microblock_height * fset->pixelsizein;  // bits size of an "in"  block by default
				block_microblock_width_bytesize_out = block_microblock_height * fset->pixelsizeout; // bits size of an "out" block by default
				// LKKT w/ 8 (byte align)
				while (block_microblock_width_bytesize_in  & 0x7) block_microblock_width_bytesize_in  <<= 1;  
				while (block_microblock_width_bytesize_out & 0x7) block_microblock_width_bytesize_out <<= 1;
				 // pixels in an "in" and "out" block
				block_microblock_width = CalculateLKKT( (block_microblock_width_bytesize_in / fset->pixelsizein), (block_microblock_width_bytesize_out / fset->pixelsizeout) );
				block_microblock_width_bytesize_in  = (block_microblock_width * fset->pixelsizein)  >> 3;
				block_microblock_width_bytesize_out = (block_microblock_width * fset->pixelsizeout) >> 3;

				blocks_in_row                       = width  / block_microblock_width;
				blocks_in_col                       = height / block_microblock_height;

				// TRY TO USE ALL EFFECTIVELY USABLE CPU CORES
				min_area_size = blocks_in_row * blocks_in_col;
				for (columns = cpu_cores; columns >= 1; --columns)
				{
					rows = cpu_cores / columns;
					int curr_area_size = (blocks_in_row / columns) * (blocks_in_col / rows);
					if (curr_area_size < min_area_size)
					{
						min_area_size = curr_area_size;
						n_min         = columns;
					}
				}

				columns = n_min;
				rows    = cpu_cores / columns;
				block_macroblock_width              = (blocks_in_row / columns) * block_microblock_width;
				block_macroblock_height             = (blocks_in_col / rows)    * block_microblock_height;
				block_macroblock_width_bytesize_in  = (block_macroblock_width / block_microblock_width) * block_microblock_width_bytesize_in;
				block_macroblock_width_bytesize_out = (block_macroblock_width / block_microblock_width) * block_microblock_width_bytesize_out;

				CpuCoresSettings					= new filtersettings[rows * columns];
				BlockColumnPixelPos					= new int[columns + 1];
				BlockRowPixelPos					= new int[rows + 1];
				BlockColumnPixelPos[columns]		= width;
				BlockRowPixelPos[rows]				= height;
				for (int i = 0; i < rows;    i++) BlockRowPixelPos[i]    = block_macroblock_height * i;
				for (int i = 0; i < columns; i++) BlockColumnPixelPos[i] = block_macroblock_width  * i;
			}
		}

	public:
		// spx = byte size of the width of a divider column between two vertical image column
		// sps = byte size of the row of FirstPassResultColumns
		int             rows, columns, width, height, stride, radius_width, radius_height, spx, sps;
		byte           *img;

		// variables for block and pixelwise filtering
		int             block_macroblock_width,             block_macroblock_height;
		int             block_microblock_width,             block_microblock_height;
		int             block_macroblock_width_bytesize_in, block_macroblock_width_bytesize_out;
		int             block_microblock_width_bytesize_in, block_microblock_width_bytesize_out;

		// variables for box filtering
		int            *SpanRowPixelPos,     *SpanColumnPixelPos,  *SpanRowBytePos,  *SpanColumnBytePos;
		int            *BlockRowPixelPos,    *BlockColumnPixelPos, *BlockRowBytePos, *BlockColumnBytePos;
		byte		   *FirstPassResultRows, *FirstPassResultColumns;
		filtersettings *CpuCoresSettings;

	private:
		int             cpu_cores;
		filtersettings *fset;
		bool            box_filter;
	};

	public ref class ImageFilterBase
	{
	public:
		//
		// BEGIN ---> -------------- DO NOT MODIFY --------------
		//
		//
		// BEGIN: DO NOT CALL FROM OUTSIDE THIS MODULE --> // MARSHALLING NOT POSSIBLE IMPLICITLY
		static int     GetOneImageSetting(xml_node^ fs, int& Width, int& Height, int& Stride, int& Packed, int& Interleaved, int& ChannelCount, int& PixelSize, byte*& Data)
		{
			int tmp;
			if (fs == nullptr) return 1;

			if (! fs->XmlGetChildValueByName("Width", Width) ) return 10;
			if (! fs->XmlGetChildValueByName("Height", Height) ) return 11;
			if (! fs->XmlGetChildValueByName("Stride", Stride) ) return 12;
			if (! fs->XmlGetChildValueByName("Packed", Packed) ) return 13;
			if (! fs->XmlGetChildValueByName("Interleaved", Interleaved) ) return 14;
			if (! fs->XmlGetChildValueByName("ChannelCount", ChannelCount) ) return 15;
			if (! fs->XmlGetChildValueByName("PixelSize", PixelSize) ) return 16;
			if (! fs->XmlGetChildValueByName("Data", tmp) ) return 17;
			Data = (byte*)(void*)tmp;

			return 0;
		}
		static void    CreateImageSetting(xml_node^ fs, String^ pin_name, int Width, int Height, int Stride, int Packed, int Interleaved, int ChannelCount, int PixelSize, byte* Data)
		{
			if (fs == nullptr) return;
			fs->children.Clear();
			fs->children.Add( gcnew xml_node(fs, "PinName", "", pin_name) );
			fs->children.Add( gcnew xml_node(fs, "Width", "", Width.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Height", "", Height.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Stride", "", Stride.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Packed", "", Packed.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Interleaved", "", Interleaved.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "ChannelCount", "", ChannelCount.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "PixelSize", "", PixelSize.ToString()) );
			fs->children.Add( gcnew xml_node(fs, "Data", "", ((int)(void*)Data).ToString()) );
			fs->children.Add( gcnew xml_node(fs, "ChannelsBitAllocation", "", "" ) );
		}

		static void    SetChannelsBitAllocation(xml_node^ fs, int Channel, int BitAllocation)
		{
			if (fs == nullptr) return;
			xml_node^ v;
			if (! fs->XmlGetChildByName(v, "Channel" + Channel.ToString()) )
				fs->children.Add( gcnew xml_node( fs, "Channel" + Channel.ToString(), "", BitAllocation.ToString() ) );
			else
				v->XmlSetIntValue( BitAllocation );
		}
		static bool    GetMaskPtr(xml_node^ fs, byte*& mask)
		{
			int tmp;
			xml_node^ xn;
			if (fs == nullptr) return false;
			if (! fs->XmlGetChildByName(xn, "Mask") ) 
			{
				mask = 0;
				return true;
			}
			if (! xn->XmlGetChildValueByName("Data", tmp) ) 
			{
				mask = 0;
				return false;
			}
			mask = (byte*)(void*)tmp;
			return true;
		}
		static void    ClipImage(xml_node^ fs, int sx, int sy, int sw, int sh)
		{
			if (fs == nullptr) return;

			int Width, Height;
			int PixelSizeIn, PackedIn, InterleavedIn, ChannelsIn, StrideIn;
			int PixelSizeOut, PackedOut, InterleavedOut, ChannelsOut, StrideOut;
			byte *DataIn, *DataOut;
			xml_node^ xn;

			// RETRIEVING INPUT INFORMATIONS
			if (! fs->XmlGetChildByName(xn, "InputImage0") ) return; // NEED SUPPORT LATER FOR MORE INPUTS FOR PREVIEWING
			if ( GetOneImageSetting(xn, Width, Height, StrideIn, PackedIn, InterleavedIn, ChannelsIn, PixelSizeIn, DataIn) ) return;

			// RETRIEVING OUTPUT INFORMATIONS
			if (! fs->XmlGetChildByName(xn, "OutputImage0") ) return;
			if ( GetOneImageSetting(xn, Width, Height, StrideOut, PackedOut, InterleavedOut, ChannelsOut, PixelSizeOut, DataOut) ) return;

			// ------- >

			if (!DataIn || !DataOut) return;
			if (PackedIn != PackedOut) return;
			if (InterleavedIn != InterleavedOut) return;
			if (PixelSizeIn != PixelSizeOut) return;

			if (sx < 0) sx = 0;
			if (sy < 0) sy = 0;
			if (sx >= Width) sx = Width - 1;
			if (sy >= Width) sy = Height - 1;

			int width = (sw) ? ((sx + sw - 1 < Width) ? sw : (Width - sx)) : (Width);
			int height = (sh) ? ((sy + sh - 1 < Height) ? sh : (Height - sy)) : (Height);
			byte *din = DataIn + (StrideIn * sy) + (sx * PixelSizeIn);
			byte *dout = DataOut;
			PixelSizeIn /= 8;

			for (int y = 0; y < height; ++y)
			{
				memcpy(dout, din, PixelSizeIn * width);
				din += StrideIn;
				dout += StrideOut;
			}
		}

		static void    ClipImage(filtersettings* f, int sx, int sy, int sw, int sh)
		{
			if (!f) return;
			if (f->pixelsizein != f->pixelsizeout) return;
		
			if (sx < 0) sx = 0;
			if (sx >= f->width) sx = f->width - 1;
			if (sy < 0) sy = 0;
			if (sy >= f->height) sy = f->height - 1;
			int width = (sw) ? ((sx + sw - 1 < f->width) ? sw : (f->width - sx)) : (f->width);
			int height = (sh) ? ((sy + sh - 1 < f->height) ? sh : (f->height - sy)) : (f->height);
			int sin = f->stridein;
			int sout = f->strideout;
			int shift = f->channelcountin;
			byte *din = f->datain + (sin * sy) + (sx * shift);
			byte *dout = f->dataout;

			for (int y = 0; y < height; ++y)
			{
				memcpy(dout, din, shift * width);
				din += sin;
				dout += sout;
			}
		} 
		static void    UpdateDestinationPointer(xml_node^ filtersetting, int stride, IntPtr data)
		{
			if (filtersetting == nullptr) return;
			
			xml_node^ xn;
			if ( !filtersetting->XmlGetChildByName(xn, "OutputImage0") ) return;

			if ( !xn->XmlGetChildByName(xn, "Stride") ) return;
			xn->XmlSetIntValue(stride);

			xn = xn->parent;

			if ( !xn->XmlGetChildByName(xn, "Data") ) return;
			xn->XmlSetIntPtrValue(data);
		}


		inline static int  GetPixelValue(byte* img, int s, int x, int y, int bits)
		{
			int  v  = 0;
			img    += s * y;
			switch (bits)
			{
			case 1:
				v = (img[x >> 3] & (1 << (~x & 7))) ? 1 : 0;
				break;
			case 4:
				v = (img[x >> 1] >> ((x & 1) << 2)) & 0x0f;
				break;
			case 8:
				v = img[x];
				break;
			case 24:
				((byte*)(&v))[0] = img[3*x+0];
				((byte*)(&v))[1] = img[3*x+1];
				((byte*)(&v))[2] = img[3*x+2];
				break;
			}
			return v;
		}

		inline static void SetPixelValue(byte* img, int s, int x, int y, int bits_src, int bits_dst, int v)
		{
			img += s * y;
			byte tmp;

			v = v & ((1 << bits_src) - 1);

			switch (bits_src)
			{
			case 1:
				tmp = (v) ? 255 : 0;
				if      (bits_dst == 4 || bits_dst == 8) v = tmp >> (8 - bits_dst);
				else if (bits_dst == 24)                 v = (tmp << 16) | (tmp << 8) | tmp;
				break;
			case 4:
				if      (bits_dst == 1)  v = v >> 3;
				else if (bits_dst == 8)	 v = v << 1;
				else if (bits_dst == 24) v = (v << 3) | (v << 11) | (v << 19);
				break;
			case 8:
				if      (bits_dst == 1)  v = v >> 7;
				else if (bits_dst == 4)	 v = v >> 4;
				else if (bits_dst == 24) v = v | (v << 8) | (v << 16);
				break;
			case 24:
				if      (bits_dst != 24) v = ((v & 0xff) + ((v >> 8) & 0xff) + ((v >> 16) & 0xff)) / 3;
				if      (bits_dst == 1)  v = v >> 7;
				else if (bits_dst == 4)	 v = v >> 4;
				break;
			}

			switch (bits_dst)
			{
			case 1:
				v           = v << (~x & 7);
				tmp         = ~(1 << (~x & 7));
				img[x >> 3] = (img[x >> 3] & tmp) | v;
				break;
			case 4:
				v            = v << ((x & 1) ? 0 : 4);
				img[x >> 1]  = (img[x >> 1] & ((x & 1) ? 0xf0 : 0x0f)) | v;
				break;
			case 8:
				img[x]      = v;
				break;
			case 24:
				img[3*x+0]  = ((byte*)(&v))[0];
				img[3*x+1]  = ((byte*)(&v))[1];
				img[3*x+2]  = ((byte*)(&v))[2];
				break;
			}
		}





		// n!Core MULTITHREADING *****************************************************************************************

		static void UpdateFilterStatus(filtersettings* f, int processed_pixels, Semaphore^ updater)
		{
			// FOR PERFORMANCE MEASUREMENT
			f->nCoreProcessedPixelsTemp += processed_pixels;

			if (f->processing_ptr == 0) return;
			if (updater != nullptr && f->nCoreProcessedPixelsTemp > MIN_CPUCORE_USAGE_IN_PIXEL)
			{
				if (updater->WaitOne(5, false))
				{
					*(f->processing_ptr) += f->nCoreProcessedPixelsTemp;
					f->nCoreProcessedPixelsTemp = 0;
					updater->Release(1);
				}
			}	
			else
			{
				if (f->processing_ptr) *(f->processing_ptr) += f->nCoreProcessedPixelsTemp;
				f->nCoreProcessedPixelsTemp = 0;
			}
		}


		// EXTENSIVE TEST SHOULD BE MADE!!!

		static bool ExecuteFilterBlock(filtersettings* fsets, filter_core_func FilterCorePtr)
		{
			// if image is small, it is not efficient to parallelize the filter
			if ( (fsets->width * fsets->height) / MIN_CPUCORE_USAGE_IN_PIXEL == 0 || fsets->nCoreEnabled < 1) return FilterCorePtr(fsets, 0, 0, fsets->width, fsets->height, nullptr);

			MultiThreadTaskVariables	mttv(fsets, false);
			int							rows      = mttv.rows;
			int							columns   = mttv.columns;
			int							width     = mttv.width;
			int							height    = mttv.height;

			cli::array<nCoreProcessor^>^ nCoreProcessors = gcnew cli::array<nCoreProcessor^>(rows * columns);
			Semaphore^ synchronizer = gcnew Semaphore(0, rows * columns);
			Semaphore^ updater = gcnew Semaphore(1, 1);

			mttv.Init_BlockMatrixTasks();
			for (int y = 0; y < rows; y++)
			for (int x = 0; x < columns; x++)
			{
				int sx = mttv.BlockColumnPixelPos[x];
				int sy = mttv.BlockRowPixelPos[y];
				int sw = mttv.BlockColumnPixelPos[x + 1] - sx;
				int sh = mttv.BlockRowPixelPos[y + 1] - sy;
				nCoreProcessors[y * columns + x] = gcnew nCoreProcessor(synchronizer, updater, FilterCorePtr, &(mttv.CpuCoresSettings[y * columns + x]), sx, sy, sw, sh);
			}

			// SYNCHRONIZATION
			for (int i = 0; i < rows * columns; i++) synchronizer->WaitOne();
			for (int i = 0; i < rows * columns; i++) delete nCoreProcessors[i];
			delete nCoreProcessors;
			delete synchronizer;
			delete updater;
			return true;
		}

		static bool ExecuteFilterBox(filtersettings* fsets, filter_core_func FilterCorePtr)
		{
			// if image is small, it is not efficient to parallelize the filter
			if ( (fsets->width * fsets->height) / MIN_CPUCORE_USAGE_IN_PIXEL == 0 || fsets->nCoreEnabled < 1) return FilterCorePtr(fsets, 0, 0, fsets->width, fsets->height, nullptr);

			// if radius is not positive, filter should be run pixelwise
			if ( fsets->nCoreExecutionSize < 1 )
			{
				int  tmp = fsets->nCoreExecutionSize;
				fsets->nCoreExecutionSize = 1;
				bool res = ExecuteFilterBlock(fsets, FilterCorePtr);
				fsets->nCoreExecutionSize = tmp;
				return res;
			}

			int i, j;
			MultiThreadTaskVariables mttv = MultiThreadTaskVariables(fsets, true);
			int rows    = mttv.rows;
			int columns = mttv.columns;
			int width   = mttv.width;
			int height  = mttv.height;
			int stride  = mttv.stride;

			// if processor splitting failed, run filter sequentially
			if (rows * columns == 1) return FilterCorePtr(fsets, 0, 0, width, height, nullptr);

			//////////////////////////////////////////////////////////
			// Start to act like a parallel box filter

			cli::array<nCoreProcessor^>^ nCoreProcessors = gcnew cli::array<nCoreProcessor^>(rows * columns);
			Semaphore^ updater = gcnew Semaphore(1, 1);          // 1 process allowed to use synchronizer maximum and initially
			Semaphore^ synchronizer;

			// CALCULATE CRITICAL SPANS

			// ROW SPANS
			#ifdef NCORE_API_DEBUG_INFOS
			System::Windows::Forms::MessageBox::Show("Running rowspans");
			#endif
			mttv.Init_BoxRowTasks();
			synchronizer = gcnew Semaphore(0, rows);
			for (i = 0; i < rows - 1; i++) nCoreProcessors[i] = gcnew nCoreProcessor(synchronizer, updater, FilterCorePtr, &(mttv.CpuCoresSettings[i]), 0, mttv.SpanRowPixelPos[i], stride, mttv.radius_height);
			for (i = 0; i < rows - 1; i++) synchronizer->WaitOne();
			delete synchronizer;

			// COLUMN SPANS
			#ifdef NCORE_API_DEBUG_INFOS
			System::Windows::Forms::MessageBox::Show("Running columnspans");
			#endif
			mttv.Init_BoxColumnTasks();
			synchronizer = gcnew Semaphore(0, columns);
			for (j = 0; j < columns - 1; j++) nCoreProcessors[j] = gcnew nCoreProcessor(synchronizer, updater, FilterCorePtr, &(mttv.CpuCoresSettings[rows + j]), mttv.SpanColumnPixelPos[j], 0, mttv.radius_width, height);
			for (j = 0; j < columns - 1; j++) synchronizer->WaitOne();
			delete synchronizer;
						
			// CALCULATE BLOCKS

			#ifdef NCORE_API_DEBUG_INFOS
			System::Windows::Forms::MessageBox::Show("Running blocks[" + (rows * columns).ToString() + "]");
			#endif
			mttv.Init_BoxMatrixTasks();
			synchronizer = gcnew Semaphore(0, rows * columns); // 0 process allowed to do something initially, max m * n do at the same time
			for (i = 0; i < rows; i++)
			for (j = 0; j < columns; j++)
				nCoreProcessors[i * columns + j] = gcnew nCoreProcessor(synchronizer, updater, FilterCorePtr, &(mttv.CpuCoresSettings[i * columns + j]), mttv.BlockColumnPixelPos[j], mttv.BlockRowPixelPos[i], mttv.SpanColumnPixelPos[j] - mttv.BlockColumnPixelPos[j], mttv.SpanRowPixelPos[i] - mttv.BlockRowPixelPos[i]);

			// SYNCHRONIZATION
			for (i = 0; i < rows * columns; i++) synchronizer->WaitOne();
			for (i = 0; i < rows * columns; i++) delete nCoreProcessors[i];
			delete nCoreProcessors;
			delete synchronizer;
			delete updater;

			mttv.FinalizeBoxFilter();
			
			return true;
		}
	};




	public class ImageFilterAccelerator
	{
	public:
		// height parameter is the real height of the image of portion to be virtual.
		// It does not contain the optional lines before and after the portion.
		// These lines should be setted via prelines and postlines parameters.
		// image should be set to the beginning of the corresponding image portion
		// instead of the first preline.
		// Your filter can address from GetVirtualImage[0] to GetVirtualImage[prelines+height+postlines-1]
		// where the first line of the image portion is GetVirtualImage[prelines]

		ImageFilterAccelerator(filtersettings* f, int& sx, int& sy, int& sw, int& sh, int _radius)
		{
			int i;

			radius         = _radius;
			stride         = f->stridein;
			int   channels = f->channelcountin;

			sx = INTERVAL_LIMITER(sx, 0, f->width - 1);
			sy = INTERVAL_LIMITER(sy, 0, f->height - 1);

			if (sw < 1) sw = f->width - sx;
			if (sh < 1) sh = f->height - sy;

			sw = INTERVAL_LIMITER(sw, 1, f->width - sx);
			sh = INTERVAL_LIMITER(sh, 1, f->height - sy);

			x_pre       = MIN(sx, radius);
			x_post      = MIN(f->width - (sx + sw), radius);
			prelines    = MIN(sy, radius);
			postlines   = MIN(f->height - (sy + sh), radius);
			height      = sh;
			scanline    = prelines - 1;
			bytewidth   = (sw + x_pre + x_post) * channels;
			byte* image = f->datain + sy * stride + (sx - x_pre) * channels;

			buffer        = new byte*[radius + 1];
			virtual_image = new byte*[height + prelines + postlines];

			for (i = 0; i <= radius; ++i) buffer[i] = new byte[bytewidth];

			for (i = 0; i < prelines; ++i) 
			{
				virtual_image[i] = buffer[i];
				memcpy(buffer[i], image + (i - prelines) * stride, bytewidth);
			}

			for (i = 0; i < height + postlines; ++i) virtual_image[prelines + i] = image + i * stride;
		}

		~ImageFilterAccelerator()
		{
			for (int i = 0; i <= radius; ++i) delete [] buffer[i];
			delete [] buffer;
			delete [] virtual_image;
		}

		void StepImageScanLine()
		{
			int buff_p = ++scanline % (radius + 1);
			memcpy(buffer[buff_p], virtual_image[scanline], bytewidth);
			virtual_image[scanline] = buffer[buff_p];
		}

		byte** GetVirtualImage(){ return virtual_image; }

		int  GetPreX(){ return x_pre; }
		int  GetPreY(){ return prelines; }
		int  GetPostX(){ return x_post; }
		int  GetPostY(){ return postlines; }

	private:
		int x_pre, x_post;
		int bytewidth, height, prelines, postlines, stride, scanline, radius;
		byte **virtual_image, **buffer;
	};
}


#endif
