#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;

#include "imagefilter.h"

namespace FilterTemplate {


	public ref class SettingsForm : public System::Windows::Forms::Form
	{

	public:
		filtersettings* FilterSetting;
		bool FilterCancelled;
		bool MouseButtonDown;

		Bitmap^ PreviewBitmap;
		Rectangle SourceRect, DestinationRect;
		int SWidth, SHeight;
		int LastX, LastY;

		System::Drawing::Imaging::PixelFormat pFormat;

	public:
		SettingsForm(filtersettings* fs)
		{
			InitializeComponent();

			FilterSetting = fs;

			SWidth = fs->width;
			SHeight = fs->height;

			FilterCancelled = true;

			if (fs->channelcountin == 1)
			{
				PreviewBitmap = gcnew Bitmap(Preview->ClientSize.Width, Preview->ClientSize.Height, pFormat = PixelFormat::Format8bppIndexed);
				ColorPalette^ cPal = PreviewBitmap->Palette;
				for (int i = 0; i < 256; i++) cPal->Entries[i] = Color::FromArgb(i, i, i);
				PreviewBitmap->Palette = cPal;
			}
			else
			{
				PreviewBitmap = gcnew Bitmap(Preview->ClientSize.Width, Preview->ClientSize.Height, pFormat = PixelFormat::Format24bppRgb);
			}

			DestinationRect.X = (SWidth < Preview->ClientSize.Width) ? ((Preview->ClientSize.Width - SWidth) / 2) : 0;
			DestinationRect.Y = (SHeight < Preview->ClientSize.Height) ? ((Preview->ClientSize.Height - SHeight) / 2) : 0;
			DestinationRect.Width = (DestinationRect.X) ? SWidth : Preview->ClientSize.Width;
			DestinationRect.Height = (DestinationRect.Y) ? SHeight : Preview->ClientSize.Height;

			SourceRect.X = 0;
			SourceRect.Y = 0;
			SourceRect.Width  = DestinationRect.Width;  // SHOULD BE EQUAL !!!
			SourceRect.Height = DestinationRect.Height; // SHOULD BE EQUAL !!!

			MouseButtonDown = false;
		}

	protected:
		~SettingsForm()
		{
			if (components)
			{
				delete components;
			}
		}


	private: 
		System::Windows::Forms::Button^			ButtonApply;
		System::Windows::Forms::Button^			ButtonCancel;
		System::Windows::Forms::PictureBox^		Preview;
		System::Windows::Forms::RadioButton^	DetectorSelect1;
		System::Windows::Forms::RadioButton^	DetectorSelect2;
		System::Windows::Forms::RadioButton^	DetectorSelect3;
		System::Windows::Forms::RadioButton^	DetectorSelect4;
		System::ComponentModel::IContainer^		components;


	private:

#pragma region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->Preview = (gcnew System::Windows::Forms::PictureBox());
			this->ButtonApply = (gcnew System::Windows::Forms::Button());
			this->ButtonCancel = (gcnew System::Windows::Forms::Button());
			this->DetectorSelect1 = (gcnew System::Windows::Forms::RadioButton());
			this->DetectorSelect2 = (gcnew System::Windows::Forms::RadioButton());
			this->DetectorSelect3 = (gcnew System::Windows::Forms::RadioButton());
			this->DetectorSelect4 = (gcnew System::Windows::Forms::RadioButton());
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->BeginInit();
			this->SuspendLayout();
			// 
			// Preview
			// 
			this->Preview->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->Preview->Cursor = System::Windows::Forms::Cursors::NoMove2D;
			this->Preview->Location = System::Drawing::Point(12, 12);
			this->Preview->Name = L"Preview";
			this->Preview->Size = System::Drawing::Size(154, 154);
			this->Preview->TabIndex = 0;
			this->Preview->TabStop = false;
			this->Preview->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseDown);
			this->Preview->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseMove);
			this->Preview->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &SettingsForm::Preview_OnPaint);
			this->Preview->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseUp);
			// 
			// ButtonApply
			// 
			this->ButtonApply->Location = System::Drawing::Point(183, 143);
			this->ButtonApply->Name = L"ButtonApply";
			this->ButtonApply->Size = System::Drawing::Size(75, 23);
			this->ButtonApply->TabIndex = 2;
			this->ButtonApply->Text = L"Apply";
			this->ButtonApply->UseVisualStyleBackColor = true;
			this->ButtonApply->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonApply_Click);
			// 
			// ButtonCancel
			// 
			this->ButtonCancel->Location = System::Drawing::Point(264, 143);
			this->ButtonCancel->Name = L"ButtonCancel";
			this->ButtonCancel->Size = System::Drawing::Size(75, 23);
			this->ButtonCancel->TabIndex = 3;
			this->ButtonCancel->Text = L"Cancel";
			this->ButtonCancel->UseVisualStyleBackColor = true;
			this->ButtonCancel->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonCancel_Click);
			// 
			// DetectorSelect1
			// 
			this->DetectorSelect1->AutoSize = true;
			this->DetectorSelect1->Checked = true;
			this->DetectorSelect1->Location = System::Drawing::Point(206, 29);
			this->DetectorSelect1->Name = L"DetectorSelect1";
			this->DetectorSelect1->Size = System::Drawing::Size(96, 17);
			this->DetectorSelect1->TabIndex = 4;
			this->DetectorSelect1->TabStop = true;
			this->DetectorSelect1->Text = L"Laplace Type-I";
			this->DetectorSelect1->UseVisualStyleBackColor = true;
			this->DetectorSelect1->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::DetectorSelect_Changed);
			// 
			// DetectorSelect2
			// 
			this->DetectorSelect2->AutoSize = true;
			this->DetectorSelect2->Location = System::Drawing::Point(206, 52);
			this->DetectorSelect2->Name = L"DetectorSelect2";
			this->DetectorSelect2->Size = System::Drawing::Size(99, 17);
			this->DetectorSelect2->TabIndex = 5;
			this->DetectorSelect2->Text = L"Laplace Type-II";
			this->DetectorSelect2->UseVisualStyleBackColor = true;
			this->DetectorSelect2->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::DetectorSelect_Changed);
			// 
			// DetectorSelect3
			// 
			this->DetectorSelect3->AutoSize = true;
			this->DetectorSelect3->Location = System::Drawing::Point(206, 75);
			this->DetectorSelect3->Name = L"DetectorSelect3";
			this->DetectorSelect3->Size = System::Drawing::Size(52, 17);
			this->DetectorSelect3->TabIndex = 6;
			this->DetectorSelect3->Text = L"Sobel";
			this->DetectorSelect3->UseVisualStyleBackColor = true;
			this->DetectorSelect3->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::DetectorSelect_Changed);
			// 
			// DetectorSelect4
			// 
			this->DetectorSelect4->AutoSize = true;
			this->DetectorSelect4->Location = System::Drawing::Point(206, 98);
			this->DetectorSelect4->Name = L"DetectorSelect4";
			this->DetectorSelect4->Size = System::Drawing::Size(57, 17);
			this->DetectorSelect4->TabIndex = 7;
			this->DetectorSelect4->Text = L"Prewitt";
			this->DetectorSelect4->UseVisualStyleBackColor = true;
			this->DetectorSelect4->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::DetectorSelect_Changed);
			// 
			// SettingsForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->BackColor = System::Drawing::Color::Silver;
			this->ClientSize = System::Drawing::Size(490, 178);
			this->Controls->Add(this->DetectorSelect4);
			this->Controls->Add(this->DetectorSelect3);
			this->Controls->Add(this->DetectorSelect2);
			this->Controls->Add(this->DetectorSelect1);
			this->Controls->Add(this->ButtonCancel);
			this->Controls->Add(this->ButtonApply);
			this->Controls->Add(this->Preview);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = L"SettingsForm";
			this->ShowIcon = false;
			this->ShowInTaskbar = false;
			this->Text = L"Edge Detectors";
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->EndInit();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion

	public:
		void RefreshPreview(bool applyfilter)
		{
			BitmapData^ bd = PreviewBitmap->LockBits( Rectangle(0, 0, PreviewBitmap->Width, PreviewBitmap->Height), ImageLockMode::ReadWrite, pFormat);

			FilterSetting->strideout = bd->Stride;
			FilterSetting->dataout = (Byte*)(void*)bd->Scan0;

			if (applyfilter)
				FilterSpace::ImageFilter::FilterCore(FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);
			else
				FilterSpace::ImageFilter::ClipImage( FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);

			PreviewBitmap->UnlockBits(bd);

			this->Refresh();
		}

	private: 
		System::Void ButtonApply_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			FilterCancelled = false;
			this->Close();
		}

		System::Void ButtonCancel_Click(System::Object^  sender, System::EventArgs^  e) { this->Close(); }

		System::Void Preview_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) 
			{
				MouseButtonDown = true;
				LastX = e->X;
				LastY = e->Y;
				RefreshPreview(false);
			}
		}

		System::Void Preview_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (!MouseButtonDown) return;
			if (!DestinationRect.X)
			{
				SourceRect.X -= e->X - LastX;
				if (SourceRect.X < 0) SourceRect.X = 0;
				if (SourceRect.X > SWidth - DestinationRect.Width) SourceRect.X = SWidth - DestinationRect.Width ;
			}
			if (!DestinationRect.Y)
			{
				SourceRect.Y -= e->Y - LastY;
				if (SourceRect.Y < 0) SourceRect.Y = 0;
				if (SourceRect.Y > SHeight - DestinationRect.Height) SourceRect.Y = SHeight - DestinationRect.Height;
			}
			RefreshPreview(false);
			LastX = e->X;
			LastY = e->Y;
		}

		System::Void Preview_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) MouseButtonDown = false;
			if (!MouseButtonDown) RefreshPreview(true);
		}

		System::Void Preview_OnPaint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) 
		{
			Graphics^ g = e->Graphics;
			Rectangle r = Rectangle(0, 0, DestinationRect.Width, DestinationRect.Height);
			g->DrawImage(PreviewBitmap, DestinationRect, r, GraphicsUnit::Pixel);
		}

		System::Void DetectorSelect_Changed(System::Object^  sender, System::EventArgs^  e) 
		{
			if (DetectorSelect1->Checked)
				FilterSetting->filter_method = 0;

			else if (DetectorSelect2->Checked)
				FilterSetting->filter_method = 1;

			else if (DetectorSelect3->Checked)
				FilterSetting->filter_method = 2;

			else
				FilterSetting->filter_method = 4;

			RefreshPreview(true);
		}
};
}
