#pragma once
#pragma warning(disable:4267) // conversion from 'size_t' to 'unsigned int', possible loss of data

#include "util_defs.h"
using namespace System;

// CONVERT FUNCTIONS  : UPCAST

byte* convert_bitplane_to_byteplane(byte* bitplane, int width, int height, int stridein, int strideout)
{
	// bit=0 -> byte=1, bit=1 -> byte=0
	byte* res = new byte[strideout * height];
	for (int y = 0; y < height; y++)
		for (int x = 0; x < width; x++)
			res[strideout * y + x] = ((bitplane[stridein * y + (x >> 3)] >> (((x) & 7) ^ 7)) & 1) ^ 1;
	return res;
}

byte* convert_byteplane_to_rgbplane(byte* byteplane, int width, int height, int stridein, int strideout)
{
	byte* res = new byte[strideout * height];
	for (int y = 0; y < height; y++)
	{
		byte* r = res + y * strideout;
		byte* p = byteplane + y * stridein;
		for (int x = 0; x < width; x++)
		{
			if (*p)
				r[0] = r[1] = r[2] = 0;
			else
				r[0] = r[1] = r[2] = 255;
			r += 3;
			p++;
		}
	}
	return res;
}

byte* convert_bitplane_to_rgbplane(byte* bitplane, int width, int height, int stridein, int strideout)
{
	byte* res = new byte[strideout * height];
	for (int y = 0; y < height; y++)
	{
		byte* p = bitplane + y * stridein;
		byte* r = res + y * strideout;
		for (int x = 0; x < width; x++)
		{
			byte v = ((p[x >> 3] >> (((x) & 7) ^ 7)) & 1) * 255;
			r[0] = r[1] = r[2] = v;
			r += 3;
		}
	}
	return res;
}

// CONVERT FUNCTIONS  : DOWNCAST

byte* convert_rgbplane_to_bitplane(byte* rgbplane, int width, int height, int stridein, int strideout)
{
	byte* res = new byte[strideout * height];
	for (int y = 0; y < height; y++)
	{
		byte* p = rgbplane + y * stridein;
		byte* r = res + y * strideout;
		for (int x = 0; x < width; x++)
		{
			byte mask = 1 << ((x & 7) ^ 7);
			r[x >> 3] = (r[x >> 3] & ~mask) | (*p & mask); // rgb value 0 or 255 -> good to AND by it
			p += 3;
		}
	}
	return res;
}

// ---------------------------------------------------------------------------------------------

int compare( const void *arg1, const void *arg2 )
{
	int a = *(int*)arg1;
	int b = *(int*)arg2;
	if (a < b)
		return -1;
	else if (a > b)
		return 1;
	return 0;
}

std::string StringTostring(String^ s)
{
	int l = s->Length;
	std::string s2 = "";
	for (int i = 0; i < l; ++i) s2 += (char)s[i];
	return s2;
}

String^ stringToString(std::string s)
{
	String^ s2 = "";
	unsigned int l = s.length();
	for (unsigned int i = 0; i < l; i++) s2 += Char(s[i]);
	return s2;			
}

char* StringToCharstring(String^ s)
{
	int l = s->Length;
	char* c = new char[l + 1];
	for (int i = 0; i < l; ++i) c[i] = (char)s[i];
	c[l] = '\0';
	return c;
}


// ---------------------------------------------------------------------------------------------
// BASE64  ENCODER/DECODER

const char Base64Enc[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
char       Base64Dec[256];

std::string base64(const unsigned char* in, int length)
{
	std::string s = "";
	int len3 = (length + 2) / 3;
	for (int i = 0; i < len3; i++)
	{
		int pos   = i * 3;
		int len   = (length - pos < 3) ? length - pos : 3;
		s += Base64Enc[ in[pos + 0] >> 2 ];
		s += Base64Enc[ ((in[pos + 0] & 0x03) << 4) | ((in[pos + 1] & 0xf0) >> 4) ];
		s += (unsigned char) (len > 1 ? Base64Enc[ ((in[pos + 1] & 0x0f) << 2) | ((in[pos + 2] & 0xc0) >> 6) ] : '=');
		s += (unsigned char) (len > 2 ? Base64Enc[ in[pos + 2] & 0x3f ] : '=');
	}
	return s;
}

void base64(std::string strdata, unsigned char*& data, int& length)
{
	length = (strdata.length() * 3) / 4;
	if (length < 1) return;
	data   = new unsigned char[length];
	length /= 3;

	for (int i = 0; i < 64; i++) Base64Dec[Base64Enc[i]] = i;
	for (int i = 0; i < length; i++)
	{
		data[i * 3 + 0] = (unsigned char)((Base64Dec[strdata[i * 4 + 0]] << 2) | (Base64Dec[strdata[i * 4 + 1]] >> 4));
		data[i * 3 + 1] = (unsigned char)((Base64Dec[strdata[i * 4 + 1]] << 4) | (Base64Dec[strdata[i * 4 + 2]] >> 2));
		data[i * 3 + 2] = (unsigned char)((Base64Dec[strdata[i * 4 + 2]] << 6) | (Base64Dec[strdata[i * 4 + 3]]));
	}
	length *= 3;
}




// ---------------------------------------------------------------------------------------------
// CPU FEATURE DETECTION FUNCTIONS

#include <intrin.h>
#include <string.h>

bool Is_Supported_MMX()
{
	int CPUInfo[4];
	int nId = 1 << 22;
	__cpuid(CPUInfo, 1);
	return (CPUInfo[3] & nId) != 0;
}

bool Is_Supported_SSE()
{
	int CPUInfo[4];
	int nId = 1 << 24;
	__cpuid(CPUInfo, 1);
	return (CPUInfo[3] & nId) != 0;
}

bool Is_Supported_SSE2()
{
	int CPUInfo[4];
	int nId = 1 << 25;
	__cpuid(CPUInfo, 1);
	return (CPUInfo[3] & nId) != 0;
}

bool Is_Supported_SSE3()
{
	int CPUInfo[4];
	__cpuid(CPUInfo, 1);
	return (CPUInfo[2] & 1) != 0;
}


std::string Get_Processor_Name_Helper()
{
	std::string res = ""; 

    int  CPUInfo[4] = {-1};
    char CPUBrandString[0x40] = { 0 };
    __cpuid(CPUInfo, 0x80000000);
    unsigned int nExIds = CPUInfo[0];

    for( unsigned int i=0x80000002; i<=nExIds; ++i)
    {
        __cpuid(CPUInfo, i);
		if (i <= 0x80000004) memcpy( CPUBrandString + (i - 0x80000002) * 16, CPUInfo, 16);
	}

	for (int i = 0, cnt = 0; i < 0x40; i++)
	{
		if (CPUBrandString[i] == 0) break;
		if (cnt == 0 || CPUBrandString[i] != ' ') res += CPUBrandString[i];
		cnt = (CPUBrandString[i] == ' ') ? 1 : 0;
	}

	return res;
}

String^ Get_Processor_Name()
{
	return stringToString(Get_Processor_Name_Helper());
}

