#pragma once
#pragma warning(disable:4311) // pointer truncation from 'void *' to 'int'
#pragma warning(disable:4312) // conversion from 'int' to 'void *' of greater size


#ifndef XML_NODE_CLASS_H
#define XML_NODE_CLASS_H

#include "util_defs.h"

using namespace System;
using namespace System::IO;
using namespace System::Collections::Generic;

public ref class xml_node
{
public: // METHODS
	xml_node(void* marshalled) { XML_Tree_DeMarshal( (byte*)marshalled ); }

	xml_node(xml_node^ _parent, String^ _name, String^ _attrs, String^ _value)
	{
		parent = _parent;
		name = _name;
		attributes = _attrs;
		value = _value;
	}

	~xml_node() 
	{
		for (int i = 0; i < children.Count; i++) delete children[i]; 
		children.Clear(); 
	}

	bool XmlIsChildExist(String^ tag)
	{
		for (int i = 0; i < children.Count; ++i) if (children[i]->name->ToUpper() == tag->ToUpper()) return true;
		return false;
	}

	bool XmlGetIntValue(int& ivalue) 
	{
		try{
			ivalue = System::Convert::ToInt32(value); 
		} catch (Exception^) { return false; }
		return true;
	}
	void XmlSetIntValue(int& ivalue) { value = ivalue.ToString(); }
	bool XmlGetIntPtrValue(IntPtr& ip)
	{
		try{
			ip = (IntPtr)((void*)System::Convert::ToInt32(value));  
		} catch (Exception^) { return false; }
		return true;
	}
	void XmlSetIntPtrValue(IntPtr ip) { value = ((int)(void*)ip).ToString(); }
	void XmlAddTag(String^ tag) { children.Add(gcnew xml_node(this, tag, "", "")); }
	bool XmlGetChildByName(xml_node^& node, String^ tag)
	{
		for (int i = 0; i < children.Count; ++i)
		{
			if (children[i]->name->ToUpper() == tag->ToUpper())
			{
				node = children[i];
				return true;
			}
		}
		return false;
	}

	void XmlRemoveChildByName(String^ tag)
	{
		for (int i = 0; i < children.Count; ++i)
		{
			if (children[i]->name->ToUpper() == tag->ToUpper())
			{
				delete children[i];
				children.RemoveAt(i);
				break;
			}
		}
	}

	bool XmlGetChildValueByName(String^ tag, int& value)
	{
		xml_node^ xn;
		if (!XmlGetChildByName(xn, tag)) return false;
		value = System::Convert::ToInt32(xn->value);
		return true;
	}
	bool XmlGetChildValueByName(String^ tag, String^& value)
	{
		xml_node^ xn;
		if (!XmlGetChildByName(xn, tag)) return false;
		value = xn->value;
		return true;
	}
	String^ XmlGetChildStringValueByName(String^ tag)
	{
		String^ s;
		s = "";
		xml_node^ xn;
		if (!XmlGetChildByName(xn, tag)) return s;
		return xn->value;
	}

	void XmlSetChildValueByName(String^ tag, int value)
	{
		for (int i = 0; i < children.Count; ++i)
		{
			if (children[i]->name->ToUpper() == tag->ToUpper())
			{
				children[i]->value = value.ToString();
				return;
			}
		}
		children.Add( gcnew xml_node(this, tag, "", value.ToString()) );
	}
	void XmlSetChildValueByName(String^ tag, int value, bool keepprevious)
	{
		for (int i = 0; i < children.Count; ++i)
		{
			if (children[i]->name->ToUpper() == tag->ToUpper())
			{
				if (!keepprevious) 
				{
					children[i]->value = value.ToString();
				}
				return;
			}
		}
		children.Add( gcnew xml_node(this, tag, "", value.ToString()) );
	}

	void XmlSetChildValueByName(String^ tag, String^ value, bool keepprevious)
	{
		for (int i = 0; i < children.Count; ++i)
		{
			if (children[i]->name->ToUpper() == tag->ToUpper())
			{
				if (!keepprevious) children[i]->value = value;
				return;
			}
		}
		children.Add( gcnew xml_node(this, tag, "", value) );
	}
	//

// ---------------------------------------------------
// RPC MARSHALLING CODES (DO NOT MODIFY!!!)
// ---------------------------------------------------


public:
	int DangerousGetSize() // Marshal helper
	{
		int mySize = 0;
		mySize = (name->Length + attributes->Length + value->Length) * sizeof(Char);
		mySize += 4 * sizeof(int);
		for (int i = 0; i < children.Count; ++i) mySize += children[i]->DangerousGetSize();
		return mySize;
	}

	byte* XML_Tree_Marshal() // The only usable marshal function from outside
	{
		int size = DangerousGetSize();
		byte* target = new byte[size + sizeof(int)];
		byte* tmp = target;
		XML_Tree_MarshalInt(tmp, size);
		XML_Tree_MarshalHelper(tmp);
		return target;
	}
	void XML_Tree_DeMarshal(byte* source)
	{
		this->~xml_node();
		int size = 4;
		size = XML_Tree_DeMarshalInt(source, size);
		XML_Tree_DeMarshalHelper( source, size );
	}

	void XML_Tree_MarshalHelper(byte*& target)
	{
		XML_Tree_MarshalString(target, name);
		XML_Tree_MarshalString(target, attributes);
		XML_Tree_MarshalString(target, value);
		XML_Tree_MarshalInt(target, children.Count);
		for (int i = 0; i < children.Count; ++i) 
			children[i]->XML_Tree_MarshalHelper(target);
	}
	void XML_Tree_DeMarshalHelper(byte*& source, int& size)
	{
		name		= XML_Tree_DeMarshalString( source, size );
		attributes	= XML_Tree_DeMarshalString( source, size );
		value		= XML_Tree_DeMarshalString( source, size );
		int childs	= XML_Tree_DeMarshalInt( source, size );

		for (int i = 0; i < childs; ++i)
		{
			children.Add( gcnew xml_node( this, "", "", "" ) );
			children[i]->XML_Tree_DeMarshalHelper( source, size );
		}
	}

private:

	void XML_Tree_MarshalInt(byte*& target, int i)
	{
		*((int*)target) = i;
		target += sizeof(int);
	}
	int XML_Tree_DeMarshalInt(byte*& source, int& size)
	{
		if (size - sizeof(int) < 0) return 0;
		int i = *((int*)source);
		source += sizeof(int);
		size -= sizeof(int);
		return i;
	}

	void XML_Tree_MarshalString(byte*& target, String^ s)
	{
		XML_Tree_MarshalInt(target, s->Length);
		for (int i = 0; i < s->Length; ++i)
		{
			Char c = s[i];
			*((Char*)target) = c;
			target += sizeof(Char);
		}
	}
	String^ XML_Tree_DeMarshalString(byte*& source, int& size)
	{
		String^ s = "";
		int length = XML_Tree_DeMarshalInt(source, size);
		if (size - length < 0)
		{
			size = 0;
			return s;
		}

		for (int i = 0; i < length; ++i)
		{
			Char c = *((Char*)source);
			s += c;
			source += sizeof(Char);
		}
		return s;
	}


public: // VARIABLES
	xml_node^ parent;
	String^ name;
	String^ attributes;
	String^ value;
	List<xml_node^> children;
};











public ref class xml_parser
{
public:

	static void write_xmltree_into_file(StreamWriter^ sw, xml_node^ node, String^ tabs)
	{
		if (sw == nullptr || node == nullptr) return;

		sw->Write(tabs + "<" + node->name);

		if (node->attributes != "") sw->Write(" " + node->attributes);

		sw->Write(">" + node->value);

		if (node->children.Count)   sw->WriteLine("");

		for (int i = 0; i < node->children.Count; i++) write_xmltree_into_file(sw, node->children[i], tabs + "\t");

		if (node->children.Count)   sw->Write(tabs);

		sw->WriteLine("</" + node->name + ">");
	}

	static void read_xml_into_xmlsubtree(StreamReader^ sr, xml_node^ node)
	{
		int j;
		String^ name;
		String^ attr;
		String^ value;
		String^ s = "";
		xml_node^ p;

		while (sr->Peek() >= 0)
		{
			value = "";
			j = read_tag(sr, name, attr);

			if (j == 0) // complete
			{
				node->children.Add(gcnew xml_node(node, name, attr, value));
			}
			else if (j == 1) // open
			{
				p = gcnew xml_node(node, name, attr, value);
				node->children.Add(p);
				node = p;
			}
			else if (j == 2)
			{
				if (node->parent != nullptr) node = node->parent; // safety construction
			}

			read_value(sr, value);

			if (value->Length) 
			{
				for (int ij = 0; ij < value->Length - 1; ++ij)
					if (value[ij] == '\\' && value[ij + 1] == 'n')
						value = value->Substring(0, ij) + "\n" + value->Substring(ij + 2);

				node->value += value;
			}
		}
	}

private:
	static int  interpret_tag(String^& xml_tag, String^& name, String^& attributes)
	{
		int res = 1;
		if (xml_tag[0] == '<') xml_tag = xml_tag->Substring(1);
		if (xml_tag[0] == '?') xml_tag = xml_tag->Substring(1);
		if (xml_tag[0] == '/') 
		{
			xml_tag = xml_tag->Substring(1);
			res = 2;
		}
		
		if (xml_tag[xml_tag->Length - 1] == '>') xml_tag = xml_tag->Substring(0, xml_tag->Length - 1);
		if (xml_tag[xml_tag->Length - 1] == '?' || xml_tag[xml_tag->Length - 1] == '/')
		{
			xml_tag = xml_tag->Substring(0, xml_tag->Length - 1);
			res = 0;
		}

		name = xml_tag;

		String^ tmp = "";
		for (int i = 0; i < name->Length; ++i)
		{
			if (name[i] != ' ')
				tmp += name[i];

			if (name[i] == ' ' && tmp->Length)
			{
				attributes = name->Substring(i + 1);
				name = tmp;
				break;
			}

		}
		return res;
	}

	static int  read_tag(StreamReader^ xml, String^& name, String^& attributes)
	{
		Char c;
		String^ tmp;
		name = attributes = "";

		while (xml->Peek() >= 0)
		{
			c = xml->Read();
			if (c != '\n' && c != '\r') tmp += c;
			if (c == '>') break;
		}

		return interpret_tag(tmp, name, attributes);
	}

	static void read_value(StreamReader^ xml, String^& value)
	{
		Char c = ' ';
		value = "";
		while (xml->Peek() >= 0)
		{
			c = xml->Read();
			if (c == '<') break;
			if (c == '\t') continue;
			if (c == '\n') continue;
			if (c == '\r') continue;
			value += c;
		}
	}
};

#endif

